/*
 * I2O SCSI Definitions
 * Copyright (C) 1999 Red Hat Software
 * Copyright (C) 2002-3 Wind River Systems
 * Copyright (C) 2003 Intel Corporation
 *  
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 */

#ifndef _I2O_SCSI_H
#define _I2O_SCSI_H

#if !defined(LINUX_VERSION_CODE)
#include <linux/version.h>
#endif

#define LinuxVersionCode(v, p, s) (((v)<<16)+((p)<<8)+(s))

#include <linux/types.h>
#include <linux/kdev_t.h>

#define I2O_SCSI_ID 15

#define I2O_SCSI_CAN_QUEUE 4
/*#define I2O_SCSI_CAN_QUEUE 16*/

#define I2O_SCSI_CMD_PER_LUN 3

#define I2O_ISCSI_MAX_LUNS 256

#define I2O_HBA_ADAPTER_RESET 0x85
#define I2O_HBA_BUS_QUIESCE   0x8b
#define I2O_HBA_BUS_RESET     0x87
#define I2O_HBA_BUS_SCAN      0x89

#define REPORT_LUNS 0xa0

/* Used for coordinating scsi commands with late replies
 * that get aborted, and maybe a bus reset as a result.
 */
#define ISCSI_CMD_ABORT_SENT        0x01
#define ISCSI_CMD_BUS_RESET_SENT    0x02
#define ISCSI_CMD_WAS_ABORTED       0x80


/* Borrowed this chunk from linux-iscsi:
 *     iSCSI driver for Linux
 *     Copyright (C) 2001 Cisco Systems, Inc.
 *     maintained by linux-iscsi@cisco.com
 */

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0) )
typedef unsigned long cpu_flags_t;
#else
typedef unsigned int cpu_flags_t;
#endif

/* for releasing the lock when we don't want it, but have it */
#if defined (SCSI_HAS_HOST_LOCK)
  /* Redhat Advanced Server is like 2.5, but uses a different name for the lock pointer.
   * At least they gave us a define to check for.
   */
# define PREFETCH_SPINLOCK
# define RELEASE_MIDLAYER_LOCK(host)  spin_unlock_irq((host)->lock)
# define REACQUIRE_MIDLAYER_LOCK(host) spin_lock_irq((host)->lock)
/* for getting the lock when we need it to call done(), but don't have it */
# define DECLARE_MIDLAYER_FLAGS cpu_flags_t midlayer_flags_
# define LOCK_MIDLAYER_LOCK(host) spin_lock_irqsave((host)->lock, midlayer_flags_);
# define UNLOCK_MIDLAYER_LOCK(host) spin_unlock_irqrestore((host)->lock, midlayer_flags_);

#elif ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,2) )
# define RELEASE_MIDLAYER_LOCK(host)  spin_unlock_irq((host)->host_lock)
# define REACQUIRE_MIDLAYER_LOCK(host) spin_lock_irq((host)->host_lock)
/* for getting the lock when we need it to call done(), but don't have it */
# define DECLARE_MIDLAYER_FLAGS cpu_flags_t midlayer_flags_
# define LOCK_MIDLAYER_LOCK(host) spin_lock_irqsave((host)->host_lock, midlayer_flags_);
# define UNLOCK_MIDLAYER_LOCK(host) spin_unlock_irqrestore((host)->host_lock, midlayer_flags_);

#else
# define PREFETCH_SPINLOCK  spin_lock_prefetch(&io_request_lock)
# define RELEASE_MIDLAYER_LOCK(host)  spin_unlock_irq(&io_request_lock)
# define REACQUIRE_MIDLAYER_LOCK(host) spin_lock_irq(&io_request_lock)
/* for getting the lock when we need it to call done(), but don't have it */
# define DECLARE_MIDLAYER_FLAGS cpu_flags_t midlayer_flags_
# define LOCK_MIDLAYER_LOCK(host) spin_lock_irqsave(&io_request_lock, midlayer_flags_);
# define UNLOCK_MIDLAYER_LOCK(host) spin_unlock_irqrestore(&io_request_lock, midlayer_flags_);
#endif


#ifndef RH90
extern int i2o_scsi_detect(Scsi_Host_Template *);
extern const char *i2o_scsi_info(struct Scsi_Host *);
extern int i2o_scsi_queuecommand(Scsi_Cmnd *, void (*done)(Scsi_Cmnd *));
extern int i2o_scsi_bios_param(Disk *, kdev_t, int *);
extern void i2o_scsi_setup(char *str, int *ints);
extern int i2o_scsi_release(struct Scsi_Host *host);

extern int is1000_eh_abort( Scsi_Cmnd *SCpnt );
/*extern int is1000_eh_host_reset( Scsi_Cmnd *sc );*/
extern int is1000_eh_bus_reset( Scsi_Cmnd *sc );
/*extern int is1000_eh_device_reset( Scsi_Cmnd *sc );*/
/*extern int is1000_eh_strategy_handler(struct Scsi_Host *); */
#else
static int i2o_scsi_detect(Scsi_Host_Template *);
static const char *i2o_scsi_info(struct Scsi_Host *);
static int i2o_scsi_queuecommand(Scsi_Cmnd *, void (*done)(Scsi_Cmnd *));
static int i2o_scsi_bios_param(Disk *, kdev_t, int *);
extern void i2o_scsi_setup(char *str, int *ints);
static int i2o_scsi_release(struct Scsi_Host *host);

static int is1000_eh_abort( Scsi_Cmnd *SCpnt );
/*static int is1000_eh_host_reset( Scsi_Cmnd *sc );*/
static int is1000_eh_bus_reset( Scsi_Cmnd *sc );
/*static int is1000_eh_device_reset( Scsi_Cmnd *sc );*/
/*static int is1000_eh_strategy_handler(struct Scsi_Host *);*/
#endif

#if (defined(CONFIG_HIGHIO) || defined(AS)) && defined(IS1000_HIGHMEMIO)
#ifdef RH90
#define I2OSCSI {                                 \
    next: NULL,               \
        proc_name:         "is1000_scsi",         \
        name:              "I2O SCSI Layer",      \
        highmem_io:        1,                     \
        detect:            i2o_scsi_detect,       \
        release:           i2o_scsi_release,      \
        info:              i2o_scsi_info,         \
        command:           NULL,                  \
        queuecommand:      i2o_scsi_queuecommand, \
        bios_param:        i2o_scsi_bios_param,   \
        can_queue:         I2O_SCSI_CAN_QUEUE,    \
        this_id:           I2O_SCSI_ID,           \
        sg_tablesize:      8,                     \
        cmd_per_lun:       I2O_SCSI_CMD_PER_LUN,  \
        unchecked_isa_dma: 0,                     \
        use_clustering:    ENABLE_CLUSTERING,     \
        abort:             NULL,                  \
        reset:             NULL,        \
        use_new_eh_code:   1,                     \
        eh_abort_handler:  is1000_eh_abort,       \
        eh_bus_reset_handler: is1000_eh_bus_reset,\
        eh_device_reset_handler: NULL, \
        eh_host_reset_handler: NULL \
        }
#else
#define I2OSCSI {                                 \
    next: NULL,               \
        proc_name:         "is1000_scsi",         \
        name:              "I2O SCSI Layer",      \
        can_dma_32:        1,                     \
        detect:            i2o_scsi_detect,       \
        release:           i2o_scsi_release,      \
        info:              i2o_scsi_info,         \
        command:           NULL,                  \
        queuecommand:      i2o_scsi_queuecommand, \
        bios_param:        i2o_scsi_bios_param,   \
        can_queue:         I2O_SCSI_CAN_QUEUE,    \
        this_id:           I2O_SCSI_ID,           \
        sg_tablesize:      8,                     \
        cmd_per_lun:       I2O_SCSI_CMD_PER_LUN,  \
        unchecked_isa_dma: 0,                     \
        use_clustering:    ENABLE_CLUSTERING,     \
        abort:             NULL,                  \
        reset:             NULL,        \
        use_new_eh_code:   1,                     \
        eh_abort_handler:  is1000_eh_abort,       \
        eh_bus_reset_handler: is1000_eh_bus_reset,\
        eh_device_reset_handler: NULL, \
        eh_host_reset_handler: NULL \
        }
#endif
#else
#define I2OSCSI {                                 \
    next: NULL,               \
        proc_name:         "is1000_scsi",         \
        name:              "I2O SCSI Layer",      \
        detect:            i2o_scsi_detect,       \
        release:           i2o_scsi_release,      \
        info:              i2o_scsi_info,         \
        command:           NULL,                  \
        queuecommand:      i2o_scsi_queuecommand, \
        bios_param:        i2o_scsi_bios_param,   \
        can_queue:         I2O_SCSI_CAN_QUEUE,    \
        this_id:           I2O_SCSI_ID,           \
        sg_tablesize:      8,                     \
        cmd_per_lun:       I2O_SCSI_CMD_PER_LUN,  \
        unchecked_isa_dma: 0,                     \
        use_clustering:    ENABLE_CLUSTERING,     \
        abort:             NULL,                  \
        reset:             NULL,        \
        use_new_eh_code:   1,                     \
        eh_abort_handler:  is1000_eh_abort,       \
        eh_bus_reset_handler: is1000_eh_bus_reset,\
        eh_device_reset_handler: NULL, \
        eh_host_reset_handler: NULL \
        }
#endif

#endif

/*        eh_device_reset_handler: is1000_eh_device_reset, \ */
/*        eh_host_reset_handler: is1000_eh_host_reset \ */
