/*
 * is1000ioctl.h
 *
 * Intel PRO/1000 T IP Storage Adapter driver interface definitions
 * Copyright (C) 2002-3 Intel Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

#ifndef __IS1000IOCTL_H__
#define __IS1000IOCTL_H__

#include "is1000common.h"

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define CHARDEVICENAME "is1000"


enum IOCTL_STATUS_TYPE
{
    IOCTL_STATUS_SUCCESS,
    IOCTL_STATUS_PENDING,
    IOCTL_STATUS_FAILED,
    IOCTL_STATUS_MORE_MEMORY,
    IOCTL_STATUS_RESET,
    IOCTL_STATUS_DHCP_CONFIG_INCOMPLETE,
    IOCTL_STATUS_DHCP_CONFIG_FAILED,
    IOCTL_STATUS_REQUEST_SIZE,
    IOCTL_STATUS_FW_NO_REPONSE
};

enum ISCSI_TARGET_RECOVERY_DATA_TYPE
{
  ISCSI_RECOVERY_DATA_SET,
  ISCSI_RECOVERY_DATA_GET
};

enum ISCSI_ADAPTER_CONFIG_STATUS
{
  ISCSI_ADAPTER_CONFIG_SUCCESS,
  ISCSI_ADAPTER_CONFIG_FAILED,
  ISCSI_ADAPTER_CONFIG_DHCP_FAILED
};

enum ISCSI_RESET_TARGET_STATUS
{
  ISCSI_RESET_TARGET_SUCCESS,
  ISCSI_RESET_TARGET_FAILED
};
  
/////////////////////////////////////////////////////////////
// Data Structure definitions
//

typedef struct
{
    u32                          adapterNumber;
    IS_MAC_ADDRESS               AdapterMACAddress;
} ISCSI_ADAPTER, *PISCSI_ADAPTER;

typedef struct
{
	//in
	IS_MAC_ADDRESS               MACAddr;
	u32                          IPAddress;
	u16                          port;
	IS_CHAP_SETTINGS             chap;

	//out
	u32                          numTargets;
	u32                          status;
} ISCSI_GET_STORAGE_TARGET_COUNT, *PISCSI_GET_STORAGE_TARGET_COUNT;

typedef struct
{
	//in
	IS_MAC_ADDRESS               MACAddr;
	u32			     IPAddress;
	u16			     port;
	u32                          targetIndex;
	
	//out
	u32                          status;
	char                         iscsi_nodename[IS_MAX_NODENAME_STRING];
	u8		             numAddresses;
	IS_TARGET_ADDRESS	     target_addresses[IS_MAX_ADDRESSES_PER_TARGET];
} ISCSI_GET_STORAGE_TARGET, *PISCSI_GET_STORAGE_TARGET;

typedef struct
{
    //in
	IS_MAC_ADDRESS               MACAddr;
    u32                          dwStatus;
    IS_STORAGE_TARGET            sTarget;
    //out
} ISCSI_STORAGE_TARGET_INFO, *PISCSI_STORAGE_TARGET_INFO;

typedef struct
{
    //in
    IS_MAC_ADDRESS               MACAddr;
    //out
    u32                          diagValue;
} ISCSI_DIAGNOSTICS, *PISCSI_DIAGNOSTICS;

typedef struct
{
    //in
    IS_MAC_ADDRESS               MACAddr;
    u16                          iSeconds;
    u32                          dwStatus;
    
} ISCSI_BLINK_LED, *PISCSI_BLINK_LED;

typedef struct
{
    //out
    ISCSI_ADAPTER                iSCSIAdapters[IS_MAX_BOARDS];
    u32                          nNumberAdapters;
    u32                          dwStatus;
} ISCSI_ENUMERATE_BOARDS, *PISCSI_ENUMERATE_BOARDS;

typedef struct
{
    //in
    u32                          dwStatus;
    IS_MAC_ADDRESS               MACAddr;
	IS_SESSION_HBA_STATS         stats;
} ISCSI_ADAPTER_STATISTICS, *PISCSI_ADAPTER_STATISTICS;

typedef struct
{ 
    //in
    IS_MAC_ADDRESS                     MACAddr;
  
    //out
    IS_ADAPTER_PROPS				props;
} ISCSI_GET_ADAPTER_PROPERTIES, *PISCSI_GET_ADAPTER_PROPERTIES;

typedef struct
{ 
    //in
    IS_MAC_ADDRESS                     MACAddr;
    IS_ADAPTER_CONF_PROPS	       props;
    //out 
    u32                          dwStatus;
} ISCSI_SET_ADAPTER_PROPERTIES, *PISCSI_SET_ADAPTER_PROPERTIES;

typedef struct
{
    //In
    IS_MAC_ADDRESS                     MACAddr;
    u32                          dwStatus;       // request status
    u32                          timeOut;        // Command Timeout in Milliseconds.
    u32                          bufferSize;     // Size of the return data buffer
    //Out
    u8                           dataBuffer[IS_EEPROM_DATA_SIZE];// Beginning of the data buffer.
                                                    // we can't handle a pointer so array is 
                                                    // physically part of the request.  EEPROM
                                                    // Data is 128 bytes in length.
} ISCSI_GET_EEPROM_DATA, *PISCSI_GET_EEPROM_DATA;

typedef struct{
  //In
  IS_MAC_ADDRESS             MACAddr;
  u32                        targetId; 
  //In/Out
  u32                        dwStatus;  // IOCTL_TARGET_RECOVERY_DATA_TYPE
  //Out
  u32                        timeOut;
  u32                        maxRetries;
  u32                        currentRetries;
} ISCSI_TARGET_RECOVERY_DATA, *PISCSI_TARGET_RECOVERY_DATA;

typedef struct
{
   IS_MAC_ADDRESS             MACAddr;
   IS_FLASH_UPDATE_PACKET     packet;
} ISCSI_FLASH_UPDATE_PACKET, *PISCSI_FLASH_UPDATE_PACKET;

typedef struct
{
	//in
	IS_MAC_ADDRESS       MACAddr;
	u32                  IPAddress;

	//out
	u32                  dwStatus;
} ISCSI_PING, *PISCSI_PING;

typedef struct
{
	//out
	u32            version;
} ISCSI_GET_DRIVER_VERSION, *PISCSI_GET_DRIVER_VERSION;

typedef struct
{
	//in
	IS_MAC_ADDRESS        MACAddr;

	//out
	u32             numTargets;
} ISCSI_GET_CONFIGURED_TARGET_COUNT, *PISCSI_GET_CONFIGURED_TARGET_COUNT;

typedef struct
{
	//in
	IS_MAC_ADDRESS        MACAddr;
	u32             targetIndex;

	//out
	IS_STORAGE_TARGET  target;
} ISCSI_GET_CONFIGURED_TARGET, *PISCSI_GET_CONFIGURED_TARGET;

typedef struct
{
	IS_MAC_ADDRESS        MACAddr;
        u32                   targetId;
        char                  targetname[IS_MAX_NODENAME_STRING];
        IS_TARGET_ADDRESS     address;
        //out
        int                   status;
} ISCSI_TARGET_RESET, *PISCSI_TARGET_RESET;

typedef struct
{
	//in
	IS_MAC_ADDRESS       MACAddr;
	u32                  buffsize;
	//in/out
	u8                   buffer[IS_MAX_LOOPBACK_BUFFER_SIZE];
} ISCSI_LOOPBACK, *PISCSI_LOOPBACK;

typedef struct
{
	//in
	IS_MAC_ADDRESS       MACAddr;
	u32                  targetId;

	//out
	IS_SESSION_STATS     stats;
} ISCSI_GET_TARGET_STATISTICS, *PISCSI_GET_TARGET_STATISTICS;
	
typedef struct
{
	//in
	IS_MAC_ADDRESS       MACAddr;
	u32                  targetId;

	//out
	u32                  sessionState;
} ISCSI_TARGET_ERROR_STATUS, *PISCSI_TARGET_ERROR_STATUS;
	
/* The u32 sessionState above is built as follows:
 * class << 24 | det << 16 | ISCSI_ISM_ERROR_LOGIN_FAIL 
 */
#define ISCSI_ISM_ERROR_LOGIN_FAIL		0x81	/* Login auth fail */
#define ISCSI_ISM_ERROR_AUTH_FAIL		0x82	/* Init rejects tgt */
#define ISCSI_ISM_ERROR_CONNECT_FAIL		0x83	/* TCP connection fail */
#define ISCSI_ISM_ERROR_MAX_REDIRECT		0x84	/* Max login redirects */
#define ISCSI_ISM_ERROR_LOGIN_TCP_FAIL		0x85	/* TCP fail during login */
#define ISCSI_ISM_ERROR_LOGIN_BAD_STATE		0x86	/* Illegal state trans */
#define ISCSI_ISM_ERROR_LOGIN_BAD_PDU		0x87	/* Unexpected PDU */
#define ISCSI_ISM_ERROR_LOGIN_BAD_USER		0x88	/* Unknown user in auth */
#define ISCSI_ISM_ERROR_LOGIN_CANT_AGREE	0x89	/* Negotiation failure */
#define ISCSI_ISM_ERROR_LOGIN_BAD_REDIRECT	0x8A	/* Bad login redirect */

#define IS1000_MAGIC_NUMBER                       'i'

#define ISCSI_IOCTL_GET_DRIVER_VERSION          _IOR (IS1000_MAGIC_NUMBER, 0,  sizeof(ISCSI_GET_DRIVER_VERSION))

//Adapters
#define ISCSI_IOCTL_ENUMERATE_ADAPTERS          _IOR (IS1000_MAGIC_NUMBER, 1,  sizeof(ISCSI_ENUMERATE_BOARDS))
#define ISCSI_IOCTL_GET_ADAPTER_PROPERTIES      _IOR (IS1000_MAGIC_NUMBER, 2,  sizeof(ISCSI_GET_ADAPTER_PROPERTIES))
#define ISCSI_IOCTL_SET_ADAPTER_PROPERTIES      _IOW (IS1000_MAGIC_NUMBER, 3,  sizeof(ISCSI_SET_ADAPTER_PROPERTIES))
#define ISCSI_IOCTL_GET_DIAGNOSTICS             _IOR (IS1000_MAGIC_NUMBER, 4,  sizeof(ISCSI_DIAGNOSTICS))
#define ISCSI_IOCTL_GET_STATISTICS              _IOR (IS1000_MAGIC_NUMBER, 5,  sizeof(ISCSI_ADAPTER_STATISTICS))
#define ISCSI_IOCTL_GET_STORAGE_TARGET_COUNT    _IOR (IS1000_MAGIC_NUMBER, 6,  sizeof(ISCSI_GET_STORAGE_TARGET_COUNT))
#define ISCSI_IOCTL_GET_STORAGE_TARGET          _IOR (IS1000_MAGIC_NUMBER, 7,  sizeof(ISCSI_GET_STORAGE_TARGET))
#define ISCSI_IOCTL_BLINK_LED                   _IOW (IS1000_MAGIC_NUMBER, 8,  sizeof(ISCSI_BLINK_LED))

//Targets
#define ISCSI_IOCTL_ADD_TARGET                  _IOW (IS1000_MAGIC_NUMBER, 9, sizeof(ISCSI_STORAGE_TARGET_INFO))
//#define ISCSI_IOCTL_REMOVE_TARGET               _IOW (IS1000_MAGIC_NUMBER, 10, sizeof(ISCSI_REMOVE_TARGET))
#define ISCSI_IOCTL_REMOVE_TARGET               _IOW (IS1000_MAGIC_NUMBER, 10, sizeof(ISCSI_STORAGE_TARGET_INFO))
#define ISCSI_IOCTL_GET_TARGET_PROPERTIES       _IOR (IS1000_MAGIC_NUMBER, 11, sizeof(ISCSI_STORAGE_TARGET_INFO))
#define ISCSI_IOCTL_STORAGE_TARGET_STATUS       _IOR (IS1000_MAGIC_NUMBER, 12, sizeof(ISCSI_STORAGE_TARGET_INFO))
#define ISCSI_IOCTL_TARGET_INITIATE_LOGIN       _IOW (IS1000_MAGIC_NUMBER, 13, sizeof(ISCSI_STORAGE_TARGET_INFO))
//#define ISCSI_IOCTL_TARGET_DISABLE              _IOW (IS1000_MAGIC_NUMBER, 14, sizeof(ISCSI_TARGET_DISABLE))
#define ISCSI_IOCTL_TARGET_DISABLE              _IOW (IS1000_MAGIC_NUMBER, 14, sizeof(ISCSI_STORAGE_TARGET_INFO))
#define ISCSI_IOCTL_TARGET_RECOVERY_DATA        _IOW (IS1000_MAGIC_NUMBER, 15, sizeof(ISCSI_TARGET_RECOVERY_DATA))
#define ISCSI_IOCTL_SET_TARGET_CHAP             _IOW (IS1000_MAGIC_NUMBER, 16, sizeof(ISCSI_STORAGE_TARGET_INFO))
#define ISCSI_IOCTL_RESET_TARGET                _IOW (IS1000_MAGIC_NUMBER, 17, sizeof(ISCSI_TARGET_RESET))
#define ISCSI_IOCTL_GET_TARGET_STATISTICS       _IOW (IS1000_MAGIC_NUMBER, 18, sizeof(ISCSI_GET_TARGET_STATISTICS))
#define ISCSI_IOCTL_TARGET_ERROR_STATUS         _IOW (IS1000_MAGIC_NUMBER, 19, sizeof(ISCSI_TARGET_ERROR_STATUS))

//Utility
#define ISCSI_IOCTL_PING                        _IOW (IS1000_MAGIC_NUMBER, 20, sizeof(ISCSI_PING))
#define ISCSI_IOCTL_GET_EEPROM_INFO             _IOR (IS1000_MAGIC_NUMBER, 21, sizeof(ISCSI_GET_EEPROM_DATA))
#define ISCSI_IOCTL_LOOPBACK                    _IOW (IS1000_MAGIC_NUMBER, 22, sizeof(ISCSI_LOOPBACK))
#define ISCSI_IOCTL_UPDATE_FLASH		_IOW (IS1000_MAGIC_NUMBER, 99, sizeof(ISCSI_FLASH_UPDATE_PACKET))
//#define ISCSI_IOCTL_RETRIEVE_FLASH		_IOW (IS1000_MAGIC_NUMBER, 100,sizeof(ISCSI_FLASH_UPDATE_PACKET))

#endif //__iS1000IOCTL_H__
  
