/*******************************************************************************
 * Copyright (C) 2002-2003 Intel Corporation                                   *
 *                                                                             *
 * This program is free software; you can redistribute it and/or               *
 * modify it under the terms of the GNU General Public License                 *
 * as published by the Free Software Foundation; either version 2              *
 * of the License, or (at your option) any later version.                      *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with this program; if not, write to the Free Software                 *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA  *
 ******************************************************************************/
/*******************************************************************************
 *                                                                             *
 * File Name:    is1000common.h                                                *
 *                                                                             *
 * Abstract:     This file represents the common data structures and defines   *
 *               for the Intel PRO/1000 T IP Storage adapter library.          *
 *                                                                             *
 * Environment:  This file is intended to be used indirectly by applications   *
 *               in order to have data types compatable with the library       *
 *                                                                             *
 * Author:       Thomas C. Lea                                                 *
 *                                                                             *
 * Origination:  April 21, 2003                                                *
 ******************************************************************************/

#ifndef __IS1000COMMON_H__
#define __IS1000COMMON_H__

#ifdef __cplusplus
extern "C"
{
#endif //__cplusplus

    typedef signed long         IS_RESULT;

#ifndef __KERNEL__
    typedef unsigned long long  u64;
    typedef unsigned long       u32;
    typedef unsigned short      u16;
    typedef unsigned char       u8;
#endif //__KERNEL__

    /** IPV4 */
    typedef u32 IP_TYPE;    

#define IS_MAX_CONNECTIONS                30
#define IS_MAX_BOARDS                     16
#define IS_MAX_USERNAME_STRING            256
#define IS_MAX_PASSWORD_STRING            256
#define IS_MAX_MANUFACTURER_STRING        64
#define IS_MAX_MODEL_STRING               64
#define IS_MAX_HW_VERSION_STRING          32
#define IS_MAX_ASIC_VERSION_STRING        32
#define IS_MAX_FW_VERSION_STRING          32
#define IS_MAX_ROM_VERSION_STRING         32
#define IS_MAX_OS_NAME_STRING             32
#define IS_MAX_NODENAME_STRING            224
#define IS_MAX_LUNS                       256
#define IS_EEPROM_DATA_SIZE               128
#define IS_TARGETID_INVALID               255
#define IS_MAX_LOOPBACK_BUFFER_SIZE       255

/** firmware limit */
#define IS_MAX_ADDRESSES_PER_TARGET       20

/** Derived from the firmware's use of microseconds stored in a u32 */
#define IS_MAX_TARGET_TIMEOUT_SECONDS     4294

/** Two minutes is a reasonable default timeout.
    zero will cause spinning on a lost connection */
#define IS_DEFAULT_TARGET_TIMEOUT_SECONDS 120

/** Normal frame size */
#define ISCSI_FRAMESIZE_1500              1500

/** Jumbo frame size */
#define ISCSI_FRAMESIZE_9000              9000

/** Flash packet update buffer size */
#define FLASH_PACKET_SIZE                 8192

/** Nothing wrong */
#define DIAGNOSTIC_OK                  0x00

/** Some targets failed */
#define DIAGNOSTIC_TARGETS_FAILED      0x01

/** Some tgts connecting */
#define DIAGNOSTIC_TARGETS_CONNECTING  0x02

/** DHCP neg failed */
#define DIAGNOSTIC_DHCP_FAILED         0x08

/** Link down */
#define DIAGNOSTIC_LINK_FAILURE        0x10

/** Firmware RESET in progress */
#define DIAGNOSTIC_RESET_IN_PROGRESS   0x20

/** Card not responding */
#define DIAGNOSTIC_CARD_NOT_RESPONDING 0x40


//////////////////////////////////////////////////////////////
//enums
//////////////////////////////////////////////////////////////

/** The current status of a target connection/session */
    enum CONNECTION_STATUS
    {
        CONNECTION_OK,
        CONNECTION_PENDING,
        CONNECTION_FAILED_LOGIN,
        CONNECTION_CONNECTION_DROPPED,
        CONNECTION_NEED_SCSI_ENUMERATION,
        CONNECTION_NO_MATCH,
        CONNECTION_DISCONNECTED,
        CONNECTION_DISABLED,
        CONNECTION_DISRUPTED,
        CONNECTION_NOT_ENABLED
    };

/** Status of enumeration operations in progress with the driver */
    enum ENUMERATION_STATUS_TYPE
    {
        ENUMERATION_STATUS_SUCCESS,
        ENUMERATION_STATUS_PENDING,
        ENUMERATION_STATUS_FAILED,
        ENUMERATION_STATUS_TIMEDOUT,
        ENUMERATION_STATUS_POLL,
        ENUMERATION_STATUS_ABORT,
        ENUMERATION_STATUS_MORE_MEMORY
    };

/** Status of a blink operation on the adapter */    
    enum BLINK_STATUS
    {
        BLINK_SUCCESS,
        BLINK_PENDING,
        BLINK_FAILED,
        BLINK_START,
        BLINK_STOP
    };

/** Status of the eeprom on the adapter */    
    enum EEPROM_STATUS_TYPE
    {
        EEPROM_STATUS_SUCCESS,
        EEPROM_MORE_MEMORY,
        EEPROM_FW_NOT_RESPONDING,
    };

/** Status of configured targets */    
    typedef enum
    {
        FIRST_AND_INVALID_TARGET_STATUS,
        TARGET_STATUS_SYNC_REQUIRED,
        TARGET_STATUS_LOGIN_REQUIRED,
        TARGET_STATUS_LOGIN_IN_PROGRESS,
        TARGET_STATUS_LOGIN_FAILURE,
        TARGET_STATUS_CONNECTED,
        TARGET_STATUS_RECOVERING,
        TARGET_STATUS_CONNECTION_LOST,
        LAST_AND_INVALID_TARGET_STATUS
    } TARGET_STATUS;

/** Valid CRC configurations */
    typedef enum
    {
        CRC_MODE_NONE,
        CRC_MODE_HEADER,
        CRC_MODE_DATA,
        CRC_MODE_BOTH
    } IS_CRC_MODE;

//////////////////////////////////////////////////////////////
//structures
//////////////////////////////////////////////////////////////

/** Standard MAC address structure */
    typedef struct
    {
        u32                          first;
        u32                          second;
    } IS_MAC_ADDRESS, *PIS_MAC_ADDRESS;

/** Structure used to identify the address of a target */    
    typedef struct
    {
        u32                     ip;
        u16                     port;
        u16                     portalgroup;
    } IS_TARGET_ADDRESS, *PIS_TARGET_ADDRESS;

/** Used to identify the system's SCSI id of a target's LUNs */
    typedef struct
    {
        u8                      host;
        u8                      id;
        u8                      lun;
    } IS_SCSI_ID, *PIS_SCSI_ID;

/** Common CHAP configuration structure */    
    typedef struct
    {
        u16                     useCHAP;
        char                    CHAPusername[IS_MAX_USERNAME_STRING];
        char                    CHAPpassword[IS_MAX_PASSWORD_STRING];
        u16                     useInboundCHAP;
        char                    inboundCHAPusername[IS_MAX_USERNAME_STRING];
        char                    inboundCHAPpassword[IS_MAX_PASSWORD_STRING];
    } IS_CHAP_SETTINGS, *PIS_CHAP_SETTINGS;

/** Common structure to hold information about a target */    
    typedef struct
    {
        IS_TARGET_ADDRESS       address;
        TARGET_STATUS           status;
        u32                     timeout;
        u32                     retries;
        u8                      targetId;
        IS_CRC_MODE             crcMode;
        char                    iscsi_nodename[IS_MAX_NODENAME_STRING];
        IS_CHAP_SETTINGS        chap;
    } IS_STORAGE_TARGET, *PIS_STORAGE_TARGET;

/** Structure to hold all configurable options for an adapter */
    typedef struct
    {
        char                    iscsi_nodename[IS_MAX_NODENAME_STRING];
        IP_TYPE                 AdapterIPAddress;
        IP_TYPE                 AdapterIPMask;
        IP_TYPE                 AdapterGateway;
        u16                     UseDhcp;
        u32                     FrameSize;
        u32                     DefaultRetries;
        u32                     DefaultTimeout;
        u16                     AutoNegEnabled;
                      // --- these fields valid only if AutoNegEnabled is true
        u16                     Disallow10Half;   // |
        u16                     Disallow10Full;   // |
        u16                     Disallow100Half;  // |
        u16                     Disallow100Full;  // |
        u16                     Disallow1000Full; // |
                      // --- these fields valid only if AutoNegEnabled is false
        u32                     ForcedLinkSpeed;  // | may be 10, 100, or 1000
        u32                     ForcedFullDuplex; // | 1 = full
        u16                     FlowControlEnabled;
        u32                     ReservedBits;
        IS_MAC_ADDRESS          LAA_Address;
    } IS_ADAPTER_CONF_PROPS, *PIS_ADAPTER_CONF_PROPS;

/** Structure that holds all properties (configurable and nonconfigurable)
    for an adapter. */
    typedef struct
    {
        u32                     ProcessorID;
        u32                     EthernetDeviceID;
        u32                     EthernetRevision;
        u32                     BoardRevisionNumber;
        u32                     CPLDRevisionNumber;
        u32                     ProcessorRevision;
        u32                     AdapterFirmwareVersion;
        u32                     AdapterRTOSVersion;
        u16                     HasLink;
        u16                     LinkSpeed;
        u16                     IsFullDuplex;
        u16                     TransmissionPaused;
        u16                     EthernetInitialized;
        u32                     MaxTargets;
        u32                     Status;
        IP_TYPE                 IPAddress;
        IS_ADAPTER_CONF_PROPS   s;
    } IS_ADAPTER_PROPS, *PIS_ADAPTER_PROPS;

/* iSCSI per session extended stats, more detailed counters than above. Note
 * that there is some duplication here, but no stat is actually counted more
 * than once since the extended stats are only collated when they are read via
 * params access.
 */

/** Per connection stats */
    typedef struct
    {
        u64   bytesSent;
        u64   bytesReceived;
        u64   pduSent;
        u64   pduReceived;
    } IS_CONNECTION_STATS, *PIS_CONNECTION_STATS;

/** Per session stats - note that since we have only one connection per session
 * these will be similar to the connection stats
 */
    typedef struct
    {
        u64   bytesSent;
        u64   bytesReceived;
        u64   pduSent;
        u64   pduReceived;
        u64   digestErrors;
        u64   connTimeouts;
        u64   formatErrors;
    } IS_EXT_SESSION_STATS, *PIS_EXT_SESSION_STATS;

/* Login stats, HBA wide */
    typedef struct
    {
        u32   accepts;
        u32   failResp;
        u32   redirect;
        u32   authFailResp;
        u32   authFailTgt;
        u32   negotiationFail;
        u32   logoutGood;
        u32   logoutBad;
        u32   loginFails;
    } IS_LOGIN_STATS, *PIS_LOGIN_STATS;

/** HBA wide stats */
    typedef struct
    {
        u32   digestErrors;
        u32   connTimeout;
        u32   formatErrors;
        u32   sessFail;
    } IS_INITIATOR_STATS, *PIS_INITIATOR_STATS;

/** Per NIC port stats */
    typedef struct
    {
        u32   bytesTx;
        u32   bytesRx;
        u32   pduTx;
        u32   pduRx;
    } IS_NIC_STATS, *PIS_NIC_STATS;

/** Convenient grouping of all extended stats */
    typedef struct
    {
        IS_CONNECTION_STATS  connStats;
        IS_EXT_SESSION_STATS sessStats;
        IS_LOGIN_STATS       loginStats;
        IS_INITIATOR_STATS   initStats;
        IS_NIC_STATS         nicStats;
    } IS_SESSION_STATS, *PIS_SESSION_STATS;

/** Convenient grouping of HBA wide extended stats */
    typedef struct
    {
        IS_LOGIN_STATS       loginStats;
        IS_INITIATOR_STATS   initStats;
        IS_NIC_STATS         nicStats;
    } IS_SESSION_HBA_STATS, *PIS_SESSION_HBA_STATS;

    typedef struct
    {
        u32                     frag_number;
        u32                     frag_size;
        u32                     max_frag;
        u32                     total_size;
        u32                     software_id;
        u8                      buffer[FLASH_PACKET_SIZE];
    } IS_FLASH_UPDATE_PACKET, *PIS_FLASH_UPDATE_PACKET;

#ifdef __cplusplus
}
#endif // __cplusplus
#endif // __IS1000COMMON_H__
