/*******************************************************************************
 * Copyright (C) 2002-2003 Intel Corporation                                   *
 *                                                                             *
 * This program is free software; you can redistribute it and/or               *
 * modify it under the terms of the GNU General Public License                 *
 * as published by the Free Software Foundation; either version 2              *
 * of the License, or (at your option) any later version.                      *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with this program; if not, write to the Free Software                 *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA  *
 ******************************************************************************/
/*******************************************************************************
 *                                                                             *
 * File Name:    is1000events.h                                                *
 *                                                                             *
 * Abstract:     This file describes the driver->isdaemon event interface.     *
 *                                                                             *
 * Environment:  Used by the driver and the isdaemon.                          *
 *                                                                             *
 * Author:       Thomas C. Lea                                                 *
 *                                                                             *
 * Origination:  April 21, 2003                                                *
 ******************************************************************************/

#ifndef __IS1000EVENTS_H__
#define __IS1000EVENTS_H__

#include "is1000common.h"

/** These structures are returned by the driver to the daemon when the blocking call
    to read() returns. */
typedef enum
{
    ISEVENT_TARGET_PERMANENTLY_REDIRECTED,
    ISEVENT_TARGET_CONNECTION_FAILED,
    ISEVENT_TARGET_CONNECTION_FAULTED,
    ISEVENT_TARGET_CONNECTION_NORMAL,
    ISEVENT_HOST_ADAPTER_LINK_DOWN,
    ISEVENT_HOST_ADAPTER_LINK_UP,
    ISEVENT_ERROR,
    ISEVENT_HOST_ADAPTER_DHCP_LEASE_NEW,
    ISEVENT_HOST_ADAPTER_DHCP_LEASE_EXPIRE,
    ISEVENT_HOST_ADAPTER_DHCP_BIND_FAIL
} ISEVENT_TYPE;

/** These error values are stored in the eventDetail when the event
    is ISEVENT_ERROR */
typedef enum
{
/** 0x00 No Error, Place holder */
    ISCSI_ISM_ERROR_NO_ERROR,

/** 0x01 NOP timeout */
    ISCSI_ISM_ERROR_NOP_FAIL,

/** 0x02 TCP error */
    ISCSI_ISM_ERROR_TCP_FAIL,

/** 0x03 Target wants tear down */
    ISCSI_ISM_ERROR_TARGET_FAIL,

/** 0x04 Header CRC failure */
    ISCSI_ISM_ERROR_HEAD_CRC_FAIL,

/** 0x05 Data CRC failure */
    ISCSI_ISM_ERROR_DATA_CRC_FAIL,

/** 0x06 Protocol violation */
    ISCSI_ISM_ERROR_PROTOCOL_FAIL,

/** 0x07 Reject PDU received */
    ISCSI_ISM_ERROR_REJECT_PDU,

/** 0x08 Initiator low resource */
    ISCSI_ISM_ERROR_NO_RESOURCE,

/** 0x09 Max login redirects */
    ISCSI_ISM_ERROR_MAX_REDIRECT_EVENT,

/** 0x0A Bad ITT from target */
    ISCSI_ISM_ERROR_BAD_ITT,

/** 0x0B DMA failure, not good */
    ISCSI_ISM_ERROR_DMA_FAIL,

/** 0x0C Logout response */
    ISCSI_ISM_ERROR_LOGOUT,

/** 0x0D Host request Bus reset */
    ISCSI_ISM_ERROR_BUS_RESET,

/** 0x0E Target timeout out */
    ISCSI_ISM_ERROR_TIMEOUT
} ISEVENT_DETAIL_TYPE;

/** This is the common event structure that is returned from the blocking
    read() to the driver. */
typedef struct
{
    ISEVENT_TYPE        event;
    ISEVENT_DETAIL_TYPE eventDetail;
    IS_MAC_ADDRESS      MACAddr;
    u32                 host;
    char                targetname[IS_MAX_NODENAME_STRING];
    u32                 targetId;
} IS1000_EVENT, *PIS1000_EVENT;

#endif //__IS1000EVENTS_H__
