/*******************************************************************************
* Copyright (C) 2002-2003 Intel Corporation                                    *
*                                                                              *
* This program is free software; you can redistribute it and/or                *
* modify it under the terms of the GNU General Public License                  *
* as published by the Free Software Foundation; either version 2               *
* of the License, or (at your option) any later version.                       *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA   *
*******************************************************************************/
/*******************************************************************************
*                                                                              *
* File Name:    adapter.cpp                                                    *
*                                                                              *
* Abstract:     This file contains the implementation for the adapter related  *
*               functions used by isiconfig.                                   *
*                                                                              *
* Environment:  This file should only be used by the isiconfig internals.      *
*                                                                              *
* Author:       Thomas C. Lea                                                  *
*                                                                              *
* Origination:  April 21, 2003                                                 *
*                                                                              *
* Modification History                                                         *
* Date     Initials   Description                                              *
* 04/21/03 TCL        Genesis                                                  *
*******************************************************************************/

// Include Files
#include <iostream>
#include <popt.h>
#include "is1000api.h"
#include "utility.h"

using namespace std;

// Global External definitions
extern int adapter;
extern int adapternumber;
extern int update;
extern char *ip;
extern int retries;
extern int timeout;

// Local Private definitions
static int blink = 0;
static int diag = 0;
static char *ping = NULL;
static char *loopback = NULL;
static int eeprom = 0;
static char *sendflash = NULL;
static int details = 0;
static char *initiatorname = NULL;
static int usedhcp = 0;
static char *gateway = NULL;
static char *subnetmask = NULL;
static int mtu = 1500;                            // 1500 is default
static char *laa = NULL;
static int manualnegotiation = 0;
static int disallow10half = 0;
static int disallow10full = 0;
static int disallow100half = 0;
static int disallow100full = 0;
static int disallow1000full = 0;
static int forceduplex = 0;
static int forcespeed = -1;
static int disableflowcontrol = 0;

struct poptOption adapterOptions[] =
{
// main commands
    {
        "update",
        'u',
        POPT_ARG_NONE,
        &update,
        0,
        NULL,
        NULL
    },
    {
        "blink",
        'b',
        POPT_ARG_INT,
        &blink,
        0,
        NULL,
        NULL
    },
    {
        "diag",
        'd',
        POPT_ARG_NONE,
        &diag,
        0,
        NULL,
        NULL
    },
    {
        "ping",
        'p',
        POPT_ARG_STRING,
        &ping,
        0,
        NULL,
        NULL
    },
    {
        "loopback",
        '\0',
        POPT_ARG_STRING,
        &loopback,
        0,
        NULL,
        NULL
    },
    {
        "eeprom",
        'e',
        POPT_ARG_NONE,
        &eeprom,
        0,
        NULL,
        NULL
    },
    {
        "sendflash",
        's',
        POPT_ARG_STRING,
        &sendflash,
        0,
        NULL,
        NULL
    },
    {
        "details",
        '\0',
        POPT_ARG_NONE,
        &details,
        0,
        NULL,
        NULL
    },

    //options
    {
        "initiatorname",
        '\0',
        POPT_ARG_STRING,
        &initiatorname,
        0,
        NULL,
        NULL
    },
    {
        "usedhcp",
        '\0',
        POPT_ARG_NONE,
        &usedhcp,
        0,
        NULL,
        NULL
    },
    {
        "gateway",
        '\0',
        POPT_ARG_STRING,
        &gateway,
        0,
        NULL,
        NULL
    },
    {
        "subnetmask",
        '\0',
        POPT_ARG_STRING,
        &subnetmask,
        0,
        NULL,
        NULL
    },
    {
        "ip",
        '\0',
        POPT_ARG_STRING,
        &ip,
        0,
        NULL,
        NULL
    },
    {
        "mtu",
        '\0',
        POPT_ARG_INT,
        &mtu,
        0,
        NULL,
        NULL
    },
    {
        "laa",
        '\0',
        POPT_ARG_STRING,
        &laa,
        0,
        NULL,
        NULL
    },
    {
        "manualnegotiation",
        '\0',
        POPT_ARG_NONE,
        &manualnegotiation,
        0,
        NULL,
        NULL
    },
    {
        "disallow10half",
        '\0',
        POPT_ARG_NONE,
        &disallow10half,
        0,
        NULL,
        NULL
    },
    {
        "disallow10full",
        '\0',
        POPT_ARG_NONE,
        &disallow10full,
        0,
        NULL,
        NULL
    },
    {
        "disallow100half",
        '\0',
        POPT_ARG_NONE,
        &disallow100half,
        0,
        NULL,
        NULL
    },
    {
        "disallow100full",
        '\0',
        POPT_ARG_NONE,
        &disallow100full,
        0,
        NULL,
        NULL
    },
    {
        "disallow1000full",
        '\0',
        POPT_ARG_NONE,
        &disallow1000full,
        0,
        NULL,
        NULL
    },
    {
        "forceduplex",
        '\0',
        POPT_ARG_NONE,
        &forceduplex,
        0,
        NULL,
        NULL
    },
    {
        "forcespeed",
        '\0',
        POPT_ARG_INT,
        &forcespeed,
        0,
        NULL,
        NULL
    },
    {
        "disableflowcontrol",
        '\0',
        POPT_ARG_NONE,
        &disableflowcontrol,
        0,
        NULL,
        NULL
    },

    POPT_TABLEEND
}; //struct poptOption adapterOptions[]

/** ****************************************************************************
 * Procedure:   showAdapterUsage
 *
 * Description: This routine displays the usage/help for the adapter options
 *
 ******************************************************************************/
void showAdapterUsage(void)
{
    cout << "[ADAPTER OPTIONS]" << endl;
    cout << "--adapternumber=ADAPTERNUMBER\tThe adapter for the operation (default 0)." << endl;
    cout << "-u, --update [ADAPTER CONFIG]\tAlter an adapter." << endl;
    cout << "-b, --blink=SECONDS\t\tBlink the LED on an adapter." << endl;
    cout << "-d, --diag\t\t\tDisplay current diagnostic status of an adapter." << endl;
    cout << "-p, --ping=IPADDRESS\t\tPing an IP address through an adapter." << endl;
    cout << "-e, --eeprom\t\t\tDump the EEPROM memory on an adapter." << endl;
    cout << "-s, --sendflash=FILENAME\tFlash the contents of FILENAME to an adapter's firmware." << endl;
    cout << "--details\t\t\tDisplay details about an adapter." << endl;
    cout << "--loopback=TEXT\t\t\tInitiate a loopback test with TEXT." << endl;
    cout << "\t\t\t\tWARNING: loopback testing will disrupt traffic!" << endl;

    cout << "[ADAPTER CONFIG]" << endl;
    cout << "--initiatorname=NAME\t\tSet this adapter's initiator name to NAME." << endl;
    cout << "--usedhcp\t\t\tUse DHCP to configure this adapter's network settings." << endl;
    cout << "--gateway=IPADDRESS\t\t[non-DHCP] The IP address of the adapter's gateway." << endl;
    cout << "--subnetmask=MASK\t\t[non-DHCP] The subnet mask of the adapter's network." << endl;
    cout << "--ip=IPADDRESS\t\t\t[non-DHCP] The adapter's IP address." << endl;
    cout << "--mtu=BYTES\t\t\tThe adapter's MTU [default=1500]." << endl;
    cout << "--timeout=SECONDS\t\tThe default connection timeout for this adapter." << endl;
    cout << "--retries=NUM\t\t\tThe default number of times to attempt to recover a connection." << endl;
    cout << "--laa=MACADDRESS\t\tAn alternative MAC address to use for this adapter." << endl;
    cout << "--manualnegotiation\t\tDisable automatic link negotiation." << endl;
    cout << "--disallow10half\t\tDisallow 10Mbps half-duplex from automatic link negotiation." << endl;
    cout << "--disallow10full\t\tDisallow 10Mbps full-duplex from automatic link negotiation." << endl;
    cout << "--disallow100half\t\tDisallow 100Mbps half-duplex from automatic link negotiation." << endl;
    cout << "--disallow100full\t\tDisallow 100Mbps full-duplex from automatic link negotiation." << endl;
    cout << "--disallow1000full\t\tDisallow 1Gbps full-duplex from automatic link negotiation." << endl;
    cout << "--forceduplex\t\t\tForce full-duplex link negotiation." << endl;
    cout << "--forcespeed=MBPS\t\tForce link speed negotiation (10 or 100)." << endl;
    cout << "--disableflowcontrol\t\tDisable Flow Control." << endl;
    cout << endl;
}

/** ****************************************************************************
 * Procedure:   performAdapter
 *
 * Description: This routine executes the selected adapter function
 *
 ******************************************************************************/
void performAdapter(void)
{
    IS_MAC_ADDRESS adapter_mac;
    u32 numadapters;
    IS_ADAPTER_PROPS_WITH_DESC currentProperties;
    IS_RESULT result = GetAdapterCount(&numadapters, 0);
    if(result != IS_SUCCESS)
    {
        cerr << "GetAdapterCount failed: " << RESULTSTRING(result) << endl;
        exit(result);
    }

    if(numadapters == 0)
    {
        cerr << "No adapters found!" << endl;
        exit(1);
    }

    if((result = getAdapterMac(adapternumber, adapter_mac)) != IS_SUCCESS)
    {
        cerr << "Unable to get MAC address for adapter #" <<
            adapternumber << endl;
        exit(result);
    }

    memset(&currentProperties, 0, sizeof(IS_ADAPTER_PROPS_WITH_DESC));
    if((result = GetAdapterProperties(&adapter_mac,
                                      &currentProperties,
                                      0)) != IS_SUCCESS)
    {
        cerr << "Unable to retrieve current adapter properties: " <<
            RESULTSTRING(result) << endl;
        exit(result);
    }

    if(update)
    {
        IP_TYPE ip_val;
        IP_TYPE gateway_val;
        IP_TYPE subnetmask_val;
        IS_MAC_ADDRESS laa_val;
        char autogeneratedInitiatorName[IS_MAX_NODENAME_STRING];

        memset(&ip_val, 0 , sizeof(IP_TYPE));
        memset(&gateway_val, 0 , sizeof(IP_TYPE));
        memset(&subnetmask_val, 0 , sizeof(IP_TYPE));
        memset(&laa_val, 0, sizeof(IS_MAC_ADDRESS));

        //validate args
        if(adapternumber < 0 || adapternumber >= (int)numadapters)
        {
            cerr << "Invalid adapter number " << adapternumber <<
                ".  Valid range is 0-" << (numadapters-1) << endl;
            exit(1);
        }

        // if no initiatorname was provided and there isnt already
        // one configured, generate one.
        if((initiatorname == NULL || strlen(initiatorname) == 0))
        {
            if(strlen(currentProperties.props.s.iscsi_nodename) == 0)
            {
                if(GenerateUniqueNodename(autogeneratedInitiatorName,
                                          IS_MAX_NODENAME_STRING,
                                          0) == IS_SUCCESS)
                {
                    initiatorname = autogeneratedInitiatorName;
                    cout << "No initiator name was provided.  Using autogenerated name:" <<
                        initiatorname << endl;

                }
                else
                {
                    cerr << "No initiator name was provided.  Error autogenerating name." << endl;
                    exit(1);
                }
            }
            else   //none was provided, but one was already set for the adapter so use it
            {
                initiatorname = currentProperties.props.s.iscsi_nodename;
            }
        }

        if(strlen(initiatorname) >= IS_MAX_NODENAME_STRING)
        {
            cerr << "Invalid initiator name." << endl;
            exit(1);
        }
        if(usedhcp == 0 &&
            (ip == NULL ||
             strlen(ip) == 0 ||
             !stringtoip(ip, ip_val) ||
             ip_val == 0))
        {
            cerr << "Invalid IP address." << endl;
            exit(1);
        }
        if(usedhcp == 0 && ip_val == 0x7f000001)  // loopback
        {
            cerr << "Invalid IP address." << endl;
            exit(1);
        }
        if(usedhcp == 0 &&
            (subnetmask == NULL ||
             strlen(subnetmask) == 0 ||
             !stringtoip(subnetmask, subnetmask_val) ||
             subnetmask_val == 0))
        {
            cerr << "Invalid subnet mask." << endl;
            exit(1);
        }
        if(usedhcp == 0 &&
            (gateway != NULL &&
             strlen(gateway) > 0 &&
             !stringtoip(gateway, gateway_val)))
        {
            cerr << "Invalid gateway." << endl;
            exit(1);
        }
        if(usedhcp == 0 &&
            (gateway_val != 0 &&
            (gateway_val & subnetmask_val) != (ip_val & subnetmask_val)))
        {
            cerr << "Invalid network configuration.  IP and gateway must be on the same network." << endl;
            exit(1);
        }
        if(forcespeed != -1 &&
            (forcespeed != 10 &&
             forcespeed != 100))
        {
            cerr << "Invalid forcespeed (must be 10 or 100)." << endl;
            exit(1);
        }
        else if(forcespeed == -1)
        {
            forcespeed = 0;
        }
        if(laa != NULL &&
            (strlen(laa) != 12 ||
             !stringtomac(laa, laa_val)))
        {
            cerr << "Invalid locally administered MAC address (laa).  laa should be 12 hex characters." << endl;
            exit(1);
        }
        if(mtu != 1500 && mtu != 9000)
        {
            cerr << "Invalid MTU.  Must be either 1500 (standard) or 9000 (jumbo)." << endl;
            exit(1);
        }

        cout << "Updating adapter #" << adapternumber << endl;

        IS_ADAPTER_CONF_PROPS props;
        memset(&props, 0, sizeof(IS_ADAPTER_CONF_PROPS));
        strncpy(props.iscsi_nodename, initiatorname, IS_MAX_NODENAME_STRING);
        props.AdapterIPAddress = ip_val;
        props.AdapterIPMask = subnetmask_val;
        props.AdapterGateway = gateway_val;
        props.UseDhcp = usedhcp;
        props.FrameSize = mtu;
        props.DefaultRetries = retries;
        props.DefaultTimeout = timeout;
        props.AutoNegEnabled = (manualnegotiation==0)?1:0;
        props.Disallow10Half = disallow10half;
        props.Disallow10Full = disallow10full;
        props.Disallow100Full = disallow100full;
        props.Disallow100Half = disallow100half;
        props.Disallow1000Full = disallow1000full;
        props.ForcedLinkSpeed = forcespeed;
        props.ForcedFullDuplex = forceduplex;
        props.FlowControlEnabled = (disableflowcontrol==0)?1:0;
        memcpy(&props.LAA_Address, &laa_val, sizeof(IS_MAC_ADDRESS));

        IS_RESULT result = SetAdapterProperties(&adapter_mac, &props, 0);
        if(result != IS_SUCCESS)
        {
            cerr << "SetAdapterProperties failed: " <<
                RESULTSTRING(result) << endl;
        }

    }
    else if(blink > 0)
    {
        if(blink > 120)
        {
            cerr << "Invalid blink interval.  Valid range is 1 - 120 seconds." << endl;
            exit(1);
        }
        
        cout << "Blinking adapter #" << adapternumber << " for " << blink <<
            " seconds." << endl;
        if((result = SetAdapterBlink(&adapter_mac,
                    (u32)blink, 0)) != IS_SUCCESS)
        {
            cerr << "SetAdapterBlink failed: " << RESULTSTRING(result) << endl;
            exit(result);
        }
    }
    else if(diag)
    {
        u32 diag_value = 0;
        if((result = GetAdapterDiagnostics(&adapter_mac,
                                           &diag_value,
                                           0)) != IS_SUCCESS)
        {
            cerr << "GetAdapterDiagnostics failed: " <<
                RESULTSTRING(result) << endl;
            exit(result);
        }

        cout << "Adapter #" << adapternumber << "'s diagnostic status = ";

        if(diag_value == 0)
            cout << "OK";
        else
        {
            bool needComma = false;

            if(diag_value & DIAGNOSTIC_TARGETS_FAILED)
            {
                cout << "TARGETS FAILED";
                needComma = true;
            }

            if(diag_value & DIAGNOSTIC_TARGETS_CONNECTING)
            {
                if(needComma) cout << ", "; else needComma = true;
                cout << "TARGETS CONNECTING";
            }

            if(diag_value & DIAGNOSTIC_DHCP_FAILED)
            {
                if(needComma) cout << ", "; else needComma = true;
                cout << "DHCP FAILED";
            }

            if(diag_value & DIAGNOSTIC_LINK_FAILURE)
            {
                if(needComma) cout << ", "; else needComma = true;
                cout << "LINK FAILURE";
            }

            if(diag_value & DIAGNOSTIC_RESET_IN_PROGRESS)
            {
                if(needComma) cout << ", "; else needComma = true;
                cout << "RESET IN PROGRESS";
            }

            if(diag_value & DIAGNOSTIC_CARD_NOT_RESPONDING)
            {
                if(needComma) cout << ", "; else needComma = true;
                cout << "CARD NOT RESPONDING";
            }

        }
        cout << endl;

    }
    else if(ping)
    {
        IP_TYPE ping_val;
        u32 ping_result = 0;

        if(strlen(ping) == 0 || !stringtoip(ping, ping_val))
        {
            cerr << "Invalid IP address for ping." << endl;
            exit(1);
        }

        if((result = Ping(&adapter_mac,
                          ping_val,
                          &ping_result,
                          0)) != IS_SUCCESS)
        {
            cerr << "Ping failed: " << RESULTSTRING(result) << endl;
            exit(result);
        }

        if(ping_result == 0)                      // success
            cout << "Ping succeeded. " << endl;
        else
            cout << "Ping failed. " << endl;

    }
    else if(loopback)
    {
        u32 textlength = strlen(loopback);
        u8  *resultbuffer = NULL;
        if(textlength > IS_MAX_LOOPBACK_BUFFER_SIZE)
        {
            cerr << "Loopback test message cannot exceed " <<
                IS_MAX_LOOPBACK_BUFFER_SIZE << " characters." << endl;
            exit(1);
        }
        else
        {
            resultbuffer = new u8[textlength+1];
        }

        cout << "Sending:" << endl << loopback << endl << endl;

        if((result = LoopbackTest(&adapter_mac,
                                  (u8*)loopback,
                                  resultbuffer,
                                  textlength,
                                  0)) != IS_SUCCESS)
        {
            cerr << "LoopbackTest failed: " << RESULTSTRING(result) << endl;
            delete [] resultbuffer;
            exit(result);
        }

        resultbuffer[textlength] = '\0';
        cout << "Received:" << endl << resultbuffer << endl << endl;

        if(memcmp(loopback, resultbuffer, textlength) == 0)
            cout << "Loopback succeeded. " << endl;
        else
            cout << "Loopback failed. " << endl;

        delete [] resultbuffer;

    }
    else if(eeprom)
    {
        u8 buffer[IS_EEPROM_DATA_SIZE];
        u32 buffer_size = IS_EEPROM_DATA_SIZE;

        if((result = GetEEPromData(&adapter_mac,
                                   buffer,
                                   &buffer_size,
                                   0)) != IS_SUCCESS)
        {
            cerr << "Unable to retrieve EEPROM data: " <<
                RESULTSTRING(result) << endl;
            exit(result);
        }

        for(u32 i = 0; i < buffer_size; i++)
            putchar(buffer[i]);
    }
    else if(sendflash)
    {
        if((result = UpdateFlash(sendflash, 0)) == IS_SUCCESS)
        {
            cout << "Firmware update complete.  Please restart the is1000 service." << endl;
        }
        else
        {
            cerr << "Firmware update failed: " << RESULTSTRING(result) << endl;
            cerr << "Please try again or use the failsafe method (see documentation)." <<
                endl << endl;
            exit(result);
        }
    }
    else if(details)
    {
        showDetails(adapternumber, adapter_mac);
    }
    else
    {
        cerr << "Invalid options." << endl;
        exit(1);
    }
}
