/*******************************************************************************
* Copyright (C) 2002-2003 Intel Corporation                                    *
*                                                                              *
* This program is free software; you can redistribute it and/or                *
* modify it under the terms of the GNU General Public License                  *
* as published by the Free Software Foundation; either version 2               *
* of the License, or (at your option) any later version.                       *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA   *
*******************************************************************************/
/*******************************************************************************
*                                                                              *
* File Name:    target.cpp                                                     *
*                                                                              *
* Abstract:     This module contains isiconfig's target specific functionality *
*                                                                              *
* Environment:  This file should only be used by isiconfig internals.          *
*                                                                              *
* Author:       Thomas C. Lea                                                  *
*                                                                              *
* Origination:  April 21, 2003                                                 *
*                                                                              *
* Modification History                                                         *
* Date     Initials   Description                                              *
* 04/21/03 TCL        Genesis                                                  *
*******************************************************************************/

// Include Files
#include <iostream>
#include "is1000api.h"
#include "utility.h"
#include <popt.h>
#include <unistd.h>

using namespace std;

// Global external definitions
extern int update;
extern char *ip;
extern int adapternumber;
extern int timeout;
extern int retries;

// Local private definitions
static int add = 0;
static int removetarget = 0;
static char *query = NULL;
static char *targetname = NULL;
static int portnum = 3260;  //the default
static char *portstr = NULL;
static int usechap = 0;
static char *username = NULL;
static char *password = NULL;
static int useinboundchap = 0;
static char *inusername = NULL;
static char *inpassword = NULL;
static int reset = 0;
static int statistics = 0;
static char *crcmodestr = NULL;
IS_CRC_MODE crcmode = CRC_MODE_NONE;

struct poptOption targetOptions[] =
{
    {
        "add",
        'c',
        POPT_ARG_NONE,
        &add,
        0,
        NULL,
        NULL
    },
    {
        "update",
        'u',
        POPT_ARG_NONE,
        &update,
        0,
        NULL,
        NULL
    },
    {
        "query",
        'q',
        POPT_ARG_STRING,
        &query,
        0,
        NULL,
        NULL
    },
    {
        "remove",
        'r',
        POPT_ARG_NONE,
        &removetarget,
        0,
        NULL,
        NULL
    },
    {
        "statistics",
        '\0',
        POPT_ARG_NONE,
        &statistics,
        0,
        NULL,
        NULL
    },

    {
        "targetname",
        '\0',
        POPT_ARG_STRING,
        &targetname,
        0,
        NULL,
        NULL
    },
    {
        "ip",
        '\0',
        POPT_ARG_STRING,
        &ip,
        0,
        NULL,
        NULL
    },
    {
        "port",
        '\0',
        POPT_ARG_STRING,
        &portstr,
        0,
        NULL,
        NULL
    },
    {
        "usechap",
        '\0',
        POPT_ARG_NONE,
        &usechap,
        0,
        NULL,
        NULL
    },
    {
        "username",
        '\0',
        POPT_ARG_STRING,
        &username,
        0,
        NULL,
        NULL
    },
    {
        "password",
        '\0',
        POPT_ARG_STRING,
        &password,
        0,
        NULL,
        NULL
    },
    {
        "useinboundchap",
        '\0',
        POPT_ARG_NONE,
        &useinboundchap,
        0,
        NULL,
        NULL
    },
    {
        "inusername",
        '\0',
        POPT_ARG_STRING,
        &inusername,
        0,
        NULL,
        NULL
    },
    {
        "inpassword",
        '\0',
        POPT_ARG_STRING,
        &inpassword,
        0,
        NULL,
        NULL
    },
    {
        "reset",
        'x',
        POPT_ARG_NONE,
        &reset,
        0,
        NULL,
        NULL
    },
    {
        "crcmode",
        '\0',
        POPT_ARG_STRING,
        &crcmodestr,
        0,
        NULL,
        NULL
    },
    POPT_TABLEEND
}; //struct poptOption targetOptions[]

/** ****************************************************************************
 * Procedure:   showTargetUsage
 *
 * Description: This routine displays the usage/help for the target options
 *
 ******************************************************************************/
void showTargetUsage()
{
    cout << "[TARGET OPTIONS]" << endl;
    cout << "--adapternumber=ADAPTERNUMBER\tThe adapter for the operation (default 0)." << endl;
    cout << "-c, --add [TARGET CONFIG]\tAdd a persistent target and initiate a connection." << endl;
    cout << "-u, --update [TARGET CONFIG]\tAlter an existing target's properties." << endl;
    cout << "-q, --query=IPADDRESS[:PORT]\tQuery a host for a list of available targets." << endl;
    cout << "-r, --remove [TARGET CONFIG]\tDisconnect an existing target and remove from the configuration." << endl;
    cout << "-x, --reset [TARGET CONFIG]\tReset a target's connection." << endl;
    cout << "    --statistics [TARGET CONFIG]\tDisplay a target's statistics." << endl;

    cout << endl;
    cout << "[TARGET CONFIG]" << endl;
    cout << "--targetname=NAME\t\tThe iSCSI nodename of the target." << endl;
    cout << "--ip=IPADDRESS\t\t\tIP Address of the Target." << endl;
    cout << "--port=PORT\t\t\tPort of the Target (default 3260)." << endl;
    cout << "--usechap\t\t\tUse CHAP for authentication to the target." << endl;
    cout << "--username=USER\t\t\tUsername for CHAP authentication." << endl;
    cout << "--password=PASS\t\t\tPassword for CHAP authentication." << endl;
    cout << "--useinboundchap\t\tUse CHAP for inbound authentication from the target." << endl;
    cout << "--inusername=USER\t\tUsername for inbound CHAP authentication." << endl;
    cout << "--inpassword=PASS\t\tPassword for inbound CHAP authentication." << endl;
    cout << "--timeout=SECONDS\t\tConnection timeout in seconds." << endl;
    cout << "--retries=NUMRETRIES\t\tNumber of times to attempt to recover a connection." << endl;
    cout << "--crcmode=MODE\t\t\tEnables CRC for the target.  MODE can be one of:" << endl;
    cout << "\t\t\t\t\t\"NONE\", \"HEADER\", \"DATA\", or \"BOTH\"" << endl;

}

/** ****************************************************************************
 * Procedure:   performTarget
 *
 * Description: This routine executes the selected target function
 *
 ******************************************************************************/
void performTarget(void)
{
    IS_CHAP_SETTINGS chap;
    IS_MAC_ADDRESS adapter_mac;
    IP_TYPE ip_val;
    u32 numadapters;
    IS_RESULT result = GetAdapterCount(&numadapters, 0);
    if(result != IS_SUCCESS)
    {
        cerr << "GetAdapterCount failed: " << RESULTSTRING(result) << endl;
        exit(result);
    }

    if(numadapters == 0)
    {
        cerr << "No adapters found!" << endl;
        exit(1);
    }

    if((result = getAdapterMac(adapternumber, adapter_mac)) != IS_SUCCESS)
    {
        cerr << "Unable to get MAC address for adapter #" <<
            adapternumber << endl;
        exit(result);
    }

    memset(&ip_val, 0, sizeof(IP_TYPE));
    if(ip != NULL && !stringtoip(ip, ip_val))
    {
        cerr << "Invalid IP address" << endl;
        exit(1);
    }

    if(portstr != NULL)
    {
        char *endptr = NULL;
        portnum = strtoul(portstr, &endptr, 10);
        if(*endptr != '\0' || portnum <= 0 || portnum > 0xffff)
        {
            cerr << "Invalid port number: " << portstr << endl;
            exit(1);
        }
    }

    memset(&chap, 0, sizeof(IS_CHAP_SETTINGS));
    if(usechap)
    {
        if(username == NULL || password == NULL)
        {
            cerr << "Invalid CHAP username or password." << endl;
            exit(1);
        }

        if(strlen(username) == 0 || strlen(password) == 0)
        {
            cerr << "Invalid CHAP username or password." << endl;
            exit(1);
        }

        chap.useCHAP = 1;
        strncpy(chap.CHAPusername, username, IS_MAX_USERNAME_STRING);
        chap.CHAPusername[IS_MAX_USERNAME_STRING-1] = '\0';
        strncpy(chap.CHAPpassword, password, IS_MAX_PASSWORD_STRING);
        chap.CHAPpassword[IS_MAX_PASSWORD_STRING-1] = '\0';
    }

    if(useinboundchap)
    {
        if(usechap == 0)
        {
            cerr << "You cannot specify inbound CHAP without outbound CHAP."
                << endl;
            exit(1);
        }

        if(inusername == NULL || inpassword == NULL)
        {
            cerr << "Invalid inbound CHAP username or password." << endl;
            exit(1);
        }

        if(strlen(inusername) == 0 || strlen(inpassword) == 0)
        {
            cerr << "Invalid inbound CHAP username or password." << endl;
            exit(1);
        }

        if(!strcmp(password, inpassword))
        {
            cerr << "Inbound and outbound CHAP passwords cannot be the same.\n" << endl;
            exit(1);
        }

        chap.useInboundCHAP = 1;
        strncpy(chap.inboundCHAPusername, inusername, IS_MAX_USERNAME_STRING);
        chap.inboundCHAPusername[IS_MAX_USERNAME_STRING-1] = '\0';
        strncpy(chap.inboundCHAPpassword, inpassword, IS_MAX_PASSWORD_STRING);
        chap.inboundCHAPpassword[IS_MAX_PASSWORD_STRING-1] = '\0';
    }

    if(crcmodestr != NULL)
    {
        if(!strcasecmp(crcmodestr, "none"))
            crcmode = CRC_MODE_NONE;
        else if(!strcasecmp(crcmodestr, "header"))
            crcmode = CRC_MODE_HEADER;
        else if(!strcasecmp(crcmodestr, "data"))
            crcmode = CRC_MODE_DATA;
        else if(!strcasecmp(crcmodestr, "both"))
            crcmode = CRC_MODE_BOTH;
        else
        {
            cerr << "Invalid CRC mode." << endl;
            exit(1);
        }
    }

    if(add || update)
    {
        IS_RESULT result;

        if(targetname == NULL)
        {
            cerr << "You must specify a targetname." << endl;
            exit(1);
        }

        if(ip == NULL)
        {
            cerr << "You must specify a valid IP address." << endl;
            exit(1);
        }

        if(add)
        {
            cout << "Adding target to adapter #" << adapternumber
                << ", MAC = " << mactostring(adapter_mac) << endl;
            if((result = AddTarget(&adapter_mac,
                                   ip_val,
                                   portnum,
                                   &chap,
                                   targetname,
                                   crcmode,
                                   0)) != IS_SUCCESS)
            {
                cerr << "Add Target failed: " << RESULTSTRING(result) << endl;
                exit(result);
            }

            updateDeviceHive();
        }
        else                                      //update
        {
            if((result = SetTargetSecurity(&adapter_mac,
                                           ip_val,
                                           portnum,
                                           targetname,
                                           &chap,
                                           0)) != IS_SUCCESS)
            {
                cerr << "SetTargetSecurity failed: " <<
                    RESULTSTRING(result) << endl;
                exit(result);
            }

            if(crcmodestr != NULL) //already been parsed and validated
            {
                if((result = SetTargetCRC(&adapter_mac,
                                          ip_val,
                                          portnum,
                                          targetname,
                                          crcmode,
                                          0)) != IS_SUCCESS)
                {
                    cerr << "SetTargetCRC failed: " <<
                        RESULTSTRING(result) << endl;
                    exit(result);
                }
            }
        }

        //if either is different than default, we need to set
        if(timeout != IS_DEFAULT_TARGET_TIMEOUT_SECONDS || retries != 0)
        {
            if((result = SetTargetParameters(&adapter_mac,
                                             ip_val,
                                             portnum,
                                             targetname,
                                             timeout,
                                             retries,
                                             0)) != IS_SUCCESS)
            {
                cerr << "SetTargetParameters failed: " <<
                    RESULTSTRING(result) << endl;
                exit(result);
            }
        }
    }
    else if(query)
    {
        int query_str_len = strlen(query);
        if(query_str_len < 7)  //minimum 0.0.0.0
        {
            cerr << "Please specify an IP address and port of the portal to query (for example, 127.0.0.1:3260)." << endl;
            exit(1);
        }

        char *colon = strchr(query, ':');
        if(colon != NULL)
        {
            char *endptr = NULL;
            *colon = '\0';
            colon++;
            portnum = strtoul(colon, &endptr, 10);
            if(*endptr != '\0' || portnum <= 0 || portnum > 0xffff)
            {
                cerr << "Invalid portal port number: " << colon << endl;
                exit(1);
            }
        }

        IP_TYPE ipnum;
        if(!stringtoip(query, ipnum))
        {
            cerr << "Invalid portal IP address." << endl;
            exit(1);
        }

        cout << "Querying Portal at " <<
            iptostring(ipnum) << ":" << portnum << " for targets..." << endl;
        u32 numtargets = 0;
        if((result = GetQueryTargetCount(&adapter_mac,
                                         ipnum,
                                         portnum,
                                         &chap,
                                         &numtargets,
                                         0)) != IS_SUCCESS)
        {
            cerr << "GetQueryTargetCount failed: " <<
                RESULTSTRING(result) << endl;
            exit(result);
        }

        cout << "Found " << numtargets << " target";
        if(numtargets != 1)
            cout << "s";
        cout << ":" << endl;

        for(u32 i = 0; i < numtargets; i++)
        {
            u32 numaddresses = IS_MAX_ADDRESSES_PER_TARGET;
            IS_TARGET_ADDRESS addresses[numaddresses];
            char nodename[IS_MAX_NODENAME_STRING];

            if((result = EnumQueryTargets(&adapter_mac,
                ipnum,
                portnum,
                &chap,
                i,
                nodename,
                IS_MAX_NODENAME_STRING,
                addresses,
                &numaddresses, 0)) != IS_SUCCESS)
            {
                cerr << "EnumQueryTargets failed: " <<
                    RESULTSTRING(result) << endl;
                exit(result);
            }
            else
            {
                cout << endl << "Name: " << nodename << endl;

                for(u32 j = 0; j < numaddresses; j++)
                {
                    cout << "Address: " << iptostring(addresses[j].ip) <<
                        ":" << addresses[j].port <<
                        "," << addresses[j].portalgroup << endl;
                }
            }
        }

    }
    else if(removetarget)
    {
        if(ip == NULL)
        {
            cerr << "You must specify a valid IP address." << endl;
            exit(1);
        }

        if(targetname == NULL)
        {
            cerr << "You must specify a valid target name." << endl;
            exit(1);
        }

        if(isTargetMounted(&adapter_mac, ip_val, portnum, targetname))
        {
            cerr << "You must unmount devices related to this target before you remove it." << endl;
            exit(1);
        }

        if((result = RemoveTarget(&adapter_mac,
                                  ip_val,
                                  portnum,
                                  targetname,
                                  0)) != IS_SUCCESS)
        {
            cerr << "RemoveTarget failed: " << RESULTSTRING(result) << endl;
            exit(result);
        }

        updateDeviceHive();
    }
    else if(reset)
    {
        if(ip == NULL)
        {
            cerr << "You must specify a valid IP address." << endl;
            exit(1);
        }

        if((result = ResetTargetConnection(&adapter_mac,
                                           ip_val,
                                           portnum,
                                           targetname,
                                           0)) != IS_SUCCESS)
        {
            cerr << "ResetTargetConnection failed: " <<
                RESULTSTRING(result) << endl;
            exit(result);
        }

        sleep(3); // wait for the target to return before sending inquiry

        updateDeviceHive();
    }
    else if(statistics)
    {
        IS_SESSION_STATS statistics;
        memset(&statistics, 0, sizeof(statistics));
        if((result = GetTargetStatistics(&adapter_mac,
                                         ip_val,
                                         portnum,
                                         targetname,
                                         &statistics,
                                         0)) != IS_SUCCESS)
        {
            cerr << "Unable to retrieve statistics.  Error = " <<
                result << endl;
        }
        else
        {
            dumpConnectionStats(statistics.connStats);
            dumpExtSessionStats(statistics.sessStats);
            dumpLoginStats(statistics.loginStats);
            dumpInitiatorStats(statistics.initStats);
            dumpNicStats(statistics.nicStats);
        }
    }
    else
    {
        cerr << "Invalid options (target)" << endl;
        exit(1);
    }
} //performTarget(void)
