/*******************************************************************************
* Copyright (C) 2002-2003 Intel Corporation                                    *
*                                                                              *
* This program is free software; you can redistribute it and/or                *
* modify it under the terms of the GNU General Public License                  *
* as published by the Free Software Foundation; either version 2               *
* of the License, or (at your option) any later version.                       *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA   *
*******************************************************************************/
/*******************************************************************************
*                                                                              *
* File Name:    is1000api.h                                                    *
*                                                                              *
* Abstract:     This file describes the Intel PRO/1000 T IP Storage Adapter    *
*               management library.                                            *
*                                                                              *
* Environment:  This file is intended to be used by any user application that  *
*               needs to manage or monitor the adapters.                       *
*                                                                              *
* Author:       Thomas C. Lea                                                  *
*                                                                              *
* Origination:  April 21, 2003                                                 *
*******************************************************************************/

#ifndef __IS1000API_H__
#define __IS1000API_H__

// Include Files
#include "is1000common.h"

#ifdef __cplusplus
extern "C"
{
#endif

    //version is of form A.B.c.d  where A is (Zama2 = 1, TL1 = 2, TL2 = 3).
    //B is project release number. c.d is minor release version
    static const u32 IS_VERSION =            0x03000001;

#define IS_RESULT_CODE_COUNT                    28

    /** No error occurred. */
    static const IS_RESULT IS_SUCCESS =            0;

    /** The API library was unable to find an Intel(R) PRO/1000 T IP adapter
        driver. */
    static const IS_RESULT IS_NO_DRIVER =          -1;

    /** The adapter was configured incorrectly. */
    static const IS_RESULT IS_BAD_CONFIGURATION =  -2;
    
    /** An index value that was not legal was passed to a function. */
    static const IS_RESULT IS_INDEX_OUT_OF_RANGE = -3;

    /** The MAC address passed to the function did not refer to a valid adapter. */
    static const IS_RESULT IS_ADAPTER_NOT_FOUND =  -4;

    /** An invalid parameter was passed to the function. */
    static const IS_RESULT IS_BAD_PARAMETER =      -5;

    /** An Ethernet link failure occurred. */
    static const IS_RESULT IS_NO_LINK =            -6;

    /** The buffer size is too small to contain all the data to be returned. */
    static const IS_RESULT IS_BUFFER_TOO_SMALL =   -7;

    /** A DHCP negotiation is underway. */
    static const IS_RESULT IS_DHCP_IN_PROGRESS =   -8;

    /** DHCP negotiation failed. */
    static const IS_RESULT IS_DHCP_FAILURE =       -9;

    /** A login operation is in progress. */
    static const IS_RESULT IS_LOGIN_IN_PROGRESS =  -10;

    /** A login operation failed. */
    static const IS_RESULT IS_LOGIN_FAILURE =      -11;

    /** The TCP/IP connection operation failed. */
    static const IS_RESULT IS_TCP_FAILURE =        -12;

    /** Outbound CHAP authentication failed. */
    static const IS_RESULT IS_AUTH_FAILURE =       -13;

    /** Inbound CHAP authentication failed. */
    static const IS_RESULT IS_IN_AUTH_FAILURE =    -14;

    /** An unspecified hardware failure occurred. */
    static const IS_RESULT IS_HARDWARE_FAILURE =   -15;

    /** A subsystem is out of memory. */
    static const IS_RESULT IS_OUT_OF_MEMORY =      -16;

    /** A limitation on the number of targets has been reached. */
    static const IS_RESULT IS_TOO_MANY_TARGETS =   -17;

    /** The specified target name could not be found. */
    static const IS_RESULT IS_TARGET_NOT_FOUND =   -18;

    /** An unspecified operation is pending. */
    static const IS_RESULT IS_OPERATION_PENDING =  -19;

    /** The library was not initialized */
    static const IS_RESULT IS_LIB_NOT_INIT =       -20;

    /** The user is not authorized (root/admin) */
    static const IS_RESULT IS_USER_NOT_AUTH =      -21;

    /** The function has not been implemented */
    static const IS_RESULT IS_NOT_IMPLEMENTED =    -22;

    /** The configuration has been corrupted */
    static const IS_RESULT IS_CONFIG_ERROR =       -23;

    /** A semaphore failure occurred */
    static const IS_RESULT IS_SYNC_FAILURE =       -24;

    /** An I/O error occurred */
    static const IS_RESULT IS_IO_ERROR =           -25;

    /** An I/O error occurred */
    static const IS_RESULT IS_GENERAL_FAILURE =    -26;

    /** A request timed out */
    static const IS_RESULT IS_OPERATION_TIMEOUT =  -27;

#define IS_MAX_TARGET_STATUS_STRING 64
    extern char TARGET_STATUS_TEXT[LAST_AND_INVALID_TARGET_STATUS][IS_MAX_TARGET_STATUS_STRING];

    /** This structure contains user readable descriptions of some of the
        adapter properties */
    typedef struct
    {
        char                    manufacturer[IS_MAX_MANUFACTURER_STRING];
        char                    model[IS_MAX_MODEL_STRING];
        char                    hw_version[IS_MAX_HW_VERSION_STRING];
        char                    ASIC_version[IS_MAX_ASIC_VERSION_STRING];
        char                    firmware_version[IS_MAX_FW_VERSION_STRING];
        char                    os_name[IS_MAX_OS_NAME_STRING];
        u32                     scsi_host_number;
        IS_ADAPTER_PROPS        props;
    } IS_ADAPTER_PROPS_WITH_DESC, *PIS_ADAPTER_PROPS_WITH_DESC;

#define IS_MAX_PRODUCT_VERSION_STRING           256
    typedef struct
    {
        char                    product_version[IS_MAX_PRODUCT_VERSION_STRING];
        u32                     api_version;
        u32                     driver_version;
        u32                     adapter_count;
        u32                     adapter_firmware[IS_MAX_BOARDS];
    } IS_VERSION_INFO, *PIS_VERSION_INFO;

/** ****************************************************************************
 * Procedure:   InitializeLibrary
 *
 * Description: Performs internal initialization of the library.  Must be
 *              performed once per process.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return IS_SUCCESS on success, or other IS_* result codes
 ******************************************************************************/
    IS_RESULT InitializeLibrary (u64 reserved);

/** ****************************************************************************
 * Procedure:   InitializeDriver
 *
 * Description: Sends configuration from the config file (default
 *              /etc/is1000.conf) to the device driver.  After adapters and
 *              targets are configured, login will be initiated for each target.
 *              Must be performed exactly once each time the driver is loaded.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT InitializeDriver (u64 reserved);

/** ****************************************************************************
 * Procedure:   GetVersionInfo
 *
 * Description: Returns all related version information (api, driver, firmware,
 *              product).
 *
 * @param   version_info the returned version information
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetVersionInfo (PIS_VERSION_INFO version_info,
                              u64 reserved);

/** ****************************************************************************
 * Procedure:   GetAdapterCount
 *
 * Description: Returns the number of Intel PRO/1000 T IP storage adapters in
 *              the system for subsequent indexing with EnumAdapters.
 *
 * @param   adapter_count the number of adapters found in the system.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetAdapterCount (u32 *adapter_count,
                               u64 reserved);

/** ****************************************************************************
 * Procedure: EnumAdapters
 *
 * Description: Returns the mac address and name of a particular adapter.
 *
 * @param   index the adapter number to query
 *
 * @param   mac the adapter's MAC address
 *
 * @param   friendly_name a textual name of the adapter that is suitable for
 *          display
 *
 * @param   name_buf_size the maximum string length that can be placed in
 *          friendly_name
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT EnumAdapters (u32 index,
                            IS_MAC_ADDRESS *mac,
                            char *friendly_name,
                            u32 name_buf_size,
                            u64 reserved);

/** ****************************************************************************
 * Procedure: GetAdapterProperties
 *
 * Description: Retrieves adapter details and configuration information from a
 *              particular adapter
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   pprops the returned properties of the adapter
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetAdapterProperties (const IS_MAC_ADDRESS *mac,
                                    PIS_ADAPTER_PROPS_WITH_DESC pprops,
                                    u64 reserved);

/** ****************************************************************************
 * Procedure: SetAdapterProperties
 *
 * Description: Sets the adapter configuration information of a particular
 *              adapter
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   pprops the properties for the adapter
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT SetAdapterProperties (const IS_MAC_ADDRESS *mac,
                                    PIS_ADAPTER_CONF_PROPS pprops,
                                    u64 reserved);

/** ****************************************************************************
 * Procedure: GetAdapterStatistics
 *
 * Description: Retrieves adapter statistical information from a particular
 *              adapter
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   pstats the statistics of the adapter
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetAdapterStatistics (const IS_MAC_ADDRESS *mac,
                                    PIS_SESSION_HBA_STATS pstats,
                                    u64 reserved);

/** ****************************************************************************
 * Procedure: GetAdapterDiagnostics
 *
 * Description: Retrieves current adapter hardware diagnostic status
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   diag_value the diagnostic value for the adapter (see DIAGNOSTIC_* values)
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetAdapterDiagnostics (const IS_MAC_ADDRESS *mac,
                                     u32 *diag_value,
                                     u64 reserved);

/** ****************************************************************************
 * Procedure: GetQueryTargetCount
 *
 * Description: Performs an iSCSI SendTargets operation and returns the number
 *              of discovered targets for subsequent indexing with
 *              EnumQueryTargets.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the portal to query
 *
 * @param   port the port number of the portal to query
 *
 * @param   pchap_settings the chap settings to use for the query (not NULL)
 *
 * @param   target_count the discovered number of targets
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetQueryTargetCount (const IS_MAC_ADDRESS *mac,
                                   u32 ip,
                                   u16 port,
                                   const PIS_CHAP_SETTINGS pchap_settings,
                                   u32 *target_count,
                                   u64 reserved);

/** ****************************************************************************
 * Procedure: EnumQueryTargets
 *
 * Description: Retrieves a particular target that was discovered via
 *              GetQueryTargetCount.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the portal to query
 *
 * @param   port the port number of the portal to query
 *
 * @param   pchap_settings the chap settings to use for the query (not NULL)
 *
 * @param   index the index of the target to retrieve (range 0 to
 *          GetQueryTargetCount result)
 *
 * @param   targetname the buffer that will receive the name of the target
 *
 * @param   targetnameBufferLength the size of the targetname buffer
 *
 * @param   addresses the array to receive the list of addresses for the target
 *
 * @param   numaddresses the number of IS_TARGET_ADDRESSES allocated as input,
 *          and the number of returned addresses as output
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT EnumQueryTargets (const IS_MAC_ADDRESS *mac,
                                u32 ip,
                                u16 port,
                                const PIS_CHAP_SETTINGS pchap_settings,
                                u32 index,
                                char *targetname,
                                u32 targetnameBufferLength,
                                IS_TARGET_ADDRESS *addresses,
                                u32 *numaddresses,
                                u64 reserved);

/** ****************************************************************************
 * Procedure: AddTarget
 *
 * Description: Adds a target to the configuration and attempts to login.  A
 *              IS_SUCCESS result code means that the target was added and
 *              connected.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   pchap_settings the chap settings to use for authentication
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   crcmode the CRC mode to use for this target (header, data, or both)
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT AddTarget (const IS_MAC_ADDRESS *mac,
                         u32 ip,
                         u16 port,
                         const PIS_CHAP_SETTINGS pchap_settings,
                         const char *nodename,
                         IS_CRC_MODE crcmode,
                         u64 reserved);

/** ****************************************************************************
 * Procedure: RemoveTarget
 *
 * Description: Removes a target from configuration and disconnects it if
 *              required.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT RemoveTarget (const IS_MAC_ADDRESS *mac,
                            u32 ip,
                            u16 port,
                            const char *nodename,
                            u64 reserved);

/** ****************************************************************************
 * Procedure: GetTargetsCount
 *
 * Description: Returns the the number of targets that have been configured on
 *              a particular adapter.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   target_count the number of targets
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetTargetsCount (const IS_MAC_ADDRESS *mac,
                               u32 *target_count,
                               u64 reserved);

/** ****************************************************************************
 * Procedure: EnumTargets
 *
 * Description: Returns a particular target on an adapter.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   index the index of the target to retrieve (range 0 to result of
 *          GetTargetsCount)
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT EnumTargets (const IS_MAC_ADDRESS *mac,
                           u32 index,
                           IS_STORAGE_TARGET *st,
                           u64 reserved);

/** ****************************************************************************
 * Procedure: GetTargetsWithLunsCount
 *
 * Description: Performs LUN discovery on the each of the connected targets for
 *              the specified adapter and returns the number of targets that
 *              had at least one LUN for later enumeration via
 *              EnumTargetsWithLuns.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   target_count the number of configured targets that have LUNs
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetTargetsWithLunsCount (const IS_MAC_ADDRESS *mac,
                                       u32 *target_count,
                                       u64 reserved);

/** ****************************************************************************
 * Procedure: EnumTargetsWithLuns
 *
 * Description: Performs LUN discovery on the each of the connected targets for
 *              the specified adapter and returns the target information,
 *              including SCSI device info, for the indexed target.
 *              GetTargetsWithLunsCount must be called first.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   index the index of the desired target (range 0 to result of
 *          GetTargetsWithLunsCount)
 *
 * @param   st details of the matching target
 *
 * @param   devices the array of SCSI device ids for the target
 *
 * @param   numdevices the number of allocated SCSI devices in the devices
 *          parameter for input, and the resulting number of matching devices
 *          on output
 *
 * @param   reserved Must be zero.
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT EnumTargetsWithLuns (const IS_MAC_ADDRESS *mac,
                                   u32 index,
                                   IS_STORAGE_TARGET *st,
                                   IS_SCSI_ID *devices,
                                   u32 *numdevices,
                                   u64 reserved);

/** ****************************************************************************
 * Procedure: SetTargetParameters
 *
 * Description: Adjusts the connection recovery parameters for a target.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   timeout the number of seconds to wait between retries during
 *          connection failures
 *
 * @param   retries the number of times to attempt reconnection before giving up
 *          and reporting a FAULTED connection.  Zero means never give up.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT SetTargetParameters (const IS_MAC_ADDRESS *mac,
                                   u32 ip,
                                   u16 port,
                                   const char *nodename,
                                   u32 timeout,
                                   u32 retries,
                                   u64 reserved);

/** ****************************************************************************
 * Procedure: SetTargetCRC
 *
 * Description: Adjusts the CRC parameter for a target.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   crcmode the CRC mode to use for this target (header, data, or both)
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT SetTargetCRC (const IS_MAC_ADDRESS *mac,
                            u32 ip,
                            u16 port,
                            const char *nodename,
                            IS_CRC_MODE crcmode,
                            u64 reserved);

/** ****************************************************************************
 * Procedure: SetTargetSecurity
 *
 * Description: Adjusts the CHAP settings for a target.  To make the settings
 *              active, call ResetTargetConnection.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   pchap_settings the CHAP settings to use for the target
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT SetTargetSecurity (const IS_MAC_ADDRESS *mac,
                                 u32 ip,
                                 u16 port,
                                 const char *nodename,
                                 const PIS_CHAP_SETTINGS pchap_settings,
                                 u64 reserved);

/** ****************************************************************************
 * Procedure: ResetTargetConnection
 *
 * Description: Causes a target to have its connection torn down and brought
 *              back up.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT ResetTargetConnection (const IS_MAC_ADDRESS *mac,
                                     u32 ip,
                                     u16 port,
                                     const char *nodename,
                                     u64 reserved);

/** ****************************************************************************
 * Procedure: GetTargetRecoveryStatus
 *
 * Description: When a target is in recovery (TARGET_STATUS_RECOVERING), this
 *              function provides information about the progress of
 *              reestablishing the connection.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   timeout the time (in seconds) between retry attempts
 *
 * @param   maxretries the maximum number of retries that will be attempted
 *          (0 for infinite)
 *
 * @param   currentretries the current retry count
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetTargetRecoveryStatus (const IS_MAC_ADDRESS *mac,
                                       u32 ip,
                                       u16 port,
                                       const char *nodename,
                                       u32 *timeout,
                                       u32 *maxretries,
                                       u32 *currentretries,
                                       u64 reserved);

/** ****************************************************************************
 * Procedure: GetTargetStatistics
 *
 * Description: Retrieves statistics related to a particular target.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the IP address of the target
 *
 * @param   port the port number of the target
 *
 * @param   nodename the iSCSI nodename of the target
 *
 * @param   pstats the structure to hold the resulting statistics
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetTargetStatistics (const IS_MAC_ADDRESS *mac,
                                   u32 ip,
                                   u16 port,
                                   const char *nodename,
                                   PIS_SESSION_STATS pstats,
                                   u64 reserved);

/** ****************************************************************************
 * Procedure: GenerateUniqueNodename
 *
 * Description: Returns an iSCSI compilant unique node name.
 *
 * @param   name the buffer to hold the generated name
 *
 * @param   name_size the maximum number of chars to store in name
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GenerateUniqueNodename (char *name,
                                      u32 name_size,
                                      u64 reserved);

/** ****************************************************************************
 * Procedure: SetAdapterBlink
 *
 * Description: Causes the link light on an adapter to blink for the specified
 *              number of seconds.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   seconds the number of seconds to blink the link light
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT SetAdapterBlink (const IS_MAC_ADDRESS *mac,
                               u32 seconds,
                               u64 reserved);

/** ****************************************************************************
 * Procedure: UpdateFlash
 *
 * Description: Uploads a firmware image to all installed adapters.
 *
 * @param   filename the filename of the firmware image
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT UpdateFlash (const char *filename,
                           u64 reserved);

/** ****************************************************************************
 * Procedure: GetEEPromData
 *
 * Description: Retrieves the information stored in the adapter's EEPROM.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   pbuffer the buffer to hold the EEPROM data
 *
 * @param   pbuffer_size the size of the pbuffer as input, the number of
 *          bytes stored in pbuffer as output
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT GetEEPromData (const IS_MAC_ADDRESS *mac,
                             u8 *pbuffer,
                             u32 *pbuffer_size,
                             u64 reserved);

/** ****************************************************************************
 * Procedure: Ping
 *
 * Description: Causes an adapter to perform an ICMP ping operation to the
 *              specified target.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the ip address of the target of the ping operation
 *
 * @param   result the result of the ping operation.  Zero is success.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT Ping (const IS_MAC_ADDRESS *mac,
                    u32 ip,
                    u32 *result,
                    u64 reserved);

/** ****************************************************************************
 * Procedure: LoopbackTest
 *
 * Description: Causes a buffer to be sent through the adapter in loopback mode
 *              for diagnostics.  The output buffer should be identical to the
 *              input buffer if the result is IS_SUCCESS.
 *
 *              CAUTION: this will interrupt all target traffic for multiple
 *                       seconds.  It is not recommended that this function be
 *                       called while there are active targets.
 *
 *              NOTE: this function supports a maximum buffer of 255 bytes.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   pinbuffer the buffer to send
 *
 * @param   poutbuffer the buffer to hold the looped back data
 *
 * @param   buffsize the size of both buffers
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
    IS_RESULT LoopbackTest (const IS_MAC_ADDRESS *mac,
                            u8 *pinbuffer,
                            u8 *poutbuffer,
                            u32 buffsize,
                            u64 reserved);

#ifdef __cplusplus
}
#endif
#endif // __IS1000API_H__
