/*******************************************************************************
* Copyright (C) 2002-2003 Intel Corporation                                    *
*                                                                              *
* This program is free software; you can redistribute it and/or                *
* modify it under the terms of the GNU General Public License                  *
* as published by the Free Software Foundation; either version 2               *
* of the License, or (at your option) any later version.                       *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA   *
*******************************************************************************/
/*******************************************************************************
*                                                                              *
* File Name:    is1000privateapi.h                                             *
*                                                                              *
* Abstract:     This file describes a utility interface to the is1000 support  *
*               library.                                                       *
*                                                                              *
* Environment:  This file is intended to be used by a daemon to update         *
*               target status and properties asynchronously.                   *
*                                                                              *
* Author:       Thomas C. Lea                                                  *
*                                                                              *
* Origination:  April 21, 2003                                                 *
*******************************************************************************/

#ifndef __IS1000PRIVATEAPI_H__
#define __IS1000PRIVATEAPI_H__

#include "is1000api.h"

/** ****************************************************************************
 * Procedure: SetTargetStatus
 *
 * Description: Update the status of an existing target
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the ip address of the target
 *
 * @param   port the port of the target
 *
 * @param   nodename the iSCSI name of the target
 *
 * @param   newStatus the new status to apply to the target
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
IS_RESULT SetTargetStatus(const IS_MAC_ADDRESS *mac,
                          u32 ip,
                          u16 port,
                          const char *nodename,
                          TARGET_STATUS newStatus,
                          u64 reserved);

/** ****************************************************************************
 * Procedure: TargetPropertiesChanged
 *
 * Description: Used by isdaemon when the driver has reported that the target's
 *              properties have changed.  Retrieves the new properties and
 *              persist them to the configuration file.
 *
 * @param   mac the MAC address of the adapter
 *
 * @param   ip the ip address of the target
 *
 * @param   port the port of the target
 *
 * @param   nodename the iSCSI name of the target
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
IS_RESULT TargetPropertiesChanged(const IS_MAC_ADDRESS *mac,
                                  u32 ip,
                                  u16 port,
                                  const char *nodename,
                                  u64 reserved);

/** ****************************************************************************
 * Procedure: UninitializeLibrary
 *
 * Description: Puts the library back in the uninitialized state.  Subsequent
 *              API calls will require a call to InitializeLibrary first.
 *              This is used only for testing.
 *
 * @param   reserved is reserved and must be zero
 *
 * @return  IS_SUCCESS on success, or other IS_* result codes.
 ******************************************************************************/
IS_RESULT UninitializeLibrary(u64 reserved);

#endif  // __IS1000PRIVATEAPI_H__
