/*
 * Core I2O structure management 
 * 
 * (C) Copyright 1999   Red Hat Software 
 * (C) Copyright 2002-3 Wind River Systems
 * (C) Copyright 2003   Intel Corporation
 *
 * Written by Alan Cox, Building Number Three Ltd 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * A lot of the I2O message side code from this is taken from the 
 * Red Creek RCPCI45 adapter driver by Red Creek Communications 
 * 
 * Fixes by: 
 *		Philipp Rumpf 
 *		Juha Sievnen <Juha.Sievanen@cs.Helsinki.FI> 
 *		Auvo Hkkinen <Auvo.Hakkinen@cs.Helsinki.FI> 
 *		Deepak Saxena <deepak@plexity.net> 
 *		Boji T Kannanthanam <boji.t.kannanthanam@intel.com>
 *		Elizabeth A. Baldwin <libby@windriver.com>
 *		Jason O'Broin <jason@windriver.com>
 *		Dan Krejsa <dan.krejsa@windriver.com>
 *		Mikal Hart <mikal.n.hart@intel.com>
 *
 * NOTE: 11/22/02
 *
 * (C) Copyright 2002   Wind River Systems
 * (C) Copyright 2003   Intel Corporation
 *
 * This code for the Intel PRO/1000 T IP Storage Adapter is1000 driver
 * was based heavily on the Red Hat i2o_core.c module. As such all 
 * term of the GNU General Public License Apply.
 *
 * 14oct03 eab Version 2.0.30: Compiled with new is1000common.h
 * 16sep03 eab Version 2.0.29: Clear LAA Address when there is
 *             none; cleanup
 * 11sep03 eab Version 2.0.28: Fix systab_send() so garbage not
 *             sent for buf addrs; Register for events on module
 *             load so always get adapter events; retrieve missing
 *             adapter properties
 * 27aug03 eab Version 2.0.27: Return locally administered
 *             address in adapter properties struct; more detailed
 *             error codes on failed target discovery
 * 26aug03 eab Version 2.0.26 started: More refined error checking
 *             on target discovery; Patched in high mem i/o changes
 * 21aug03 eab Version 2.0.25 started: Make queried target get
 *             use same target info as done in preceding target
 *             query op 
 * 13aug03 eab Version 2.0.24 started: cleanup scsi error output;
 *             bound max enumerated targets index by number of
 *             targets found, not system max; cleaned up code.
 * 11aug03 eab Version 2.0.23 started: Support for permanent
 *             redirection
 * 07aug03 eab Version 2.0.22 started: Support for DHCP 
 *             error events, diagnostics; Get adapter 
 *             IP config from card, not cache
 * 04aug03 eab Version 2.0.21 started; remove of target in any
 *             state and w/ outstanding cmds now works
 * 29jul03 eab Version 2.0.20: Newer build scheme, cleaned up
 *             msg frame size, pool size business
 * 24jul03 eab Version 2.0.19: New all-kernel build scheme
 * 22jul03 eab Version 2.0.18: Removed unused ioctls
 * 17jul03 eab Version 2.0.17: fixed bug in chap setting ioctl;
 *             improved error recovery.
 * 16jun03 eab Porting to Red Hat 9 started: version 2.0.7
 * 13jun03 eab Version 2.0.5 delivered for Alpha 3 release
 * 07may03 eab Compile with new headers, add stubs
 * 07may03 eab First round support for i2o class added
 * 07mar03 eab Modified access to i2o_controllers[] to really use use
 *             count (users field), ultimately used in controller
 *             delete routine, but also used to control access to 
 *             controllers. See is1000_scsi.c for fix there.
 * 19feb03 mnh support for software download reinserted
 * 20dec02 eab Can now read 8 target names, log into targets consistently,
 *             and report correct mac address if locally administered
 * 22nov02 eab Written, based on the RH 7.2 i2o_core.c file.
 * 
 * To Do: 
 * Minimize copy_to_user data by coping back only the status
 *     or requested info in structs returned to user in Ioctl()
 * Find out if target status needs lock, since sending certain
 *     scsi cmds to an offline target may cause problems
 * Reference linux-iscsi/iscsi-kernel.h file for kernel specific
 *     changes needed in code. Look for job at Microsoft.
 * Figure out changes needed for 2.4.20-8 in particular,
 *     concerning signals, and locking around signal processing
 * Cleanup all RH90 <=> RHAS porting issues
 * Get rid of all cruft that's currently commented out
 * Get rid of residual i2o functionality that's not used
 */

#include <linux/config.h>


#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/pci.h>

#include <linux/i2o.h>

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/smp_lock.h>

#include <linux/bitops.h>
#include <linux/wait.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/tqueue.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>
#include <linux/completion.h>

#include <asm/io.h>
#include <linux/reboot.h>

/* New Headers - includes ../common/is1000common.h */
#include "is1000ioctl.h"
#include "is1000events.h"

#include "is1000.h"

//#define DRIVERDEBUG
//#define IOCTLDEBUG


#ifdef DRIVERDEBUG
#define dprintk(s, args...) printk(s, ## args)
#else
#define dprintk(s, args...)
#endif

/* OSM table */
static struct i2o_handler *i2o_handlers[MAX_I2O_MODULES];

/* Controller list */
static struct i2o_controller *i2o_controllers[MAX_I2O_CONTROLLERS];
struct i2o_controller *i2o_controller_chain;
int i2o_num_controllers;

/* iSCSI struct */
struct iscsi_adapter_private iscsi_adapter_priv[IS_MAX_BOARDS];

/* Need to save the number of targets found in the
 * last target inquiry, and the pointer to the struct
 * containing all the target data.
 */
int numEnumTargets; 
static ISCSI_TOTAL_ENUM_TARGETS *pEnumTargets;

/* Initiator Context for Core message */
static int core_context;

/* Initialization && shutdown functions */
//static void i2o_sys_init(void);
static void is1000_sys_init(void);
static void i2o_sys_shutdown(void);
static int i2o_reset_controller(struct i2o_controller *);
static int i2o_reboot_event(struct notifier_block *, unsigned long , void *);
static int i2o_online_controller(struct i2o_controller *);
static int i2o_init_outbound_q(struct i2o_controller *);
static int i2o_post_outbound_messages(struct i2o_controller *);
static int ioctl_swdl(unsigned long arg);

static int ControlMajor;
static const char *ControlName = "is1000";
static int ControlOpen( struct inode *inode, struct file *file );
static int ControlClose( struct inode *inode, struct file *file );
static int ControlIoctl( struct inode *inode,
                         struct file *file,
                         unsigned int cmd,
                         unsigned long arg );
static ssize_t ControlRead(struct file *file, char *buf, 
			   size_t count, loff_t *ptr);

#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0) )
static struct file_operations ControlFops = {
	owner: THIS_MODULE,
	ioctl: ControlIoctl,    /* ioctl */
	open: ControlOpen,      /* open */
	release: ControlClose,  /* release */
	read: ControlRead,		/* read */
};
#else
static struct file_operations ControlFops = {
	NULL,                   /* lseek */
	ControlRead,            /* read */
	NULL,                   /* write */
	NULL,                   /* readdir */
	NULL,                   /* poll */
	ControlIoctl,       /* ioctl */
	NULL,                   /* mmap */
	ControlOpen,        /* open */
	NULL,                   /* flush */
	ControlClose,       /* release */
};
#endif

/* Reply handler */
static void i2o_core_reply(struct i2o_handler *, struct i2o_controller *,
			   struct i2o_message *);

/* Various helper functions */
static int i2o_lct_get(struct i2o_controller *);
static int i2o_lct_notify(struct i2o_controller *);
static int i2o_hrt_get(struct i2o_controller *);

static int i2o_build_sys_table(void);
static int i2o_systab_send(struct i2o_controller *c);

/* I2O core event handler */
static int i2o_core_evt(void *);
static int evt_pid;
static int evt_running;

/* Dynamic LCT update handler */
static int i2o_dyn_lct(void *);

void i2o_report_controller_unit(struct i2o_controller *, struct i2o_device *);

/*
 * I2O System Table.  Contains information about
 * all the IOPs in the system.  Used to inform IOPs
 * about each other's existence.
 *
 * sys_tbl_ver is the CurrentChangeIndicator that is
 * used by IOPs to track changes.
 */
static struct i2o_sys_tbl *sys_tbl;
static int sys_tbl_ind;
static int sys_tbl_len;


/* Socket Event handler */
#define NO_CONTEXT -1

struct i2o_handler sIs1000EventHandler =
{
	is1000EventHandler,  /* Function that handles Socket Events */
	NULL,                   /* New device notification handler */
	NULL,                   /* Device deletion handler */
	NULL,                   /* Reboot notification handler */
	"IS1000 OSM",           /* OSM name */
	NO_CONTEXT,             /* Initiator Context (last 16 bits) */
	/* Filled in by i2o_core, initialized to -1 */
	I2O_CLASS_BUS_ADAPTER_PORT  /* I2O classes that the driver handles */
};

/*
 * I2O configuration spinlock. This isnt a big deal for contention
 * so we have one only
 */

static DECLARE_MUTEX(i2o_configuration_lock);

/*
 * This spin lock is used to keep a device from being
 * added and deleted concurrently across CPUs or interrupts.
 * This can occur when a user creates a device and immediatelly
 * deletes it before the new_dev_notify() handler is called.
 */
static spinlock_t i2o_dev_lock = SPIN_LOCK_UNLOCKED;

#ifndef RH90
#ifdef MODULE
/* 
 * Function table to send to bus specific layers
 * See <include/linux/i2o.h> for explanation of this
 */
static struct i2o_core_func_table i2o_core_functions =
{
	i2o_install_controller,
	i2o_activate_controller,
	i2o_find_controller,
	i2o_unlock_controller,
	i2o_run_queue,
	i2o_delete_controller
};

extern int i2o_pci_core_attach(struct i2o_core_func_table *);
extern void i2o_pci_core_detach(void);

#endif /* MODULE */
#else /* RH90 */

extern int i2o_pci_core_attach(void);
extern void i2o_pci_core_detach(void);

#endif

extern int i2o_scsi_init_wrapper(struct i2o_controller *c, struct i2o_device *d, u32 hp);
extern struct i2o_scsi_host * is1000_get_hostdata(u32 unit);
extern int is1000_tid_match(u8 *pTargetId, char *iscsi_nodename, int unit);
extern int is1000_bus_reset(u32 unit, u32 targetId, u32 context);
//extern void is1000_remove_target (char *targetName, int adapterNum);

/*
 * Structures and definitions for synchronous message posting.
 * See i2o_post_wait() for description.
 */ 
struct i2o_post_wait_data
{
	int *status;		/* Pointer to status block on caller stack */
	int *complete;		/* Pointer to completion flag on caller stack */
	u32 id;			/* Unique identifier */
	wait_queue_head_t *wq;	/* Wake up for caller (NULL for dead) */
	struct i2o_post_wait_data *next;	/* Chain */
	void *mem[2];		/* Memory blocks to recover on failure path */
};
static struct i2o_post_wait_data *post_wait_queue;
static u32 post_wait_id;	// Unique ID for each post_wait
static spinlock_t post_wait_lock = SPIN_LOCK_UNLOCKED;
static void i2o_post_wait_complete(u32, int);

/* OSM descriptor handler */ 
static struct i2o_handler i2o_core_handler =
{
	(void *)i2o_core_reply,
	NULL,
	NULL,
	NULL,
	"I2O core layer",
	0,
	I2O_CLASS_EXECUTIVE
};

/*
 * Used when queueing a reply to be handled later
 */
 
struct reply_info
{
	struct i2o_controller *iop;
	u32 msg[MSG_FRAME_SIZE]; // msg size in words
};
static struct reply_info evt_reply;
static struct reply_info events[I2O_EVT_Q_LEN];
static int evt_in;
static int evt_out;
static int evt_q_len;
#define MODINC(x,y) ((x) = ((x) + 1) % (y))

/* 
 * Event spinlock.  Used to keep event queue sane and from
 * handling multiple events simultaneously.
 */
static spinlock_t i2o_evt_lock = SPIN_LOCK_UNLOCKED;

/*
 * Semaphore used to synchronize event handling thread with 
 * interrupt handler.
 */
 
static DECLARE_MUTEX(evt_sem);
static DECLARE_COMPLETION(evt_dead);
DECLARE_WAIT_QUEUE_HEAD(evt_wait);


/* Structs and such for is1000 specific event
 * handling and delivery to daemon thread.
 */
#define IS1000_EVT_Q_LEN 16
static IS1000_EVENT connectionEvent[IS1000_EVT_Q_LEN];
static int is1000_evt_in;
static int is1000_evt_out;
static int is1000_evt_q_len;

static spinlock_t is1000_evt_lock = SPIN_LOCK_UNLOCKED;

static u32 iscsiDiagnostic;
/*
 * Semaphore and queue used for is1000 Event Handling
 * and daemon management.
 */
static DECLARE_MUTEX(is1000_evt_sem);

static struct notifier_block i2o_reboot_notifier =
{
        i2o_reboot_event,
        NULL,
        0
};

/*
 *	Config options
 */

static int verbose;
MODULE_PARM(verbose, "i");

void inet_ntoa
(
	/*    struct in_addr inetAddress,  inet address */
	uint inetAddress, /* inet address */
	char *pString               /* where to return ASCII string */
	)
{
	register char *p = (char *)&inetAddress;

#define	UC(b)	(((int)b)&0xff)
	(void) sprintf (pString, "%d.%d.%d.%d",
			UC(p[0]), UC(p[1]), UC(p[2]), UC(p[3]));
}

/*
 * i2o_util_nop
 *
 * Function to create and send a UtilNop message to the IOP. This is 
 * a useful way of returning an unused message frame to the IOP.
 */
void i2o_util_nop(struct i2o_controller *c, u32 m)
{
	u32 *pMsg = (u32 *) (c->mem_offset + m);

#ifdef RH90
	i2o_raw_writel(  I2O_MESSAGE_SIZE(sizeof(struct i2o_util_nop_message))
			 | I2OVER15, pMsg);
	i2o_raw_writel(I2O_CMD_UTIL_NOP << 24 | HOST_TID << 12, pMsg+1);
	i2o_raw_writel(0, pMsg+2);
#else
	__raw_writel(  I2O_MESSAGE_SIZE(sizeof(struct i2o_util_nop_message))
		       | I2OVER15, pMsg);
	__raw_writel(I2O_CMD_UTIL_NOP << 24 | HOST_TID << 12, pMsg+1);
	__raw_writel(0, pMsg+2);
#endif
	i2o_post_message(c, m);
}

/*
 * I2O Core reply handler
 */
static void i2o_core_reply(struct i2o_handler *h, struct i2o_controller *c,
			   struct i2o_message *m)
{
	u32 *msg=(u32 *)m;
	u32 status;
	u32 context = msg[2];

	if (msg[0] & MSG_FAIL) // Fail bit is set
	{
		u32 *preserved_msg = (u32*)(c->mem_offset + msg[7]);

		i2o_report_status(KERN_INFO, "i2o_core", msg);
		i2o_dump_message(preserved_msg);

		/* If the failed request needs special treatment,
		 * it should be done here. */

                /* Release the preserved msg by resubmitting it as a NOP */

#ifdef RH90
		preserved_msg[0] = cpu_to_le32(THREE_WORD_MSG_SIZE | SGL_OFFSET_0);
		preserved_msg[1] = cpu_to_le32(I2O_CMD_UTIL_NOP << 24 | HOST_TID << 12 | 0);
#else
		preserved_msg[0] = THREE_WORD_MSG_SIZE | SGL_OFFSET_0;
		preserved_msg[1] = I2O_CMD_UTIL_NOP << 24 | HOST_TID << 12 | 0;
#endif
		preserved_msg[2] = 0;
		i2o_post_message(c, msg[7]);

		/* If reply to i2o_post_wait failed, return causes a timeout */

		return;
	}       

#ifdef DRIVERDEBUG
	i2o_report_status(KERN_INFO, "i2o_core", msg);
#endif

	if(msg[2]&0x80000000)	// Post wait message
	{
		if (msg[4] >> 24)
			status = (msg[4] & 0xFFFF);
		else
			status = I2O_POST_WAIT_OK;
	
		i2o_post_wait_complete(context, status);
		return;
	}

	if(m->function == I2O_CMD_UTIL_EVT_REGISTER)
	{
#ifdef IOCTLDEBUG
		printk("\nis1000(): i2o_core_reply handling I2O_CMD_UTIL_EVT_REGISTER reply\n");
#endif
		memcpy(events[evt_in].msg, msg, (msg[0]>>16)<<2);
		events[evt_in].iop = c;

		spin_lock(&i2o_evt_lock);
		MODINC(evt_in, I2O_EVT_Q_LEN);
		if(evt_q_len == I2O_EVT_Q_LEN)
			MODINC(evt_out, I2O_EVT_Q_LEN);
		else
			evt_q_len++;
		spin_unlock(&i2o_evt_lock);

		up(&evt_sem);
		
		wake_up_interruptible(&evt_wait);
		return;
	}

	if(m->function == I2O_CMD_LCT_NOTIFY)
	{
		up(&c->lct_sem);
		return;
	}

	/*
	 * If this happens, we want to dump the message to the syslog so
	 * it can be sent back to the card manufacturer by the end user
	 * to aid in debugging.
	 * 
	 */
	printk(KERN_WARNING "%s: Unsolicited message reply sent to core!"
	       "Message dumped to syslog\n", 
	       c->name);
	i2o_dump_message(msg);

	return;
}

/**
 *	i2o_install_handler - install a message handler
 *	@h: Handler structure
 *
 *	Install an I2O handler - these handle the asynchronous messaging
 *	from the card once it has initialised. If the table of handlers is
 *	full then -ENOSPC is returned. On a success 0 is returned and the
 *	context field is set by the function. The structure is part of the
 *	system from this time onwards. It must not be freed until it has
 *	been uninstalled
 */
 
int i2o_install_handler(struct i2o_handler *h)
{
	int i;
	down(&i2o_configuration_lock);
	for(i=0;i<MAX_I2O_MODULES;i++)
	{
		if(i2o_handlers[i]==NULL)
		{
			h->context = i;
			i2o_handlers[i]=h;
			up(&i2o_configuration_lock);
			return 0;
		}
	}
	up(&i2o_configuration_lock);
	return -ENOSPC;
}

/**
 *	i2o_remove_handler - remove an i2o message handler
 *	@h: handler
 *
 *	Remove a message handler previously installed with i2o_install_handler.
 *	After this function returns the handler object can be freed or re-used
 */
 
int i2o_remove_handler(struct i2o_handler *h)
{
	i2o_handlers[h->context]=NULL;
	return 0;
}
	

/*
 *	Each I2O controller has a chain of devices on it.
 * Each device has a pointer to it's LCT entry to be used
 * for fun purposes.
 */

/**
 *	i2o_install_device	-	attach a device to a controller
 *	@c: controller
 *	@d: device
 * 	
 *	Add a new device to an i2o controller. This can be called from
 *	non interrupt contexts only. It adds the device and marks it as
 *	unclaimed. The device memory becomes part of the kernel and must
 *	be uninstalled before being freed or reused. Zero is returned
 *	on success.
 */
 
int i2o_install_device(struct i2o_controller *c, struct i2o_device *d)
{
	int i;

	down(&i2o_configuration_lock);
	d->controller=c;
	d->owner=NULL;
	if (c->devices == NULL)
	{
		d->prev = d;
		d->next = NULL;
		c->devices = d;
	}
	else
	{
		d->prev = c->devices->prev;
		d->next = NULL;
		c->devices->prev->next = d;
		c->devices->prev = d;
	}
	*d->dev_name = 0;

	for(i = 0; i < I2O_MAX_MANAGERS; i++)
		d->managers[i] = NULL;

	up(&i2o_configuration_lock);
	return 0;
}

/* we need this version to call out of i2o_delete_controller */

int __i2o_delete_device(struct i2o_device *d)
{
	struct i2o_device **p;
	int i;

	p=&(d->controller->devices);

	/*
	 *	Hey we have a driver!
	 * Check to see if the driver wants us to notify it of 
	 * device deletion. If it doesn't we assume that it
	 * is unsafe to delete a device with an owner and 
	 * fail.
	 */
	if(d->owner)
	{
		if(d->owner->dev_del_notify)
		{
			dprintk(KERN_INFO "Device has owner, notifying\n");
			d->owner->dev_del_notify(d->controller, d);
			if(d->owner)
			{
				printk(KERN_WARNING 
				       "Driver \"%s\" did not release device!\n", d->owner->name);
				return -EBUSY;
			}
		}
		else
			return -EBUSY;
	}

	/*
	 * Tell any other users who are talking to this device
	 * that it's going away.  We assume that everything works.
	 */
	for(i=0; i < I2O_MAX_MANAGERS; i++)
	{
		if(d->managers[i] && d->managers[i]->dev_del_notify)
			d->managers[i]->dev_del_notify(d->controller, d);
	}
	 			
	while(*p!=NULL)
	{
		if(*p==d)
		{
			/*
			 *	Destroy
			 */
			if (d->next == NULL)
				d->controller->devices->prev = d->prev;
			else
				d->next->prev = d->prev;
			*p=d->next;
			kfree(d);
			return 0;
		}
		p=&((*p)->next);
	}
	printk(KERN_ERR "i2o_delete_device: passed invalid device.\n");
	return -EINVAL;
}

/**
 *	i2o_delete_device	-	remove an i2o device
 *	@d: device to remove
 *
 *	This function unhooks a device from a controller. The device
 *	will not be unhooked if it has an owner who does not wish to free
 *	it, or if the owner lacks a dev_del_notify function. In that case
 *	-EBUSY is returned. On success 0 is returned. Other errors cause
 *	negative errno values to be returned
 */
 
int i2o_delete_device(struct i2o_device *d)
{
	int ret;

	down(&i2o_configuration_lock);

	/*
	 *	Seek, locate
	 */

	ret = __i2o_delete_device(d);

	up(&i2o_configuration_lock);

	return ret;
}

/**
 *	i2o_install_controller	-	attach a controller
 *	@c: controller
 * 	
 *	Add a new controller to the i2o layer. This can be called from
 *	non interrupt contexts only. It adds the controller and marks it as
 *	unused with no devices. If the tables are full or memory allocations
 *	fail then a negative errno code is returned. On success zero is
 *	returned and the controller is bound to the system. The structure
 *	must not be freed or reused until being uninstalled.
 */
 
int i2o_install_controller(struct i2o_controller *c)
{
	int i;
	down(&i2o_configuration_lock);
	for(i=0;i<MAX_I2O_CONTROLLERS;i++)
	{
		if(i2o_controllers[i]==NULL)
		{
		        c->dlct = (i2o_lct*)kmalloc(8192, GFP_KERNEL);
			if(c->dlct==NULL)
			{
				up(&i2o_configuration_lock);
				return -ENOMEM;
			}
			i2o_controllers[i]=c;
			c->devices = NULL;
			c->next=i2o_controller_chain;
			i2o_controller_chain=c;
			c->unit = i;
			c->page_frame = NULL;
			c->hrt = NULL;
			c->lct = NULL;
			c->status_block = NULL;
			sprintf(c->name, "is1000_%d", i);
			i2o_num_controllers++;
			init_MUTEX_LOCKED(&c->lct_sem);
			up(&i2o_configuration_lock);
			return 0;
		}
	}
	printk(KERN_ERR "No free i2o controller slots.\n");
	up(&i2o_configuration_lock);
	return -EBUSY;
}

/**
 *	i2o_delete_controller	- delete a controller
 *	@c: controller
 *	
 *	Remove an i2o controller from the system. If the controller or its
 *	devices are busy then -EBUSY is returned. On a failure a negative
 *	errno code is returned. On success zero is returned.
 */
  
int i2o_delete_controller(struct i2o_controller *c)
{
	struct i2o_controller **p;
	int users;
	char name[16];
	int stat;

	dprintk(KERN_INFO "Deleting controller %s\n", c->name);

	/*
	 * Clear event registration as this can cause weird behavior
	 */
	if(c->status_block->iop_state == ADAPTER_STATE_OPERATIONAL)
		i2o_event_register(c, core_context, 0, 0, 0);

	down(&i2o_configuration_lock);

	if((users=atomic_read(&c->users)))
	{
		dprintk(KERN_INFO "I2O: %d users for controller %s\n", users,
			c->name);
		up(&i2o_configuration_lock);
		return -EBUSY;
	}
	while(c->devices)
	{
		if(__i2o_delete_device(c->devices)<0)
		{
			/* Shouldnt happen */
			c->bus_disable(c);
			up(&i2o_configuration_lock);
			return -EBUSY;
		}
	}

	/*
	 * If this is shutdown time, the thread's already been killed
	 */
	if(c->lct_running) {
		stat = kill_proc(c->lct_pid, SIGTERM, 1);
		if(!stat) {
			int count = 10 * 100;
			while(c->lct_running && --count) {
				current->state = TASK_INTERRUPTIBLE;
				schedule_timeout(1);
			}
		
			if(!count)
				printk(KERN_ERR 
				       "%s: LCT thread still running!\n", 
				       c->name);
		}
	}

	p=&i2o_controller_chain;

	while(*p)
	{
		if(*p==c)
		{
 			/* Ask the IOP to switch to RESET state */
		        i2o_reset_controller(c);

			/* Release IRQ */
			c->destructor(c);

			*p=c->next;
			up(&i2o_configuration_lock);

			if(c->page_frame)
			{
#ifdef RH90

#if defined(CONFIG_HIGHIO) && defined(IS1000_HIGHMEMIO)
				pci_unmap_page(c->pdev, c->page_frame_map, MSG_POOL_SIZE, PCI_DMA_FROMDEVICE);
#else
				pci_unmap_single(c->pdev, c->page_frame_map, MSG_POOL_SIZE, PCI_DMA_FROMDEVICE);
#endif

#endif
				kfree(c->page_frame);
			}
			if(c->hrt)
				kfree(c->hrt);
			if(c->lct)
				kfree(c->lct);
			if(c->status_block)
				kfree(c->status_block);
			if(c->dlct)
			        kfree(c->dlct);

			i2o_controllers[c->unit]=NULL;
			memcpy(name, c->name, strlen(c->name)+1);
			kfree(c);
			dprintk(KERN_INFO "%s: Deleted from controller chain.\n", name);
			
			i2o_num_controllers--;
			return 0;
		}
		p=&((*p)->next);
	}
	up(&i2o_configuration_lock);
	printk(KERN_ERR "i2o_delete_controller: bad pointer!\n");
	return -ENOENT;
}

/**
 *	i2o_unlock_controller	-	unlock a controller
 *	@c: controller to unlock
 *
 *	Take a lock on an i2o controller. This prevents it being deleted.
 *	i2o controllers are not refcounted so a deletion of an in use device
 *	will fail, not take affect on the last dereference.
 */
 
void i2o_unlock_controller(struct i2o_controller *c)
{
	atomic_dec(&c->users);
}

/**
 *	i2o_find_controller - return a locked controller
 *	@n: controller number
 *
 *	Returns a pointer to the controller object. The controller is locked
 *	on return. NULL is returned if the controller is not found.
 */
 
struct i2o_controller *i2o_find_controller(int n)
{
	struct i2o_controller *c;
	
	if(n<0 || n>=MAX_I2O_CONTROLLERS)
		return NULL;
	
	down(&i2o_configuration_lock);
	c=i2o_controllers[n];
	if(c!=NULL)
		atomic_inc(&c->users);

	up(&i2o_configuration_lock);
	return c;
}

/**
 *	i2o_issue_claim	- claim or release a device
 *	@cmd: command
 *	@c: controller to claim for
 *	@tid: i2o task id
 *	@type: type of claim
 *
 *	Issue I2O UTIL_CLAIM and UTIL_RELEASE messages. The message to be sent
 *	is set by cmd. The tid is the task id of the object to claim and the
 *	type is the claim type (see the i2o standard)
 *
 *	Zero is returned on success.
 */
 
static int i2o_issue_claim(u32 cmd, struct i2o_controller *c, int tid, u32 type)
{
	u32 msg[5];

	msg[0] = FIVE_WORD_MSG_SIZE | SGL_OFFSET_0;
	msg[1] = cmd << 24 | HOST_TID<<12 | tid;
	msg[3] = 0;
	msg[4] = type;
	
	return i2o_post_wait(c, msg, sizeof(msg), 60);
}

/*
 * 	i2o_claim_device - claim a device for use by an OSM
 *	@d: device to claim
 *	@h: handler for this device
 *
 *	Do the leg work to assign a device to a given OSM on Linux. The
 *	kernel updates the internal handler data for the device and then
 *	performs an I2O claim for the device, attempting to claim the
 *	device as primary. If the attempt fails a negative errno code
 *	is returned. On success zero is returned.
 */
 
int i2o_claim_device(struct i2o_device *d, struct i2o_handler *h)
{
	down(&i2o_configuration_lock);
	if (d->owner) {
		printk(KERN_INFO "Device claim called, but dev already owned by %s!",
		       h->name);
		up(&i2o_configuration_lock);
		return -EBUSY;
	}
	d->owner=h;

	if(i2o_issue_claim(I2O_CMD_UTIL_CLAIM ,d->controller,d->lct_data.tid, 
			   I2O_CLAIM_PRIMARY))
	{
		d->owner = NULL;
		return -EBUSY;
	}
	up(&i2o_configuration_lock);
	return 0;
}

/**
 *	i2o_release_device - release a device that the OSM is using
 *	@d: device to claim
 *	@h: handler for this device
 *
 *	Drop a claim by an OSM on a given I2O device. The handler is cleared
 *	and 0 is returned on success.
 *
 *	AC - some devices seem to want to refuse an unclaim until they have
 *	finished internal processing. It makes sense since you don't want a
 *	new device to go reconfiguring the entire system until you are done.
 *	Thus we are prepared to wait briefly.
 */

int i2o_release_device(struct i2o_device *d, struct i2o_handler *h)
{
	int err = 0;
	int tries;

	down(&i2o_configuration_lock);
	if (d->owner != h) {
		printk(KERN_INFO "Claim release called, but not owned by %s!\n",
		       h->name);
		up(&i2o_configuration_lock);
		return -ENOENT;
	}	

	for(tries=0;tries<10;tries++)
	{
		d->owner = NULL;

		/*
		 *	If the controller takes a nonblocking approach to
		 *	releases we have to sleep/poll for a few times.
		 */
		 
		if((err=i2o_issue_claim(I2O_CMD_UTIL_RELEASE, d->controller, d->lct_data.tid, I2O_CLAIM_PRIMARY)) )
		{
			err = -ENXIO;
			current->state = TASK_UNINTERRUPTIBLE;
			schedule_timeout(HZ);
		}
		else
		{
			err=0;
			break;
		}
	}
	up(&i2o_configuration_lock);
	return err;
}

/**
 * 	i2o_device_notify_on	-	Enable deletion notifiers
 *	@d: device for notification
 *	@h: handler to install
 *
 *	Called by OSMs to let the core know that they want to be
 *	notified if the given device is deleted from the system.
 */

int i2o_device_notify_on(struct i2o_device *d, struct i2o_handler *h)
{
	int i;

	if(d->num_managers == I2O_MAX_MANAGERS)
		return -ENOSPC;

	for(i = 0; i < I2O_MAX_MANAGERS; i++)
	{
		if(!d->managers[i])
		{
			d->managers[i] = h;
			break;
		}
	}
	
	d->num_managers++;
	
	return 0;
}

/**
 * 	i2o_device_notify_off	-	Remove deletion notifiers
 *	@d: device for notification
 *	@h: handler to remove
 *
 * Called by OSMs to let the core know that they no longer
 * are interested in the fate of the given device.
 */
int i2o_device_notify_off(struct i2o_device *d, struct i2o_handler *h)
{
	int i;

	for(i=0; i < I2O_MAX_MANAGERS; i++)
	{
		if(d->managers[i] == h)
		{
			d->managers[i] = NULL;
			d->num_managers--;
			return 0;
		}
	}

	return -ENOENT;
}

/**
 *	i2o_event_register	-	register interest in an event
 * 	@c: Controller to register interest with
 *	@tid: I2O task id
 *	@init_context: initiator context to use with this notifier
 *	@tr_context: transaction context to use with this notifier
 *	@evt_mask: mask of events
 *
 *	Create and posts an event registration message to the task. No reply
 *	is waited for, or expected. Errors in posting will be reported.
 */
 
int i2o_event_register(struct i2o_controller *c, u32 tid, 
		       u32 init_context, u32 tr_context, u32 evt_mask)
{
	u32 msg[5];	// Not performance critical, so we just 
			// i2o_post_this it instead of building it
			// in IOP memory
	
	msg[0] = FIVE_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1] = I2O_CMD_UTIL_EVT_REGISTER<<24 | HOST_TID<<12 | tid;
	msg[2] = init_context;
	msg[3] = tr_context;
	msg[4] = evt_mask;

#ifdef IOCTLDEBUG
	printk(KERN_INFO "is1000: event registering with evt_mask 0x%08x\n",
	       evt_mask);
#endif
	return i2o_post_this(c, msg, sizeof(msg));
}

/*
 * 	i2o_event_ack	-	acknowledge an event
 *	@c: controller 
 *	@msg: pointer to the UTIL_EVENT_REGISTER reply we received
 *
 *	We just take a pointer to the original UTIL_EVENT_REGISTER reply
 *	message and change the function code since that's what spec
 *	describes an EventAck message looking like.
 */
 
int i2o_event_ack(struct i2o_controller *c, u32 *msg)
{
	struct i2o_message *m = (struct i2o_message *)msg;

	m->function = I2O_CMD_UTIL_EVT_ACK;

	return i2o_post_wait(c, msg, m->size * 4, 2);
}

/*
 * Core event handler.  Runs as a separate thread and is woken
 * up whenever there is an Executive class event.
 */
static int i2o_core_evt(void *reply_data)
{
	struct reply_info *reply = (struct reply_info *) reply_data;
	u32 *msg = reply->msg;
	struct i2o_controller *c = NULL;
	unsigned long flags;

	lock_kernel();
	daemonize();
	unlock_kernel();

	strcpy(current->comm, "is1000evtd");
	evt_running = 1;

	while(1)
	{
		if(down_interruptible(&evt_sem))
		{
			dprintk(KERN_INFO "is1000 event thread dead\n");
			printk("exiting...");
			evt_running = 0;
			complete_and_exit(&evt_dead, 0);
		}

		/* 
		 * Copy the data out of the queue so that we don't have to lock
		 * around the whole function and just around the qlen update
		 */
		spin_lock_irqsave(&i2o_evt_lock, flags);
		memcpy(reply, &events[evt_out], sizeof(struct reply_info));
		MODINC(evt_out, I2O_EVT_Q_LEN);
		evt_q_len--;
		spin_unlock_irqrestore(&i2o_evt_lock, flags);
	
		c = reply->iop;
	 	dprintk(KERN_INFO "I2O IRTOS EVENT: iop%d, event %#10x\n", c->unit, msg[4]);

		/* 
		 * We do not attempt to delete/quiesce/etc. the controller if
		 * some sort of error indidication occurs.  We may want to do
		 * so in the future, but for now we just let the user deal with 
		 * it.  One reason for this is that what to do with an error
		 * or when to send what rror is not really agreed on, so
		 * we get errors that may not be fatal but just look like they
		 * are...so let the user deal with it.
		 */
		switch(msg[4])
		{
		case I2O_EVT_IND_EXEC_RESOURCE_LIMITS:
			printk(KERN_ERR "%s: Out of resources\n", c->name);
			break;

		case I2O_EVT_IND_EXEC_POWER_FAIL:
			printk(KERN_ERR "%s: Power failure\n", c->name);
			break;

		case I2O_EVT_IND_EXEC_HW_FAIL:
		{
			char *fail[] = 
				{ 
					"Unknown Error",
					"Power Lost",
					"Code Violation",
					"Parity Error",
					"Code Execution Exception",
					"Watchdog Timer Expired" 
				};

			if(msg[5] <= 6)
				printk(KERN_ERR "%s: Hardware Failure: %s\n", 
				       c->name, fail[msg[5]]);
			else
				printk(KERN_ERR "%s: Unknown Hardware Failure\n", c->name);

			break;
		}

		/*
		 * New device created
		 * - Create a new i2o_device entry
		 * - Inform all interested drivers about this device's existence
		 */
		case I2O_EVT_IND_EXEC_NEW_LCT_ENTRY:
		{
			struct i2o_device *d = (struct i2o_device *)
				kmalloc(sizeof(struct i2o_device), GFP_KERNEL);
			int i;

			if (d == NULL) {
				printk(KERN_EMERG "i2oevtd: out of memory\n");
				break;
			}
			memcpy(&d->lct_data, &msg[5], sizeof(i2o_lct_entry));
	
			d->next = NULL;
			d->controller = c;
			d->flags = 0;
	
			i2o_report_controller_unit(c, d);
#ifdef IOCTLDEBUG
			printk("is1000: i2o class: i2o_install_device():0:\n");
#endif
			i2o_install_device(c,d);
	
			for(i = 0; i < MAX_I2O_MODULES; i++)
			{
				if(i2o_handlers[i] && 
				   i2o_handlers[i]->new_dev_notify &&
				   (i2o_handlers[i]->class&d->lct_data.class_id))
				{
					spin_lock(&i2o_dev_lock);
					i2o_handlers[i]->new_dev_notify(c,d);
					spin_unlock(&i2o_dev_lock);
				}
			}
			
			break;
		}
	
		/*
		 * LCT entry for a device has been modified, so update it
		 * internally.
		 */
		case I2O_EVT_IND_EXEC_MODIFIED_LCT:
		{
			struct i2o_device *d;
			i2o_lct_entry *new_lct = (i2o_lct_entry *)&msg[5];

			for(d = c->devices; d; d = d->next)
			{
				if(d->lct_data.tid == new_lct->tid)
				{
					memcpy(&d->lct_data, new_lct, sizeof(i2o_lct_entry));
					break;
				}
			}
			break;
		}
	
		case I2O_EVT_IND_CONFIGURATION_FLAG:
			printk(KERN_WARNING "%s requires user configuration\n", c->name);
			break;
	
		case I2O_EVT_IND_GENERAL_WARNING:
			printk(KERN_WARNING "%s: Warning notification received!"
			       "Check configuration for errors!\n", c->name);
			break;
				
		case I2O_EVT_IND_EVT_MASK_MODIFIED:
			/* Well I guess that was us hey .. */
			break;
					
		default:
			dprintk(KERN_WARNING "%s: No handler for event (0x%08x)\n", c->name, msg[4]);
			break;
		}
	}

	return 0;
}

/*
 * Dynamic LCT update.  This compares the LCT with the currently
 * installed devices to check for device deletions..this needed b/c there
 * is no DELETED_LCT_ENTRY EventIndicator for the Executive class so
 * we can't just have the event handler do this...annoying
 *
 * This is a hole in the spec that will hopefully be fixed someday.
 */
static int i2o_dyn_lct(void *foo)
{
	struct i2o_controller *c = (struct i2o_controller *)foo;
 struct i2o_device *d = NULL;
	struct i2o_device *d1 = NULL;
	int i = 0;
	int found = 0;
	int entries;
	void *tmp;
	char name[16];

	lock_kernel();
	daemonize();
	unlock_kernel();

	sprintf(name, "iop%d_lctd", c->unit);
	strcpy(current->comm, name);	
	
	c->lct_running = 1;

	while(1)
	{
		down_interruptible(&c->lct_sem);
		if(signal_pending(current))
		{
			dprintk(KERN_ERR "%s: LCT thread dead\n", c->name);
			c->lct_running = 0;
			return 0;
		}

		entries = c->dlct->table_size;
		entries -= 3;
		entries /= 9;

		dprintk(KERN_INFO "%s: Dynamic LCT contains %d entries\n", c->name, entries);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "                  c->dlct->change_ind %d  c->lct->change_ind %d\n",
			c->dlct->change_ind, c->dlct->change_ind);
#endif

		if(!entries)
		{
			printk(KERN_INFO "%s: Empty LCT???\n", c->name);
			continue;
		}

		/*
		 * Loop through all the devices on the IOP looking for their
		 * LCT data in the LCT.  We assume that TIDs are not repeated.
		 * as that is the only way to really tell.  It's been confirmed
		 * by the IRTOS vendor(s?) that TIDs are not reused until they 
		 * wrap arround(4096), and I doubt a system will up long enough
		 * to create/delete that many devices.
		 */
		for(d = c->devices; d; )
		{
			found = 0;
			d1 = d->next;
			
			for(i = 0; i < entries; i++) 
			{ 
				if(d->lct_data.tid == c->dlct->lct_entry[i].tid) 
				{ 
					found = 1; 
					break; 
				} 
			} 
			if(!found) 
			{
				dprintk(KERN_INFO "i2o_core: Deleted device!\n"); 
#ifdef IOCTLDEBUG
				printk(KERN_INFO 
				       "i2o_core: Deleted device with class id %d   tid %d\n",
				       d->lct_data.class_id, d->lct_data.tid); 
#endif
				spin_lock(&i2o_dev_lock);
				i2o_delete_device(d); 
				spin_unlock(&i2o_dev_lock);
			} 
			d = d1; 
		}


		/*
		 * Copy new LCT into public LCT
		 *
		 * Possible race if someone is reading LCT while  we are copying 
		 * over it. If this happens, we'll fix it then. but I doubt that
		 * the LCT will get updated often enough or will get read by
		 * a user often enough to worry.
		 */
		if(c->lct->table_size < c->dlct->table_size)
		{
			tmp = c->lct;
			c->lct = kmalloc(c->dlct->table_size<<2, GFP_KERNEL);
			if(!c->lct)
			{
				printk(KERN_ERR "%s: No memory for LCT!\n", c->name);
				c->lct = tmp;
				continue;
			}
			kfree(tmp);
		}
		memcpy(c->lct, c->dlct, c->dlct->table_size<<2);

		/* Check new table for BUS_ADAPTER_PORT class devices, and
		   call scsi init in them. If the device is already init'd
		   in scsi land, it will have no effect.
		*/
		for(d = c->devices; d; d = d->next)
		{
			int tidEntry;
			
			dprintk(KERN_INFO "i2o_core: in lct device with class id %d and tid %d\n",
				d->lct_data.class_id, d->lct_data.tid); 

			if (d->lct_data.class_id == I2O_CLASS_BUS_ADAPTER_PORT)
			{ 
#ifdef IOCTLDEBUG
				printk(KERN_INFO 
				       "is1000: dyn_lct: found I2O_CLASS_BUS_ADAPTER_PORT, tid %d\n",
				       d->lct_data.tid);
#endif
				tidEntry = 
					i2o_scsi_init_wrapper(c, d, iscsi_adapter_priv[c->unit].shpnt);
				if (tidEntry == -1)
				{
#ifdef IOCTLDEBUG
					printk(KERN_INFO "is1000: device already added to tid table\n");
#endif
				}
				else
				{
					i2o_event_register(c, d->lct_data.tid, 
							   sIs1000EventHandler.context, 0,0x80000000);
#ifdef IOCTLDEBUG
					printk(KERN_INFO "is1000: NEW DEVICE added to tid table\n");
#endif
				}
			    
			} 
		}
		/* 
		 * Tell LCT to renotify us next time there is a change
		 */
		i2o_lct_notify(c);

	}
	return 0;
}

/**
 *	i2o_run_queue	-	process pending events on a controller
 *	@c: controller to process
 *
 *	This is called by the bus specific driver layer when an interrupt
 *	or poll of this card interface is desired.
 */
 
void i2o_run_queue(struct i2o_controller *c)
{
	struct i2o_message *m;
	u32 mv;
	u32 *msg;

	/*
	 * Old 960 steppings had a bug in the I2O unit that caused
	 * the queue to appear empty when it wasn't.
	 */
	if((mv=I2O_REPLY_READ32(c))==0xFFFFFFFF)
		mv=I2O_REPLY_READ32(c);

	while(mv!=0xFFFFFFFF)
	{
		struct i2o_handler *i;
		m=(struct i2o_message *)bus_to_virt(mv);
		msg=(u32*)m;

		i=i2o_handlers[m->initiator_context&(MAX_I2O_MODULES-1)];
		if(i && i->reply)
			i->reply(i,c,m);
		else
		{
			printk(KERN_WARNING "I2O: Spurious reply to handler %d\n", 
			       m->initiator_context&(MAX_I2O_MODULES-1));
		}	
	 	i2o_flush_reply(c,mv);
		mb();

		/* That 960 bug again... */	
		if((mv=I2O_REPLY_READ32(c))==0xFFFFFFFF)
			mv=I2O_REPLY_READ32(c);
	}		
}


/**
 *	i2o_get_class_name - 	do i2o class name lookup
 *	@class: class number
 *
 *	Return a descriptive string for an i2o class
 */
 
const char *i2o_get_class_name(int class)
{
	int idx = 17;
	static char *i2o_class_name[] = {
		"Executive",
		"Device Driver Module",
		"Block Device",
		"Tape Device",
		"LAN Interface",
		"WAN Interface",
		"Fibre Channel Port",
		"Fibre Channel Device",
		"SCSI Device",
		"ATE Port",
		"ATE Device",
		"Floppy Controller",
		"Floppy Device",
		"Secondary Bus Port",
		"Peer Transport Agent",
		"Peer Transport",
		"Socket",
		"Unknown"
	};
	
	switch(class&0xFFF)
	{
	case I2O_CLASS_EXECUTIVE:
		idx = 0; break;
	case I2O_CLASS_DDM:
		idx = 1; break;
	case I2O_CLASS_RANDOM_BLOCK_STORAGE:
		idx = 2; break;
	case I2O_CLASS_SEQUENTIAL_STORAGE:
		idx = 3; break;
	case I2O_CLASS_LAN:
		idx = 4; break;
	case I2O_CLASS_WAN:
		idx = 5; break;
	case I2O_CLASS_FIBRE_CHANNEL_PORT:
		idx = 6; break;
	case I2O_CLASS_FIBRE_CHANNEL_PERIPHERAL:
		idx = 7; break;
	case I2O_CLASS_SCSI_PERIPHERAL:
		idx = 8; break;
	case I2O_CLASS_ATE_PORT:
		idx = 9; break;
	case I2O_CLASS_ATE_PERIPHERAL:
		idx = 10; break;
	case I2O_CLASS_FLOPPY_CONTROLLER:
		idx = 11; break;
	case I2O_CLASS_FLOPPY_DEVICE:
		idx = 12; break;
	case I2O_CLASS_BUS_ADAPTER_PORT:
		idx = 13; break;
	case I2O_CLASS_PEER_TRANSPORT_AGENT:
		idx = 14; break;
	case I2O_CLASS_PEER_TRANSPORT:
		idx = 15; break;
	}

	return i2o_class_name[idx];
}


/**
 *	i2o_wait_message	-	obtain an i2o message from the IOP
 *	@c: controller
 *	@why: explanation 
 *
 *	This function waits up to 5 seconds for a message slot to be
 *	available. If no message is available it prints an error message
 *	that is expected to be what the message will be used for (eg
 *	"get_status"). 0xFFFFFFFF is returned on a failure.
 *
 *	On a success the message is returned. This is the physical page
 *	frame offset address from the read port. (See the i2o spec)
 */
 
u32 i2o_wait_message(struct i2o_controller *c, char *why)
{
	long time=jiffies;
	u32 m;
	while((m=I2O_POST_READ32(c))==0xFFFFFFFF)
	{
		if((jiffies-time)>=5*HZ)
		{
			dprintk(KERN_ERR "%s: Timeout waiting for message frame to send %s.\n", 
				c->name, why);
			return 0xFFFFFFFF;
		}
		schedule();
		barrier();
	}
	return m;
}
	
/**
 *	i2o_report_controller_unit - print information about a tid
 *	@c: controller
 *	@d: device
 *	
 *	Dump an information block associated with a given unit (TID). The
 *	tables are read and a block of text is output to printk that is
 *	formatted intended for the user.
 */
 
void i2o_report_controller_unit(struct i2o_controller *c, struct i2o_device *d)
{
	char buf[64];
	char str[22];
	int ret;
	int unit = d->lct_data.tid;

	if(verbose==0)
		return;
		
	printk(KERN_INFO "Target ID %d.\n", unit);
	if((ret=i2o_query_scalar(c, unit, 0xF100, 3, buf, 16))>=0)
	{
		buf[16]=0;
		printk(KERN_INFO "     Vendor: %s\n", buf);
	}
	if((ret=i2o_query_scalar(c, unit, 0xF100, 4, buf, 16))>=0)
	{
		buf[16]=0;
		printk(KERN_INFO "     Device: %s\n", buf);
	}
	if(i2o_query_scalar(c, unit, 0xF100, 5, buf, 16)>=0)
	{
		buf[16]=0;
		printk(KERN_INFO "     Description: %s\n", buf);
	}
	if((ret=i2o_query_scalar(c, unit, 0xF100, 6, buf, 8))>=0)
	{
		buf[8]=0;
		printk(KERN_INFO "        Rev: %s\n", buf);
	}

	printk(KERN_INFO "    Class: ");
	sprintf(str, "%-21s", i2o_get_class_name(d->lct_data.class_id));
	printk("%s\n", str);
		
	printk(KERN_INFO "  Subclass: 0x%04X\n", d->lct_data.sub_class);
	printk(KERN_INFO "     Flags: ");
		
	if(d->lct_data.device_flags&(1<<0))
		printk("C");		// ConfigDialog requested
	if(d->lct_data.device_flags&(1<<1))
		printk("U");		// Multi-user capable
	if(!(d->lct_data.device_flags&(1<<4)))
		printk("P");		// Peer service enabled!
	if(!(d->lct_data.device_flags&(1<<5)))
		printk("M");		// Mgmt service enabled!
	printk("\n");
			
}


/*
 *	Parse the hardware resource table. Right now we print it out
 *	and don't do a lot with it. We should collate these and then
 *	interact with the Linux resource allocation block.
 *
 *	Lets prove we can read it first eh ?
 *
 *	This is full of endianisms!
 */
 
static int i2o_parse_hrt(struct i2o_controller *c)
{
#ifdef DRIVERDEBUG
	u32 *rows=(u32*)c->hrt;
	u8 *p=(u8 *)c->hrt;
	u8 *d;
	int count;
	int length;
	int i;
	int state;
	
	if(p[3]!=0)
	{
		printk(KERN_ERR "%s: HRT table for controller is too new a version.\n",
		       c->name);
		return -1;
	}
		
	count=p[0]|(p[1]<<8);
	length = p[2];
	
	printk(KERN_INFO "%s: HRT has %d entries of %d bytes each.\n",
	       c->name, count, length<<2);

	rows+=2;
	
	for(i=0;i<count;i++)
	{
		printk(KERN_INFO "Adapter %08X: ", rows[0]);
		p=(u8 *)(rows+1);
		d=(u8 *)(rows+2);
		state=p[1]<<8|p[0];
		
		printk("TID %04X:[", state&0xFFF);
		state>>=12;
		if(state&(1<<0))
			printk("H");		/* Hidden */
		if(state&(1<<2))
		{
			printk("P");		/* Present */
			if(state&(1<<1))
				printk("C");	/* Controlled */
		}
		if(state>9)
			printk("*");		/* Hard */
		
		printk("]:");
		
		switch(p[3]&0xFFFF)
		{
		case 0:
			/* Adapter private bus - easy */
			printk("Local bus %d: I/O at 0x%04X Mem 0x%08X", 
			       p[2], d[1]<<8|d[0], *(u32 *)(d+4));
			break;
		case 1:
			/* ISA bus */
			printk("ISA %d: CSN %d I/O at 0x%04X Mem 0x%08X",
			       p[2], d[2], d[1]<<8|d[0], *(u32 *)(d+4));
			break;
					
		case 2: /* EISA bus */
			printk("EISA %d: Slot %d I/O at 0x%04X Mem 0x%08X",
			       p[2], d[3], d[1]<<8|d[0], *(u32 *)(d+4));
			break;

		case 3: /* MCA bus */
			printk("MCA %d: Slot %d I/O at 0x%04X Mem 0x%08X",
			       p[2], d[3], d[1]<<8|d[0], *(u32 *)(d+4));
			break;

		case 4: /* PCI bus */
			printk("PCI %d: Bus %d Device %d Function %d",
			       p[2], d[2], d[1], d[0]);
			break;

		case 0x80: /* Other */
		default:
			printk("Unsupported bus type.");
			break;
		}
		printk("\n");
		rows+=length;
	}
#endif
	return 0;
}
	
/*
 *	The logical configuration table tells us what we can talk to
 *	on the board. Most of the stuff isn't interesting to us. 
 */

static int i2o_parse_lct(struct i2o_controller *c)
{
	int i;
	int max;
	int tid;
	struct i2o_device *d;
	i2o_lct *lct = c->lct;

	if (lct == NULL) {
		printk(KERN_ERR "%s: LCT is empty???\n", c->name);
		return -1;
	}

	max = lct->table_size;
	max -= 3;
	max /= 9;
	
	dprintk(KERN_INFO "%s: LCT has %d entries.\n", c->name, max);
	
	if(lct->iop_flags&(1<<0))
#ifdef IOCTLDEBUG
		printk(KERN_WARNING "%s: Configuration dialog desired.\n", c->name);
#endif
		
	for(i=0;i<max;i++)
	{
		d = (struct i2o_device *)kmalloc(sizeof(struct i2o_device), GFP_KERNEL);
		if(d==NULL)
		{
			printk(KERN_CRIT "i2o_core: Out of memory for I2O device data.\n");
			return -ENOMEM;
		}
		
		d->controller = c;
		d->next = NULL;

		memcpy(&d->lct_data, &lct->lct_entry[i], sizeof(i2o_lct_entry));

		d->flags = 0;
		tid = d->lct_data.tid;
		
		i2o_report_controller_unit(c, d);
		
#ifdef IOCTLDEBUG
		printk("is1000: i2o class: i2o_install_device():1:\n");
#endif
		i2o_install_device(c, d);

	}
	return 0;
}


/**
 *	i2o_quiesce_controller - quiesce controller
 *	@c: controller 
 *
 *	Quiesce an IOP. Causes IOP to make external operation quiescent
 *	(i2o 'READY' state). Internal operation of the IOP continues normally.
 */
 
int i2o_quiesce_controller(struct i2o_controller *c)
{
	u32 msg[4];
	int ret;

	i2o_status_get(c);

	/* SysQuiesce discarded if IOP not in READY or OPERATIONAL state */

	if ((c->status_block->iop_state != ADAPTER_STATE_READY) &&
	    (c->status_block->iop_state != ADAPTER_STATE_OPERATIONAL))
	{
		return 0;
	}

	msg[0] = FOUR_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1] = I2O_CMD_SYS_QUIESCE<<24|HOST_TID<<12|ADAPTER_TID;
	msg[3] = 0;

	/* Long timeout needed for quiesce if lots of devices */

	if ((ret = i2o_post_wait(c, msg, sizeof(msg), 240)))
		printk(KERN_INFO "%s: Unable to quiesce (status=%#x).\n",
		       c->name, -ret);
	else
		dprintk(KERN_INFO "%s: Quiesced.\n", c->name);

	i2o_status_get(c); // Entered READY state
	return ret;
}

/**
 *	i2o_enable_controller - move controller from ready to operational
 *	@c: controller
 *
 *	Enable IOP. This allows the IOP to resume external operations and
 *	reverses the effect of a quiesce. In the event of an error a negative
 *	errno code is returned.
 */
 
int i2o_enable_controller(struct i2o_controller *c)
{
	u32 msg[4];
	int ret;

	i2o_status_get(c);
	
	/* Enable only allowed on READY state */	
	if(c->status_block->iop_state != ADAPTER_STATE_READY)
		return -EINVAL;

	msg[0]=FOUR_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1]=I2O_CMD_SYS_ENABLE<<24|HOST_TID<<12|ADAPTER_TID;

	/* How long of a timeout do we need? */

	if ((ret = i2o_post_wait(c, msg, sizeof(msg), 240)))
		printk(KERN_ERR "%s: Could not enable (status=%#x).\n",
		       c->name, -ret);
	else
		dprintk(KERN_INFO "%s: Enabled.\n", c->name);

	i2o_status_get(c); // entered OPERATIONAL state

	return ret;
}

/**
 *	i2o_clear_controller	-	clear a controller
 *	@c: controller
 *
 *	Clear an IOP to HOLD state, ie. terminate external operations, clear all
 *	input queues and prepare for a system restart. IOP's internal operation
 *	continues normally and the outbound queue is alive.
 *	The IOP is not expected to rebuild its LCT.
 */
 
int i2o_clear_controller(struct i2o_controller *c)
{
	struct i2o_controller *iop;
	u32 msg[4];
	int ret;

	/* Quiesce all IOPs first */

	for (iop = i2o_controller_chain; iop; iop = iop->next)
		i2o_quiesce_controller(iop);

	msg[0]=FOUR_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1]=I2O_CMD_ADAPTER_CLEAR<<24|HOST_TID<<12|ADAPTER_TID;
	msg[3]=0;

	if ((ret=i2o_post_wait(c, msg, sizeof(msg), 30)))
		printk(KERN_INFO "%s: Unable to clear (status=%#x).\n",
		       c->name, -ret);
	else
		dprintk(KERN_INFO "%s: Cleared.\n",c->name);

	i2o_status_get(c);

	/* Enable other IOPs */

	for (iop = i2o_controller_chain; iop; iop = iop->next)
		if (iop != c)
			i2o_enable_controller(iop);

	return ret;
}


/**
 *	i2o_reset_controller	-	reset an IOP
 *	@c: controller to reset
 *
 *	Reset the IOP into INIT state and wait until IOP gets into RESET state.
 *	Terminate all external operations, clear IOP's inbound and outbound
 *	queues, terminate all DDMs, and reload the IOP's operating environment
 *	and all local DDMs. The IOP rebuilds its LCT.
 */
 
static int i2o_reset_controller(struct i2o_controller *c)
{
	u32 m;
	u8 *status;
	u32 *msg;
	long time;

	/* Quiesce all IOPs first */

	/* Quiesce only this controller, not all in host */
	i2o_quiesce_controller(c);

	m=i2o_wait_message(c, "AdapterReset");
	if(m==0xFFFFFFFF)	
		return -ETIMEDOUT;
	msg=(u32 *)(c->mem_offset+m);
	
	status=(void *)kmalloc(4, GFP_KERNEL);
	if(status==NULL) {
		printk(KERN_ERR "IOP reset failed - no free memory.\n");
		return -ENOMEM;
	}
	memset(status, 0, 4);
	
	msg[0]=EIGHT_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1]=I2O_CMD_ADAPTER_RESET<<24|HOST_TID<<12|ADAPTER_TID;
	msg[2]=core_context;
	msg[3]=0;
	msg[4]=0;
	msg[5]=0;
	msg[6]=virt_to_bus(status);
	msg[7]=0;	/* 64bit host FIXME */

	i2o_post_message(c,m);

	/* Wait for a reply */
	time=jiffies;
	while(*status==0)
	{
		if((jiffies-time)>=20*HZ)
		{
			printk(KERN_ERR "IOP reset timeout.\n");
			// Better to leak this for safety: kfree(status);
			return -ETIMEDOUT;
		}
		schedule();
		barrier();
	}

	if (*status==I2O_CMD_IN_PROGRESS)
	{ 
		/* 
		 * Once the reset is sent, the IOP goes into the INIT state 
		 * which is indeterminate.  We need to wait until the IOP 
		 * has rebooted before we can let the system talk to 
		 * it. We read the inbound Free_List until a message is 
		 * available.  If we can't read one in the given ammount of 
		 * time, we assume the IOP could not reboot properly.  
		 */ 

		dprintk(KERN_INFO "%s: Reset in progress, waiting for reboot...\n",
			c->name); 

		time = jiffies; 
		m = I2O_POST_READ32(c); 
		while(m == 0XFFFFFFFF) 
		{ 
			if((jiffies-time) >= 30*HZ)
			{
				printk(KERN_ERR "%s: Timeout waiting for IOP reset.\n", 
				       c->name); 
				return -ETIMEDOUT; 
			} 
			schedule(); 
			barrier(); 
			m = I2O_POST_READ32(c); 
		}
		i2o_util_nop(c,m);
	}

	/* If IopReset was rejected or didn't perform reset, try IopClear */

	i2o_status_get(c);
	if (status[0] == I2O_CMD_REJECTED || 
	    c->status_block->iop_state != ADAPTER_STATE_RESET)
	{
		printk(KERN_WARNING "%s: Reset rejected, trying to clear\n",c->name);
		i2o_clear_controller(c);
	}
	else
		dprintk(KERN_INFO "%s: Reset completed.\n", c->name);

	kfree(status);
	return 0;
}


/**
 * 	i2o_status_get	-	get the status block for the IOP
 *	@c: controller
 *
 *	Issue a status query on the controller. This updates the
 *	attached status_block. If the controller fails to reply or an
 *	error occurs then a negative errno code is returned. On success
 *	zero is returned and the status_blok is updated.
 */
 
int i2o_status_get(struct i2o_controller *c)
{
	long time;
	u32 m;
	u32 *msg;
	u8 *status_block;

	if (c->status_block == NULL) 
	{
		c->status_block = (i2o_status_block *)
			kmalloc(sizeof(i2o_status_block),GFP_KERNEL);
		if (c->status_block == NULL)
		{
			printk(KERN_CRIT "%s: Get Status Block failed; Out of memory.\n",
			       c->name);
			return -ENOMEM;
		}
	}

	status_block = (u8*)c->status_block;
	memset(c->status_block,0,sizeof(i2o_status_block));
	
	m=i2o_wait_message(c, "StatusGet");
	if(m==0xFFFFFFFF)
		return -ETIMEDOUT;	
	msg=(u32 *)(c->mem_offset+m);

	msg[0]=NINE_WORD_MSG_SIZE|SGL_OFFSET_0;
	msg[1]=I2O_CMD_STATUS_GET<<24|HOST_TID<<12|ADAPTER_TID;
	msg[2]=core_context;
	msg[3]=0;
	msg[4]=0;
	msg[5]=0;
	msg[6]=virt_to_bus(c->status_block);
	msg[7]=0;   /* 64bit host FIXME */
	msg[8]=sizeof(i2o_status_block); /* always 88 bytes */

	i2o_post_message(c,m);

	/* Wait for a reply */

	time=jiffies;
	while(status_block[87]!=0xFF)
	{
		if((jiffies-time)>=5*HZ)
		{
			printk(KERN_ERR "%s: Get status timeout.\n",c->name);
			return -ETIMEDOUT;
		}
		schedule();
		barrier();
	}

#ifdef DRIVERDEBUG
	printk(KERN_INFO "%s: State = ", c->name);
	switch (c->status_block->iop_state) {
	case 0x01:  
		printk("INIT\n");
		break;
	case 0x02:
		printk("RESET\n");
		break;
	case 0x04:
		printk("HOLD\n");
		break;
	case 0x05:
		printk("READY\n");
		break;
	case 0x08:
		printk("OPERATIONAL\n");
		break;
	case 0x10:
		printk("FAILED\n");
		break;
	case 0x11:
		printk("FAULTED\n");
		break;
	default: 
		printk("%x (unknown !!)\n",c->status_block->iop_state);
	}     
#endif   

	return 0;
}

/*
 * Get the Hardware Resource Table for the device.
 * The HRT contains information about possible hidden devices
 * but is mostly useless to us 
 */
int i2o_hrt_get(struct i2o_controller *c)
{
	u32 msg[6];
	int ret, size = sizeof(i2o_hrt);

	/* First read just the header to figure out the real size */

	do  {
		if (c->hrt == NULL) {
			c->hrt=kmalloc(size, GFP_KERNEL);
			if (c->hrt == NULL) {
				printk(KERN_CRIT "%s: Hrt Get failed; Out of memory.\n", c->name);
				return -ENOMEM;
			}
		}

		msg[0]= SIX_WORD_MSG_SIZE| SGL_OFFSET_4;
		msg[1]= I2O_CMD_HRT_GET<<24 | HOST_TID<<12 | ADAPTER_TID;
		msg[3]= 0;
		msg[4]= (0xD0000000 | size);	/* Simple transaction */
		msg[5]= virt_to_bus(c->hrt);	/* Dump it here */

		ret = i2o_post_wait_mem(c, msg, sizeof(msg), 20, c->hrt, NULL);
		
		if(ret == -ETIMEDOUT)
		{
			/* The HRT block we used is in limbo somewhere. When the iop wakes up
			   we will recover it */
			c->hrt = NULL;
			return ret;
		}
		
		if(ret<0)
		{
			printk(KERN_ERR "%s: Unable to get HRT (status=%#x)\n",
			       c->name, -ret);	
			return ret;
		}

		if (c->hrt->num_entries * c->hrt->entry_len << 2 > size) {
			size = c->hrt->num_entries * c->hrt->entry_len << 2;
			kfree(c->hrt);
			c->hrt = NULL;
		}
	} while (c->hrt == NULL);

	i2o_parse_hrt(c); // just for debugging

	return 0;
}

/*
 * Send the I2O System Table to the specified IOP
 *
 * The system table contains information about all the IOPs in the
 * system.  It is build and then sent to each IOP so that IOPs can
 * establish connections between each other.
 *
 */
static int i2o_systab_send(struct i2o_controller *iop)
{
	u32 msg[12];
	int ret;
	u32 *privbuf = kmalloc(16, GFP_KERNEL);
	if(privbuf == NULL)
		return -ENOMEM;
	
	memset(privbuf, 0, 16);

	if(iop->type == I2O_TYPE_PCI)
	{
		struct resource *root;
		
		if(iop->status_block->current_mem_size < iop->status_block->desired_mem_size)
		{
			struct resource *res = &iop->mem_resource;
#ifdef RH90
			res->name = iop->pdev->bus->name;
#else
			res->name = iop->bus.pci.pdev->bus->name;
#endif
			res->flags = IORESOURCE_MEM;
			res->start = 0;
			res->end = 0;
#ifdef IOCTLDEBUG
			printk("%s: requires private memory resources.\n", iop->name);
#endif

#ifdef RH90
			root = pci_find_parent_resource(iop->pdev, res);
#else
			root = pci_find_parent_resource(iop->bus.pci.pdev, res);
#endif
			if(root==NULL)
				printk("Can't find parent resource!\n");
			if(root && allocate_resource(root, res, 
						     iop->status_block->desired_mem_size,
						     iop->status_block->desired_mem_size,
						     iop->status_block->desired_mem_size,
						     1<<20,	/* Unspecified, so use 1Mb and play safe */
						     NULL,
						     NULL)>=0)
			{
				iop->mem_alloc = 1;
				iop->status_block->current_mem_size = 1 + res->end - res->start;
				iop->status_block->current_mem_base = res->start;
				printk(KERN_INFO "%s: allocated %ld bytes of PCI memory at 0x%08lX.\n", 
				       iop->name, 1+res->end-res->start, res->start);
			}
		}

		if(iop->status_block->current_io_size < iop->status_block->desired_io_size)
		{
			struct resource *res = &iop->io_resource;
#ifdef RH90
			res->name = iop->pdev->bus->name;
#else
			res->name = iop->bus.pci.pdev->bus->name;
#endif
			res->flags = IORESOURCE_IO;
			res->start = 0;
			res->end = 0;

#ifdef IOCTLDEBUG
			printk("%s: requires private memory resources.\n", iop->name);
#endif
#ifdef RH90
			root = pci_find_parent_resource(iop->pdev, res);
#else
			root = pci_find_parent_resource(iop->bus.pci.pdev, res);
#endif
			if(root==NULL)
				printk("Can't find parent resource!\n");
			if(root &&  allocate_resource(root, res, 
						      iop->status_block->desired_io_size,
						      iop->status_block->desired_io_size,
						      iop->status_block->desired_io_size,
						      1<<20,	/* Unspecified, so use 1Mb and play safe */
						      NULL,
						      NULL)>=0)
			{
				iop->io_alloc = 1;
				iop->status_block->current_io_size = 1 + res->end - res->start;
				iop->status_block->current_io_base = res->start;
				printk(KERN_INFO "%s: allocated %ld bytes of PCI I/O at 0x%08lX.\n", 
				       iop->name, 1+res->end-res->start, res->start);
			}
		}
		/*
		 * FIXME: Do we need to store current_*_base and current_*_size
		 * in privbuf for this case, so they get sent to the iop in
		 * this systab setup message?  eab
		 *
		 * NOTE: For Intel PRO 1000 T IP adapter we never need additional
		 * memory or i/o space, so never get HERE. However, correct
		 * behavior in the general case is always desired.
		 */
	}
	else
	{	
		privbuf[0] = iop->status_block->current_mem_base;
		privbuf[1] = iop->status_block->current_mem_size;
		privbuf[2] = iop->status_block->current_io_base;
		privbuf[3] = iop->status_block->current_io_size;
	}

	msg[0] = I2O_MESSAGE_SIZE(12) | SGL_OFFSET_6;
	msg[1] = I2O_CMD_SYS_TAB_SET<<24 | HOST_TID<<12 | ADAPTER_TID;
	msg[3] = 0;
	msg[4] = (0<<16) | ((iop->unit+2) );      /* Host 0 IOP ID (unit + 2) */
	msg[5] = 0;                               /* Segment 0 */

	/* 
 	 * Provide three SGL-elements:
 	 * System table (SysTab), Private memory space declaration and 
 	 * Private i/o space declaration  
 	 *
 	 * FIXME: provide these for controllers needing them
 	 */
	msg[6] = 0x54000000 | sys_tbl_len;
	msg[7] = virt_to_bus(sys_tbl);

#ifdef RH90
	msg[8] = 0x54000000 | privbuf[1];
	msg[9] = privbuf[0];
	msg[10] = 0xD4000000 | privbuf[3];
	msg[11] = privbuf[2];
#else
	msg[8] = 0x54000000 | 8;
	msg[9] = virt_to_bus(privbuf);
	msg[10] = 0xD4000000 | 8;
	msg[11] = virt_to_bus(privbuf+2);
#endif

	ret=i2o_post_wait_mem(iop, msg, sizeof(msg), 120, privbuf, NULL);
	
	if(ret==-ETIMEDOUT)
	{
		printk(KERN_ERR "%s: SysTab setup timed out.\n", iop->name);
		/* privbuf gets freed when response arrives */
	}
	else if(ret<0)
	{
		printk(KERN_ERR "%s: Unable to set SysTab (status=%#x).\n", 
		       iop->name, -ret);
		kfree(privbuf);
	}
	else
	{
		dprintk(KERN_INFO "%s: SysTab set.\n", iop->name);
		kfree(privbuf);
	}
	i2o_status_get(iop); // Entered READY state

	return ret;	

}

/*
 * Initialize I2O subsystem.
 */
static void __init is1000_sys_init(void)
{
	struct i2o_controller *iop, *niop = NULL;

	//	printk(KERN_INFO "Activating I2O controllers...\n");
	//	printk(KERN_INFO "This may take a few minutes if there are many devices\n");
	
	/* In INIT state, Activate IOPs */
	for (iop = i2o_controller_chain; iop; iop = niop) {
		dprintk(KERN_INFO "Calling i2o_activate_controller for %s...\n", 
			iop->name);
		niop = iop->next;
		if (i2o_activate_controller(iop) < 0)
			i2o_delete_controller(iop);
	}

	/* Active IOPs in HOLD state */

 rebuild_sys_tab:
	if (i2o_controller_chain == NULL)
		return;

	/*
	 * If build_sys_table fails, we kill everything and bail
	 * as we can't init the IOPs w/o a system table
	 */	
	dprintk(KERN_INFO "i2o_core: Calling i2o_build_sys_table...\n");
	if (i2o_build_sys_table() < 0) {
		i2o_sys_shutdown();
		return;
	}

	/* If IOP don't get online, we need to rebuild the System table */
	for (iop = i2o_controller_chain; iop; iop = niop) {
		niop = iop->next;
		dprintk(KERN_INFO "Calling i2o_online_controller for %s...\n", iop->name);
		if (i2o_online_controller(iop) < 0) {
			i2o_delete_controller(iop);	
			goto rebuild_sys_tab;
		}
	}
	
	/* Active IOPs now in OPERATIONAL state */

	/*
	 * Register for status updates from all IOPs
	 */
	for(iop = i2o_controller_chain; iop; iop=iop->next) {

		/* Create a kernel thread to deal with dynamic LCT updates */
		iop->lct_pid = kernel_thread(i2o_dyn_lct, iop, CLONE_SIGHAND);
	
		/* Update change ind on DLCT */
		iop->dlct->change_ind = iop->lct->change_ind;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000_sys_init(): iop->dlct->change_ind  %d    iop->lct->change_ind %d\n",
		       iop->dlct->change_ind, iop->lct->change_ind);
#endif
		/* Start dynamic LCT updates */
		i2o_lct_notify(iop);

		/* Register for all events from IRTOS */
		i2o_event_register(iop, core_context, 0, 0, 0xFFFFFFFF);
	}
}

/**
 *	i2o_sys_shutdown - shutdown I2O system
 *
 *	Bring down each i2o controller and then return. Each controller
 *	is taken through an orderly shutdown
 */
 
static void i2o_sys_shutdown(void)
{
	struct i2o_controller *iop, *niop;

	/* Delete all IOPs from the controller chain */
	/* that will reset all IOPs too */

	for (iop = i2o_controller_chain; iop; iop = niop) {
		niop = iop->next;
		i2o_delete_controller(iop);
	}
}

/**
 *	i2o_activate_controller	-	bring controller up to HOLD
 *	@iop: controller
 *
 *	This function brings an I2O controller into HOLD state. The adapter
 *	is reset if neccessary and then the queues and resource table
 *	are read. -1 is returned on a failure, 0 on success.
 *	
 */
 
int i2o_activate_controller(struct i2o_controller *iop)
{
	/* In INIT state, Wait Inbound Q to initialize (in i2o_status_get) */
	/* In READY state, Get status */

	if (i2o_status_get(iop) < 0) {
		printk(KERN_INFO "Unable to obtain status of %s, "
		       "attempting a reset.\n", iop->name);
		if (i2o_reset_controller(iop) < 0)
			return -1;
	}

	if(iop->status_block->iop_state == ADAPTER_STATE_FAULTED) {
		printk(KERN_CRIT "%s: hardware fault\n", iop->name);
		return -1;
	}

	/* In I2O Version 2.0 the reserved field is used to indicate
	 * lowest and highest version of I2O supported. Lower byte
	 * indicates lowest, higher byte indicates highest version.
	 * Reserved field should be (left at) 0 for <= 1.5 versions.
	 */

	if (iop->status_block->i2o_version > I2OVER15) {
		dprintk(KERN_INFO "%s: Not running vrs. 1.5 of the I2O Specification %s\n",
			iop->name, "\n          but vrs. 1.5 is supported");
	}

	if (iop->status_block->iop_state == ADAPTER_STATE_READY ||
	    iop->status_block->iop_state == ADAPTER_STATE_OPERATIONAL ||
	    iop->status_block->iop_state == ADAPTER_STATE_HOLD ||
	    iop->status_block->iop_state == ADAPTER_STATE_FAILED)
	{
		dprintk(KERN_INFO "%s: Already running, trying to reset...\n",
			iop->name);
		if (i2o_reset_controller(iop) < 0)
			return -1;
	}

	if (i2o_init_outbound_q(iop) < 0)
		return -1;

	if (i2o_post_outbound_messages(iop)) 
		return -1;

	/* In HOLD state */
	
	if (i2o_hrt_get(iop) < 0)
		return -1;

	return 0;
}


/**
 *	i2o_init_outbound_queue	- setup the outbound queue
 *	@c: controller
 *
 *	Clear and (re)initialize IOP's outbound queue. Returns 0 on
 *	success or a negative errno code on a failure.
 */
 
int i2o_init_outbound_q(struct i2o_controller *c)
{
	u8 *status;
	u32 m;
	u32 *msg;
	u32 time;

	dprintk(KERN_INFO "%s: Initializing Outbound Queue...\n", c->name);
	m=i2o_wait_message(c, "OutboundInit");
	if(m==0xFFFFFFFF)
		return -ETIMEDOUT;
	msg=(u32 *)(c->mem_offset+m);

	status = kmalloc(4,GFP_KERNEL);
	if (status==NULL) {
		printk(KERN_ERR "%s: Outbound Queue initialization failed - no free memory.\n",
		       c->name);
		return -ENOMEM;
	}
	memset(status, 0, 4);

	msg[0]= EIGHT_WORD_MSG_SIZE| TRL_OFFSET_6;
	msg[1]= I2O_CMD_OUTBOUND_INIT<<24 | HOST_TID<<12 | ADAPTER_TID;
	msg[2]= core_context;
	msg[3]= 0x0106;				/* Transaction context */
	msg[4]= 4096;				/* Host page frame size */
	msg[5]= MSG_FRAME_SIZE<<16|0x80;	/* Outbound msg frame size and Initcode */
	msg[6]= 0xD0000004;			/* Simple SG LE, EOB */
	msg[7]= virt_to_bus(status);

	i2o_post_message(c,m);
	
	barrier();
	time=jiffies;
	while(status[0] < I2O_CMD_REJECTED)
	{
		if((jiffies-time)>=30*HZ)
		{
			if(status[0]==0x00)
				printk(KERN_ERR "%s: Ignored queue initialize request.\n",
				       c->name);
			else  
				printk(KERN_ERR "%s: Outbound queue initialize timeout.\n",
				       c->name);
			kfree(status);
			return -ETIMEDOUT;
		}  
		schedule();
		barrier();
	}  

	if(status[0] != I2O_CMD_COMPLETED)
	{
		printk(KERN_ERR "%s: IOP outbound initialise failed.\n", c->name);
		kfree(status);
		return -ETIMEDOUT;
	}

	return 0;
}

/**
 *	i2o_post_outbound_messages	-	fill message queue
 *	@c: controller
 *
 *	Allocate a message frame and load the messages into the IOP. The
 *	function returns zero on success or a negative errno code on
 *	failure.
 */

int i2o_post_outbound_messages(struct i2o_controller *c)
{
	int i;
	u32 m;
#if defined(CONFIG_HIGHIO) && defined(IS1000_HIGHMEMIO) && defined(RH90)
	struct page *page;
	unsigned long offset;
#endif

	/* Alloc space for IOP's outbound queue message frames */

	c->page_frame = kmalloc(MSG_POOL_SIZE, GFP_KERNEL);
	if(c->page_frame==NULL) {
		printk(KERN_CRIT "%s: Outbound Q initialize failed; out of memory.\n",
		       c->name);
		return -ENOMEM;
	}

#ifdef RH90
#if defined(CONFIG_HIGHIO) && defined(IS1000_HIGHMEMIO)
	page = virt_to_page(c->page_frame);
	offset = (unsigned long)c->page_frame & ~PAGE_MASK;

	c->page_frame_map = pci_map_page(c->pdev, page, offset, MSG_POOL_SIZE, PCI_DMA_FROMDEVICE);
#else
	c->page_frame_map = pci_map_single(c->pdev, c->page_frame, MSG_POOL_SIZE, PCI_DMA_FROMDEVICE);
#endif

	if(c->page_frame_map == 0)
	{
		kfree(c->page_frame);
		printk(KERN_ERR "%s: Unable to map outbound queue.\n", c->name);
		return -ENOMEM;
	}

	m = c->page_frame_map;
#else
	m=virt_to_bus(c->page_frame);
#endif

	/* Post frames */

	for(i=0; i< NMBR_MSG_FRAMES; i++) {
		I2O_REPLY_WRITE32(c,m);
		mb();

		//printk(KERN_ERR "is1000: i %d  m 0x%08x\n", i, m);
		// m is u32 - not a word ptr, so both cases use in bytes
		m += MSG_FRAME_SIZE << 2;
	}

	return 0;
}

/*
 * Get the IOP's Logical Configuration Table
 */
int i2o_lct_get(struct i2o_controller *c)
{
	u32 msg[8];
	int ret, size = c->status_block->expected_lct_size;

	do {
		if (c->lct == NULL) {
			c->lct = kmalloc(size, GFP_KERNEL);
			if(c->lct == NULL) {
				printk(KERN_CRIT "%s: Lct Get failed. Out of memory.\n",
				       c->name);
				return -ENOMEM;
			}
		}
		memset(c->lct, 0, size);

		msg[0] = EIGHT_WORD_MSG_SIZE|SGL_OFFSET_6;
		msg[1] = I2O_CMD_LCT_NOTIFY<<24 | HOST_TID<<12 | ADAPTER_TID;
		/* msg[2] filled in i2o_post_wait */
		msg[3] = 0;
		msg[4] = 0xFFFFFFFF;	/* All devices */
		msg[5] = 0x00000000;	/* Report now */
		msg[6] = 0xD0000000|size;
		msg[7] = virt_to_bus(c->lct);

		ret=i2o_post_wait_mem(c, msg, sizeof(msg), 120, c->lct, NULL);
		
		if(ret == -ETIMEDOUT)
		{
			c->lct = NULL;
			return ret;
		}
		
		if(ret<0)
		{
			printk(KERN_ERR "%s: LCT Get failed (status=%#x.\n", 
			       c->name, -ret);	
			return ret;
		}

		if (c->lct->table_size << 2 > size) {
			size = c->lct->table_size << 2;
			kfree(c->lct);
			c->lct = NULL;
		}
	} while (c->lct == NULL);

        if ((ret=i2o_parse_lct(c)) < 0)
                return ret;

	return 0;
}

/*
 * Like above, but used for async notification.  The main
 * difference is that we keep track of the CurrentChangeIndiicator
 * so that we only get updates when it actually changes.
 *
 */
int i2o_lct_notify(struct i2o_controller *c)
{
	u32 msg[8];

	msg[0] = EIGHT_WORD_MSG_SIZE|SGL_OFFSET_6;
	msg[1] = I2O_CMD_LCT_NOTIFY<<24 | HOST_TID<<12 | ADAPTER_TID;
	msg[2] = core_context;
	msg[3] = 0xDEADBEEF;	
	msg[4] = 0xFFFFFFFF;	/* All devices */

#ifdef IOCTLDEBUG
	printk(KERN_INFO "i2o_lct_notify(): ->change_ind %d\n",
	       c->dlct->change_ind);
#endif
	msg[5] = c->dlct->change_ind; /* Not incrementing anymore 10/09/02 eab */

	msg[6] = 0xD0000000|8192;
	msg[7] = virt_to_bus(c->dlct);

	return i2o_post_this(c, msg, sizeof(msg));
}
		
/*
 *	Bring a controller online into OPERATIONAL state. 
 */
 
int i2o_online_controller(struct i2o_controller *iop)
{
	u32 v;
	
	if (i2o_systab_send(iop) < 0)
		return -1;

	/* In READY state */

	dprintk(KERN_INFO "%s: Attempting to enable...\n", iop->name);
	if (i2o_enable_controller(iop) < 0)
		return -1;

	/* In OPERATIONAL state  */

	dprintk(KERN_INFO "%s: Attempting to get/parse lct...\n", iop->name);
	if (i2o_lct_get(iop) < 0)
		return -1;

	/* Check battery status */
	 
	iop->battery = 0;
	if(i2o_query_scalar(iop, ADAPTER_TID, 0x0000, 4, &v, 4)>=0)
	{
		if(v&16)
			iop->battery = 1;
	}

	return 0;
}

/*
 * Build system table
 *
 * The system table contains information about all the IOPs in the
 * system (duh) and is used by the Executives on the IOPs to establish
 * peer2peer connections.  We're not supporting peer2peer at the moment,
 * but this will be needed down the road for things like lan2lan forwarding.
 */
static int i2o_build_sys_table(void)
{
	struct i2o_controller *iop = NULL;
	struct i2o_controller *niop = NULL;
	int count = 0;

	sys_tbl_len = sizeof(struct i2o_sys_tbl) +	// Header + IOPs
		(i2o_num_controllers) *
		sizeof(struct i2o_sys_tbl_entry);

	if(sys_tbl)
		kfree(sys_tbl);

	sys_tbl = kmalloc(sys_tbl_len, GFP_KERNEL);
	if(!sys_tbl) {
		printk(KERN_CRIT "SysTab Set failed. Out of memory.\n");
		return -ENOMEM;
	}
	memset((void*)sys_tbl, 0, sys_tbl_len);

	sys_tbl->num_entries = i2o_num_controllers;
	sys_tbl->version = I2OVERSION; /* TODO: Version 2.0 */
	sys_tbl->change_ind = sys_tbl_ind++;

	for(iop = i2o_controller_chain; iop; iop = niop)
	{
		niop = iop->next;

		/* 
		 * Get updated IOP state so we have the latest information
		 *
		 * We should delete the controller at this point if it
		 * doesn't respond since  if it's not on the system table 
		 * it is techninically not part of the I2O subsytem...
		 */
		if(i2o_status_get(iop)) {
			printk(KERN_ERR "%s: Deleting b/c could not get status while"
			       "attempting to build system table\n", iop->name);
			i2o_delete_controller(iop);		
			sys_tbl->num_entries--;
			continue; // try the next one
		}

		sys_tbl->iops[count].org_id = iop->status_block->org_id;
		sys_tbl->iops[count].iop_id = iop->unit + 2;
		sys_tbl->iops[count].seg_num = 0;
		sys_tbl->iops[count].i2o_version = 
			iop->status_block->i2o_version;
		sys_tbl->iops[count].iop_state = 
			iop->status_block->iop_state;
		sys_tbl->iops[count].msg_type = 
			iop->status_block->msg_type;
		sys_tbl->iops[count].frame_size = 
			iop->status_block->inbound_frame_size;
		sys_tbl->iops[count].last_changed = sys_tbl_ind - 1; // ??
		sys_tbl->iops[count].iop_capabilities = 
			iop->status_block->iop_capabilities;
		sys_tbl->iops[count].inbound_low = 
#ifdef RH90
			iop->post_port;
#else
		(u32)virt_to_bus(iop->post_port);
#endif
		sys_tbl->iops[count].inbound_high = 0;	// TODO: 64-bit support

		count++;
	}

#ifdef DRIVERDEBUG
	{
		u32 *table;
		table = (u32*)sys_tbl;
		for(count = 0; count < (sys_tbl_len >>2); count++)
			printk(KERN_INFO "sys_tbl[%d] = %0#10x\n", count, table[count]);
	}
#endif

	return 0;
}


/*
 *	Run time support routines
 */
 
/*
 *	Generic "post and forget" helpers. This is less efficient - we do
 *	a memcpy for example that isnt strictly needed, but for most uses
 *	this is simply not worth optimising
 */

int i2o_post_this(struct i2o_controller *c, u32 *data, int len)
{
	u32 m;
	u32 *msg;
	unsigned long t=jiffies;

	do
	{
		mb();
		m = I2O_POST_READ32(c);
	}
	while(m==0xFFFFFFFF && (jiffies-t)<HZ);
	
	if(m==0xFFFFFFFF)
	{
		printk(KERN_ERR "%s: Timeout waiting for message frame!\n",
		       c->name);
		return -ETIMEDOUT;
	}
	msg = (u32 *)(c->mem_offset + m);
 	memcpy_toio(msg, data, len);
	i2o_post_message(c,m);
	return 0;
}

/**
 * 	i2o_post_wait_mem	-	I2O query/reply with DMA buffers
 *	@c: controller
 *	@msg: message to send
 *	@len: length of message
 *	@timeout: time in seconds to wait
 *	@mem1: attached memory buffer 1
 *	@mem2: attached memory buffer 2
 *
 * 	This core API allows an OSM to post a message and then be told whether
 *	or not the system received a successful reply. 
 *
 *	If the message times out then the value '-ETIMEDOUT' is returned. This
 *	is a special case. In this situation the message may (should) complete
 *	at an indefinite time in the future. When it completes it will use the
 *	memory buffers attached to the request. If -ETIMEDOUT is returned then
 *	the memory buffers must not be freed. Instead the event completion will
 *	free them for you. In all other cases the buffers are your problem.
 *
 *	Pass NULL for unneeded buffers.
 */
 
int i2o_post_wait_mem(struct i2o_controller *c, u32 *msg, int len, int timeout, void *mem1, void *mem2)
{
	DECLARE_WAIT_QUEUE_HEAD(wq_i2o_post);
#ifdef RH90
	DECLARE_WAITQUEUE(wait, current);
#endif
	int complete = 0;
	int status;
	unsigned long flags = 0;
	struct i2o_post_wait_data *wait_data =
		kmalloc(sizeof(struct i2o_post_wait_data), GFP_KERNEL);

	if(!wait_data)
		return -ENOMEM;

	/*
	 *	Create a new notification object
	 */
	wait_data->status = &status;
	wait_data->complete = &complete;
	wait_data->mem[0] = mem1;
	wait_data->mem[1] = mem2;
	/* 
	 *	Queue the event with its unique id
	 */
	spin_lock_irqsave(&post_wait_lock, flags);

	wait_data->next = post_wait_queue;
	post_wait_queue = wait_data;
	wait_data->id = (++post_wait_id) & 0x7fff;
	wait_data->wq = &wq_i2o_post;

	spin_unlock_irqrestore(&post_wait_lock, flags);

	/*
	 *	Fill in the message id
	 */
	 
	msg[2] = 0x80000000|(u32)core_context|((u32)wait_data->id<<16);
	
	/*
	 *	Post the message to the controller. At some point later it 
	 *	will return. If we time out before it returns then
	 *	complete will be zero.  From the point post_this returns
	 *	the wait_data may have been deleted.
	 */
#ifdef RH90
	add_wait_queue(&wq_i2o_post, &wait);
	set_current_state(TASK_INTERRUPTIBLE);
	if ((status = i2o_post_this(c, msg, len))==0) {
		schedule_timeout(HZ * timeout);
	}  
	else
	{
		remove_wait_queue(&wq_i2o_post, &wait);
		return -EIO;
	}
	remove_wait_queue(&wq_i2o_post, &wait);
#else
	if ((status = i2o_post_this(c, msg, len))==0) {
		sleep_on_timeout(&wq_i2o_post, HZ * timeout);
	}  
	else
		return -EIO;
#endif		
	if(signal_pending(current))
		status = -EINTR;
		
	spin_lock_irqsave(&post_wait_lock, flags);
	barrier();	/* Be sure we see complete as it is locked */
	if(!complete)
	{
		/* 
		 *	Mark the entry dead. We cannot remove it. This is important.
		 *	When it does terminate (which it must do if the controller hasnt
		 *	died..) then it will otherwise scribble on stuff.
		 *	!complete lets us safely check if the entry is still
		 *	allocated and thus we can write into it
		 */
		wait_data->wq = NULL;
		status = -ETIMEDOUT;
	}
	else
	{
		/* Debugging check - remove me soon */
		if(status == -ETIMEDOUT)
		{
			printk("TIMEDOUT BUG!\n");
			status = -EIO;
		}
	}
	/* And the wait_data is not leaked either! */	 
	spin_unlock_irqrestore(&post_wait_lock, flags);
	return status;
}

/**
 * 	i2o_post_wait		-	I2O query/reply
 *	@c: controller
 *	@msg: message to send
 *	@len: length of message
 *	@timeout: time in seconds to wait
 *
 * 	This core API allows an OSM to post a message and then be told whether
 *	or not the system received a successful reply. 
 */
 
int i2o_post_wait(struct i2o_controller *c, u32 *msg, int len, int timeout)
{
	return i2o_post_wait_mem(c, msg, len, timeout, NULL, NULL);
}

/*
 * i2o_post_wait is completed and we want to wake up the 
 * sleeping proccess. Called by core's reply handler.
 */

static void i2o_post_wait_complete(u32 context, int status)
{
	struct i2o_post_wait_data **p1, *q;
	unsigned long flags;
	
	/* 
	 * We need to search through the post_wait 
	 * queue to see if the given message is still
	 * outstanding.  If not, it means that the IOP 
	 * took longer to respond to the message than we 
	 * had allowed and timer has already expired.  
	 * Not much we can do about that except log
	 * it for debug purposes, increase timeout, and recompile
	 *
	 * Lock needed to keep anyone from moving queue pointers 
	 * around while we're looking through them.
	 */

	spin_lock_irqsave(&post_wait_lock, flags);

	for(p1 = &post_wait_queue; *p1!=NULL; p1 = &((*p1)->next)) 
	{
		q = (*p1);
		if(q->id == ((context >> 16) & 0x7fff)) {
			/*
			 *	Delete it 
			 */
			 
			*p1 = q->next;
			
			/*
			 *	Live or dead ?
			 */
			 
			if(q->wq)
			{
				/* Live entry - wakeup and set status */
				*q->status = status;
				*q->complete = 1;
				wake_up(q->wq);
			}
			else
			{
				/*
				 *	Free resources. Caller is dead
				 */
				if(q->mem[0])
					kfree(q->mem[0]);
				if(q->mem[1])
					kfree(q->mem[1]);
				printk(KERN_WARNING "i2o_post_wait event completed after timeout.\n");
			}
			kfree(q);
			spin_unlock(&post_wait_lock);
			return;
		}
	}
	spin_unlock(&post_wait_lock);

	printk(KERN_DEBUG "i2o_post_wait: Bogus reply!\n");
}

/*	Issue UTIL_PARAMS_GET or UTIL_PARAMS_SET
 *
 *	This function can be used for all UtilParamsGet/Set operations.
 *	The OperationList is given in oplist-buffer, 
 *	and results are returned in reslist-buffer.
 *	Note that the minimum sized reslist is 8 bytes and contains
 *	ResultCount, ErrorInfoSize, BlockStatus and BlockSize.
 */
int i2o_issue_params(int cmd, struct i2o_controller *iop, int tid, 
		     void *oplist, int oplen, void *reslist, int reslen)
{
	u32 msg[9]; 
	u32 *res32 = (u32*)reslist;
	u32 *restmp = (u32*)reslist;
	int len = 0;
	int i = 0;
	int wait_status;
	u32 *opmem, *resmem;
	
	/* Get DMAable memory */
	opmem = kmalloc(oplen, GFP_KERNEL);
	if(opmem == NULL)
		return -ENOMEM;
	memcpy(opmem, oplist, oplen);
	
	resmem = kmalloc(reslen, GFP_KERNEL);
	if(resmem == NULL)
	{
		kfree(opmem);
		return -ENOMEM;
	}
	
	msg[0] = NINE_WORD_MSG_SIZE | SGL_OFFSET_5;
	msg[1] = cmd << 24 | HOST_TID << 12 | tid; 
	msg[3] = 0;
	msg[4] = 0;
	msg[5] = 0x54000000 | oplen;	/* OperationList */
	msg[6] = virt_to_bus(opmem);
	msg[7] = 0xD0000000 | reslen;	/* ResultList */
	msg[8] = virt_to_bus(resmem);

	wait_status = i2o_post_wait_mem(iop, msg, sizeof(msg), 280, opmem, resmem);
	
	/*
	 *	This only looks like a memory leak - don't "fix" it.	
	 */
	if(wait_status == -ETIMEDOUT)
		return wait_status;

	/* Query failed */
	if(wait_status != 0)
	{
		kfree(resmem);
		kfree(opmem);
		return wait_status;
	}
	
	memcpy(reslist, resmem, reslen);
	/*
	 * Calculate number of bytes of Result LIST
	 * We need to loop through each Result BLOCK and grab the length
	 */
	restmp = res32 + 1;
	len = 1;
	for(i = 0; i < (res32[0]&0X0000FFFF); i++)
	{
		if(restmp[0]&0x00FF0000)	/* BlockStatus != SUCCESS */
		{
			printk(KERN_WARNING "%s - Error:\n  ErrorInfoSize = 0x%02x, " 
			       "BlockStatus = 0x%02x, BlockSize = 0x%04x\n",
			       (cmd == I2O_CMD_UTIL_PARAMS_SET) ? "PARAMS_SET"
			       : "PARAMS_GET",   
			       res32[1]>>24, (res32[1]>>16)&0xFF, res32[1]&0xFFFF);
	
			/*
			 *	If this is the only request,than we return an error
			 */
			if((res32[0]&0x0000FFFF) == 1)
			{
				return -((res32[1] >> 16) & 0xFF); /* -BlockStatus */
			}
		}
		len += restmp[0] & 0x0000FFFF;	/* Length of res BLOCK */
		restmp += restmp[0] & 0x0000FFFF;	/* Skip to next BLOCK */
	}
	return (len << 2);  /* bytes used by result list */
}

/*
 *	 Query one scalar group value or a whole scalar group.
 */                  	
int i2o_query_scalar(struct i2o_controller *iop, int tid, 
                     int group, int field, void *buf, int buflen)
{
	u16 opblk[] = { 1, 0, I2O_PARAMS_FIELD_GET, group, 1, field };
	u8  resblk[8+((buflen+3)&~3)]; /* 8 bytes for header */
	int size;

	if (field == -1)  		/* whole group */
       		opblk[4] = -1;
              
	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_GET, iop, tid, 
				opblk, sizeof(opblk), resblk, sizeof(resblk));
		
	memcpy(buf, resblk+8, buflen);  /* cut off header */
	
	if(size>buflen)
		return buflen;
	return size;
}

/*
 *	Set a scalar group value or a whole group.
 */
int i2o_set_scalar(struct i2o_controller *iop, int tid, 
		   int group, int field, void *buf, int buflen)
{
	u16 *opblk;
	u8  resblk[8+((buflen+3)&~3)]; /* 8 bytes for header */
        int size;

	opblk = kmalloc(buflen+64, GFP_KERNEL);
	if (opblk == NULL)
	{
		printk(KERN_ERR "i2o: no memory for operation buffer.\n");
		return -ENOMEM;
	}

	opblk[0] = 1;                        /* operation count */
	opblk[1] = 0;                        /* pad */
	opblk[2] = I2O_PARAMS_FIELD_SET;
	opblk[3] = group;

	if(field == -1) {               /* whole group */
		opblk[4] = -1;
		memcpy(opblk+5, buf, buflen);
	}
	else                            /* single field */
	{
		opblk[4] = 1;
		opblk[5] = field;
		memcpy(opblk+6, buf, buflen);
	}   

	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_SET, iop, tid, 
				opblk, 12+buflen, resblk, sizeof(resblk));

	kfree(opblk);
	if(size>buflen)
		return buflen;
	return size;
}

/* 
 * 	if oper == I2O_PARAMS_LIST_SET, get from specific rows
 * 		if fieldcount == -1 return all fields
 *			ibuf contains rowcount, keyvalues
 * 		else return specific fields
 *			fieldcount is # of fieldindexes
 *  			ibuf contains fieldindexes, rowcount, keyvalues
 *
 *	You could also use directly function i2o_issue_params().
 */
int i2o_set_list_table(int oper, struct i2o_controller *iop, int tid, int group,
		       int fieldcount, void *ibuf, int ibuflen,
		       void *resblk, int reslen) 
{
	u16 *opblk;
	int size;

	opblk = kmalloc(10 + ibuflen, GFP_KERNEL);
	if (opblk == NULL)
	{
		printk(KERN_ERR "i2o: no memory for query buffer.\n");
		return -ENOMEM;
	}

	opblk[0] = 1;				/* operation count */
	opblk[1] = 0;				/* pad */
	opblk[2] = I2O_PARAMS_LIST_SET;
	opblk[3] = group;		
	opblk[4] = fieldcount;
	memcpy(opblk+5, ibuf, ibuflen);		/* other params */

	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_SET,iop, tid, 
				opblk, 10+ibuflen, resblk, reslen);

	kfree(opblk);
	if(size>reslen)
		return reslen;
	return size;
}

/* 
 * 	if oper == I2O_PARAMS_TABLE_GET, get from all rows 
 * 		if fieldcount == -1 return all fields
 *			ibuf and ibuflen are unused (use NULL, 0)
 * 		else return specific fields
 *  			ibuf contains fieldindexes
 *
 * 	if oper == I2O_PARAMS_LIST_GET, get from specific rows
 * 		if fieldcount == -1 return all fields
 *			ibuf contains rowcount, keyvalues
 * 		else return specific fields
 *			fieldcount is # of fieldindexes
 *  			ibuf contains fieldindexes, rowcount, keyvalues
 *
 *	You could also use directly function i2o_issue_params().
 */
int i2o_query_table(int oper, struct i2o_controller *iop, int tid, int group,
		    int fieldcount, void *ibuf, int ibuflen,
		    void *resblk, int reslen) 
{
	u16 *opblk;
	int size;

	opblk = kmalloc(10 + ibuflen, GFP_KERNEL);
	if (opblk == NULL)
	{
		printk(KERN_ERR "i2o: no memory for query buffer.\n");
		return -ENOMEM;
	}

	opblk[0] = 1;				/* operation count */
	opblk[1] = 0;				/* pad */
	opblk[2] = oper;
	opblk[3] = group;		
	opblk[4] = fieldcount;
	memcpy(opblk+5, ibuf, ibuflen);		/* other params */

	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_GET,iop, tid, 
				opblk, 10+ibuflen, resblk, reslen);

	kfree(opblk);
	if(size>reslen)
		return reslen;
	return size;
}

/*
 * 	Clear table group, i.e. delete all rows.
 */
int i2o_clear_table(struct i2o_controller *iop, int tid, int group)
{
	u16 opblk[] = { 1, 0, I2O_PARAMS_TABLE_CLEAR, group };
	u8  resblk[32]; /* min 8 bytes for result header */

	return i2o_issue_params(I2O_CMD_UTIL_PARAMS_SET, iop, tid, 
				opblk, sizeof(opblk), resblk, sizeof(resblk));
}

/*
 * 	Add a new row into a table group.
 *
 * 	if fieldcount==-1 then we add whole rows
 *		buf contains rowcount, keyvalues
 * 	else just specific fields are given, rest use defaults
 *  		buf contains fieldindexes, rowcount, keyvalues
 */	
int i2o_row_add_table(struct i2o_controller *iop, int tid,
		      int group, int fieldcount, void *buf, int buflen)
{
	u16 *opblk;
	u8  resblk[32]; /* min 8 bytes for header */
	int size;

	opblk = kmalloc(buflen+64, GFP_KERNEL);
	if (opblk == NULL)
	{
		printk(KERN_ERR "i2o: no memory for operation buffer.\n");
		return -ENOMEM;
	}

	opblk[0] = 1;			/* operation count */
	opblk[1] = 0;			/* pad */
	opblk[2] = I2O_PARAMS_ROW_ADD;
	opblk[3] = group;	
	opblk[4] = fieldcount;
	memcpy(opblk+5, buf, buflen);

	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_SET, iop, tid, 
				opblk, 10+buflen, resblk, sizeof(resblk));

	kfree(opblk);
	if(size>buflen)
		return buflen;
	return size;
}


/*
 * 	Delete a row from a table group.
 *
 * 	if fieldcount==-1 then we delete whole rows
 *		buf contains rowcount, keyvalues
 * 	else just specific fields are given, rest use defaults
 *  		buf contains fieldindexes, rowcount, keyvalues
 */	
int i2o_row_del_table(struct i2o_controller *iop, int tid,
		      int group, int fieldcount, void *buf, int buflen)
{
	u16 *opblk;
	u8  resblk[40]; /* min 8 bytes for header  + ??*/
	int size;

	opblk = kmalloc(buflen+64, GFP_KERNEL);
	if (opblk == NULL)
	{
		printk(KERN_ERR "i2o: no memory for operation buffer.\n");
		return -ENOMEM;
	}

	opblk[0] = 1;			/* operation count */
	opblk[1] = 0;			/* pad */
	opblk[2] = I2O_PARAMS_ROW_DELETE;
	opblk[3] = group;	
	opblk[4] = fieldcount;
	memcpy(opblk+5, buf, buflen);

	size = i2o_issue_params(I2O_CMD_UTIL_PARAMS_SET, iop, tid, 
				opblk, 10+buflen, resblk, sizeof(resblk));

	kfree(opblk);
	if(size>buflen)
		return buflen;
	return size;
}


/*
 * Used for error reporting/debugging purposes.
 * Following fail status are common to all classes.
 * The preserved message must be handled in the reply handler. 
 */
void i2o_report_fail_status(u8 req_status, u32* msg)
{
	static char *FAIL_STATUS[] = { 
		"0x80",				/* not used */
		"SERVICE_SUSPENDED", 		/* 0x81 */
		"SERVICE_TERMINATED", 		/* 0x82 */
		"CONGESTION",
		"FAILURE",
		"STATE_ERROR",
		"TIME_OUT",
		"ROUTING_FAILURE",
		"INVALID_VERSION",
		"INVALID_OFFSET",
		"INVALID_MSG_FLAGS",
		"FRAME_TOO_SMALL",
		"FRAME_TOO_LARGE",
		"INVALID_TARGET_ID",
		"INVALID_INITIATOR_ID",
		"INVALID_INITIATOR_CONTEX",	/* 0x8F */
		"UNKNOWN_FAILURE"		/* 0xFF */
	};

	if (req_status == I2O_FSC_TRANSPORT_UNKNOWN_FAILURE)
		printk("TRANSPORT_UNKNOWN_FAILURE (%0#2x)\n.", req_status);
	else
		printk("TRANSPORT_%s.\n", FAIL_STATUS[req_status & 0x0F]);

	/* Dump some details */

	printk(KERN_ERR "  InitiatorId = %d, TargetId = %d\n",
	       (msg[1] >> 12) & 0xFFF, msg[1] & 0xFFF); 
	printk(KERN_ERR "  LowestVersion = 0x%02X, HighestVersion = 0x%02X\n",
	       (msg[4] >> 8) & 0xFF, msg[4] & 0xFF);
	printk(KERN_ERR "  FailingHostUnit = 0x%04X,  FailingIOP = 0x%03X\n",
	       msg[5] >> 16, msg[5] & 0xFFF);

	printk(KERN_ERR "  Severity:  0x%02X ", (msg[4] >> 16) & 0xFF); 
	if (msg[4] & (1<<16))
		printk("(FormatError), "
		       "this msg can never be delivered/processed.\n");
	if (msg[4] & (1<<17))
		printk("(PathError), "
		       "this msg can no longer be delivered/processed.\n");
	if (msg[4] & (1<<18))
		printk("(PathState), "
		       "the system state does not allow delivery.\n");
	if (msg[4] & (1<<19))
		printk("(Congestion), resources temporarily not available;"
		       "do not retry immediately.\n");
}

/*
 * Used for error reporting/debugging purposes.
 * Following reply status are common to all classes.
 */
void i2o_report_common_status(u8 req_status)
{
	static char *REPLY_STATUS[] = { 
		"SUCCESS", 
		"ABORT_DIRTY", 
		"ABORT_NO_DATA_TRANSFER",
		"ABORT_PARTIAL_TRANSFER",
		"ERROR_DIRTY",
		"ERROR_NO_DATA_TRANSFER",
		"ERROR_PARTIAL_TRANSFER",
		"PROCESS_ABORT_DIRTY",
		"PROCESS_ABORT_NO_DATA_TRANSFER",
		"PROCESS_ABORT_PARTIAL_TRANSFER",
		"TRANSACTION_ERROR",
		"PROGRESS_REPORT"	
	};

	if (req_status > I2O_REPLY_STATUS_PROGRESS_REPORT)
		printk("RequestStatus = %0#2x", req_status);
	else
		printk("%s", REPLY_STATUS[req_status]);
}

/*
 * Used for error reporting/debugging purposes.
 * Following detailed status are valid  for executive class, 
 * utility class, DDM class and for transaction error replies.
 */
static void i2o_report_common_dsc(u16 detailed_status)
{
	static char *COMMON_DSC[] = { 
		"SUCCESS",
		"0x01",				// not used
		"BAD_KEY",
		"TCL_ERROR",
		"REPLY_BUFFER_FULL",
		"NO_SUCH_PAGE",
		"INSUFFICIENT_RESOURCE_SOFT",
		"INSUFFICIENT_RESOURCE_HARD",
		"0x08",				// not used
		"CHAIN_BUFFER_TOO_LARGE",
		"UNSUPPORTED_FUNCTION",
		"DEVICE_LOCKED",
		"DEVICE_RESET",
		"INAPPROPRIATE_FUNCTION",
		"INVALID_INITIATOR_ADDRESS",
		"INVALID_MESSAGE_FLAGS",
		"INVALID_OFFSET",
		"INVALID_PARAMETER",
		"INVALID_REQUEST",
		"INVALID_TARGET_ADDRESS",
		"MESSAGE_TOO_LARGE",
		"MESSAGE_TOO_SMALL",
		"MISSING_PARAMETER",
		"TIMEOUT",
		"UNKNOWN_ERROR",
		"UNKNOWN_FUNCTION",
		"UNSUPPORTED_VERSION",
		"DEVICE_BUSY",
		"DEVICE_NOT_AVAILABLE"		
	};

	if (detailed_status > I2O_DSC_DEVICE_NOT_AVAILABLE)
		printk(" / DetailedStatus = %0#4x.\n", detailed_status);
	else
		printk(" / %s.\n", COMMON_DSC[detailed_status]);
}

/*
 * Used for error reporting/debugging purposes
 */
static void i2o_report_util_cmd(u8 cmd)
{
	switch (cmd) {
	case I2O_CMD_UTIL_NOP:
		printk("UTIL_NOP, ");
		break;			
	case I2O_CMD_UTIL_ABORT:
		printk("UTIL_ABORT, ");
		break;
	case I2O_CMD_UTIL_CLAIM:
		printk("UTIL_CLAIM, ");
		break;
	case I2O_CMD_UTIL_RELEASE:
		printk("UTIL_CLAIM_RELEASE, ");
		break;
	case I2O_CMD_UTIL_CONFIG_DIALOG:
		printk("UTIL_CONFIG_DIALOG, ");
		break;
	case I2O_CMD_UTIL_DEVICE_RESERVE:
		printk("UTIL_DEVICE_RESERVE, ");
		break;
	case I2O_CMD_UTIL_DEVICE_RELEASE:
		printk("UTIL_DEVICE_RELEASE, ");
		break;
	case I2O_CMD_UTIL_EVT_ACK:
		printk("UTIL_EVENT_ACKNOWLEDGE, ");
		break;
	case I2O_CMD_UTIL_EVT_REGISTER:
		printk("UTIL_EVENT_REGISTER, ");
		break;
	case I2O_CMD_UTIL_LOCK:
		printk("UTIL_LOCK, ");
		break;
	case I2O_CMD_UTIL_LOCK_RELEASE:
		printk("UTIL_LOCK_RELEASE, ");
		break;
	case I2O_CMD_UTIL_PARAMS_GET:
		printk("UTIL_PARAMS_GET, ");
		break;
	case I2O_CMD_UTIL_PARAMS_SET:
		printk("UTIL_PARAMS_SET, ");
		break;
	case I2O_CMD_UTIL_REPLY_FAULT_NOTIFY:
		printk("UTIL_REPLY_FAULT_NOTIFY, ");
		break;
	default:
		printk("Cmd = %0#2x, ",cmd);	
	}
}

/*
 * Used for error reporting/debugging purposes
 */
static void i2o_report_exec_cmd(u8 cmd)
{
	switch (cmd) {
	case I2O_CMD_ADAPTER_ASSIGN:
		printk("EXEC_ADAPTER_ASSIGN, ");
		break;
	case I2O_CMD_ADAPTER_READ:
		printk("EXEC_ADAPTER_READ, ");
		break;
	case I2O_CMD_ADAPTER_RELEASE:
		printk("EXEC_ADAPTER_RELEASE, ");
		break;
	case I2O_CMD_BIOS_INFO_SET:
		printk("EXEC_BIOS_INFO_SET, ");
		break;
	case I2O_CMD_BOOT_DEVICE_SET:
		printk("EXEC_BOOT_DEVICE_SET, ");
		break;
	case I2O_CMD_CONFIG_VALIDATE:
		printk("EXEC_CONFIG_VALIDATE, ");
		break;
	case I2O_CMD_CONN_SETUP:
		printk("EXEC_CONN_SETUP, ");
		break;
	case I2O_CMD_DDM_DESTROY:
		printk("EXEC_DDM_DESTROY, ");
		break;
	case I2O_CMD_DDM_ENABLE:
		printk("EXEC_DDM_ENABLE, ");
		break;
	case I2O_CMD_DDM_QUIESCE:
		printk("EXEC_DDM_QUIESCE, ");
		break;
	case I2O_CMD_DDM_RESET:
		printk("EXEC_DDM_RESET, ");
		break;
	case I2O_CMD_DDM_SUSPEND:
		printk("EXEC_DDM_SUSPEND, ");
		break;
	case I2O_CMD_DEVICE_ASSIGN:
		printk("EXEC_DEVICE_ASSIGN, ");
		break;
	case I2O_CMD_DEVICE_RELEASE:
		printk("EXEC_DEVICE_RELEASE, ");
		break;
	case I2O_CMD_HRT_GET:
		printk("EXEC_HRT_GET, ");
		break;
	case I2O_CMD_ADAPTER_CLEAR:
		printk("EXEC_IOP_CLEAR, ");
		break;
	case I2O_CMD_ADAPTER_CONNECT:
		printk("EXEC_IOP_CONNECT, ");
		break;
	case I2O_CMD_ADAPTER_RESET:
		printk("EXEC_IOP_RESET, ");
		break;
	case I2O_CMD_LCT_NOTIFY:
		printk("EXEC_LCT_NOTIFY, ");
		break;
	case I2O_CMD_OUTBOUND_INIT:
		printk("EXEC_OUTBOUND_INIT, ");
		break;
	case I2O_CMD_PATH_ENABLE:
		printk("EXEC_PATH_ENABLE, ");
		break;
	case I2O_CMD_PATH_QUIESCE:
		printk("EXEC_PATH_QUIESCE, ");
		break;
	case I2O_CMD_PATH_RESET:
		printk("EXEC_PATH_RESET, ");
		break;
	case I2O_CMD_STATIC_MF_CREATE:
		printk("EXEC_STATIC_MF_CREATE, ");
		break;
	case I2O_CMD_STATIC_MF_RELEASE:
		printk("EXEC_STATIC_MF_RELEASE, ");
		break;
	case I2O_CMD_STATUS_GET:
		printk("EXEC_STATUS_GET, ");
		break;
	case I2O_CMD_SW_DOWNLOAD:
		printk("EXEC_SW_DOWNLOAD, ");
		break;
	case I2O_CMD_SW_UPLOAD:
		printk("EXEC_SW_UPLOAD, ");
		break;
	case I2O_CMD_SW_REMOVE:
		printk("EXEC_SW_REMOVE, ");
		break;
	case I2O_CMD_SYS_ENABLE:
		printk("EXEC_SYS_ENABLE, ");
		break;
	case I2O_CMD_SYS_MODIFY:
		printk("EXEC_SYS_MODIFY, ");
		break;
	case I2O_CMD_SYS_QUIESCE:
		printk("EXEC_SYS_QUIESCE, ");
		break;
	case I2O_CMD_SYS_TAB_SET:
		printk("EXEC_SYS_TAB_SET, ");
		break;
	default:
		printk("Cmd = %#02x, ",cmd);	
	}
}


/*
 * Used for error reporting/debugging purposes.
 * Report Cmd name, Request status, Detailed Status.
 */
void i2o_report_status(const char *severity, const char *str, u32 *msg)
{
	u8 cmd = (msg[1]>>24)&0xFF;
	u8 req_status = (msg[4]>>24)&0xFF;
	u16 detailed_status = msg[4]&0xFFFF;

	printk("%s%s: ", severity, str);

	if (cmd < 0x1F) 			// Utility cmd
		i2o_report_util_cmd(cmd);
	
	else if (cmd >= 0xA0 && cmd <= 0xEF) 	// Executive cmd
		i2o_report_exec_cmd(cmd);
	else
        	printk("Cmd = %0#2x, ", cmd);	// Other cmds

	if (msg[0] & MSG_FAIL) {
		i2o_report_fail_status(req_status, msg);
		return;
	}
	
	i2o_report_common_status(req_status);

	if (cmd < 0x1F || (cmd >= 0xA0 && cmd <= 0xEF))
		i2o_report_common_dsc(detailed_status);	
	else
		printk(" / DetailedStatus = %0#4x.\n", detailed_status); 
}

/* Used to dump a message to syslog during debugging */
void i2o_dump_message(u32 *msg)
{
#ifdef IOCTLDEBUG
	int i;
	printk(KERN_INFO "Dumping I2O message size %d @ %p\n", 
	       msg[0]>>16&0xffff, msg);
	for(i = 0; i < ((msg[0]>>16)&0xffff); i++)
		printk(KERN_INFO "  msg[%d] = %0#10x\n", i, msg[i]);
#endif
}

/*
 * I2O reboot/shutdown notification.
 *
 * - Call each OSM's reboot notifier (if one exists)
 * - Quiesce each IOP in the system
 *
 * Each IOP has to be quiesced before we can ensure that the system
 * can be properly shutdown as a transaction that has already been
 * acknowledged still needs to be placed in permanent store on the IOP.
 * The SysQuiesce causes the IOP to force all HDMs to complete their
 * transactions before returning, so only at that point is it safe
 * 
 */
static int i2o_reboot_event(struct notifier_block *n, unsigned long code, void
			    *p)
{
	int i = 0;
	struct i2o_controller *c = NULL;

	if(code != SYS_RESTART && code != SYS_HALT && code != SYS_POWER_OFF)
		return NOTIFY_DONE;

	printk(KERN_INFO "Shutting down IS1000...\n");
	/*	printk(KERN_INFO 
		"   This could take a few minutes if there are many devices attached\n");
	*/

	for(i = 0; i < MAX_I2O_MODULES; i++)
	{
		if(i2o_handlers[i] && i2o_handlers[i]->reboot_notify)
			i2o_handlers[i]->reboot_notify();
	}

	for(c = i2o_controller_chain; c; c = c->next)
	{
		if(i2o_quiesce_controller(c))
		{
			printk(KERN_WARNING "i2o: Could not quiesce %s.\n", c->name);
			printk(KERN_WARNING "i2o: Verify setup on next system power up.\n");
		}
	}

	/*	printk(KERN_INFO "I2O system down.\n");*/
	return NOTIFY_DONE;
}


/* 
 * There's no need for exporting any of these. However, in case of future
 * work (not in kernel, links with other parts, etc) I'm leaving in here.
 */
#if 0
EXPORT_SYMBOL(i2o_controller_chain);
EXPORT_SYMBOL(i2o_num_controllers);
EXPORT_SYMBOL(i2o_find_controller);
EXPORT_SYMBOL(i2o_unlock_controller);
EXPORT_SYMBOL(i2o_status_get);

EXPORT_SYMBOL(i2o_install_handler);
EXPORT_SYMBOL(i2o_remove_handler);

EXPORT_SYMBOL(i2o_install_controller);
EXPORT_SYMBOL(i2o_delete_controller);
EXPORT_SYMBOL(i2o_run_queue);

EXPORT_SYMBOL(i2o_claim_device);
EXPORT_SYMBOL(i2o_release_device);
EXPORT_SYMBOL(i2o_device_notify_on);
EXPORT_SYMBOL(i2o_device_notify_off);

EXPORT_SYMBOL(i2o_post_this);
EXPORT_SYMBOL(i2o_post_wait);
EXPORT_SYMBOL(i2o_post_wait_mem);

EXPORT_SYMBOL(i2o_query_scalar);
EXPORT_SYMBOL(i2o_set_scalar);
EXPORT_SYMBOL(i2o_query_table);
EXPORT_SYMBOL(i2o_clear_table);
EXPORT_SYMBOL(i2o_row_add_table);
EXPORT_SYMBOL(i2o_issue_params);

EXPORT_SYMBOL(i2o_event_register);
EXPORT_SYMBOL(i2o_event_ack);

EXPORT_SYMBOL(i2o_report_status);
EXPORT_SYMBOL(i2o_dump_message);

EXPORT_SYMBOL(i2o_get_class_name);

EXPORT_SYMBOL(i2o_util_nop);
#endif

#ifdef RH90
EXPORT_SYMBOL_GPL(is1000_sys_init);
#endif

//#ifdef MODULE

MODULE_AUTHOR("Intel Corporation");
MODULE_DESCRIPTION("Intel PRO/1000 T IP Storage Adapter");
MODULE_LICENSE("GPL");

static int ControlOpen( struct inode *inode, struct file *file )
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int ControlClose( struct inode *inode, struct file *file )
{
        MOD_DEC_USE_COUNT;
	return 0;
}

static struct i2o_controller * iscsi_find_adapter(int *adapterNum, PIS_MAC_ADDRESS pMacAddr)
{
	struct i2o_controller *c = NULL;
	int i;
	*adapterNum = -1;
	for (i = 0; i < i2o_num_controllers ; i++)
	{
		if (!memcmp(pMacAddr, &iscsi_adapter_priv[i].macAddr, 
			    MAC_ADDR_LENGTH))
		{
			*adapterNum = i;
			c = i2o_find_controller(i);
			break;
		}
	}
#ifdef IOCTLDEBUG
	printk(KERN_ERR "is1000: iscsi_find_adapter returning %X %d\n", (int)c, *adapterNum);
#endif
	return c;
}

/* ControlRead - */
static ssize_t ControlRead(struct file *file, char *buf, size_t count, loff_t *ptr)
{
	unsigned long flags;
	IS1000_EVENT localEvent;

#ifdef IOCTLDEBUG
	printk(KERN_INFO "\n*********** ControlRead entered.\n");
#endif
	if (count != sizeof(IS1000_EVENT))
	{
#ifdef IOCTLDEBUG
		printk(KERN_ERR "is1000: ControlRead: buf size %d, expecting %d\n", 
		       count, sizeof(IS1000_EVENT));
#endif
		return -EINVAL;
	}
	if (buf == NULL)
	{
#ifdef IOCTLDEBUG
		printk(KERN_ERR "is1000: daemon provided null buffer\n");
#endif
		return -EINVAL;
	}
	
	down_interruptible(&is1000_evt_sem);
	if(signal_pending(current))
	{
		dprintk(KERN_ERR "is1000: daemon thread dead\n");
		return 0;
	}

	//  interruptible_sleep_on(&evt_bus_wait);
  
	spin_lock_irqsave(&is1000_evt_lock, flags);
	memcpy(&localEvent, &connectionEvent[is1000_evt_out], sizeof(IS1000_EVENT));
	MODINC(is1000_evt_out, IS1000_EVT_Q_LEN);
	is1000_evt_q_len--;
	spin_unlock_irqrestore(&is1000_evt_lock, flags);
	
	if (copy_to_user (buf, &localEvent, sizeof(IS1000_EVENT))) 
	{
		dprintk(KERN_ERR "is1000: Error copying to user space\n");
		return -EFAULT;
	}

#ifdef IOCTLDEBUG
	printk(KERN_INFO "is1000: ControlRead: returning success\n");
#endif
	return sizeof(IS1000_EVENT);
}


static void is1000_remove_target (char *targetName, int adapterNum)
{
	struct i2o_scsi_host *h;
	int i;

	h = is1000_get_hostdata(adapterNum);

	if (h == NULL)
	{
		printk(KERN_ERR "is1000: iSCSI host data not available\n");
		return;
	}

	/* Find this target in the tid data list */
	for (i = 0; i < IOP_MAXIMUM_TARGETS ; i++)
	{
		if (!strncmp(targetName, h->aTidData[i].name,
			     ISCSI_TARGET_NAME_LENGTH - 1))
		{
#ifdef IOCTLDEBUG
			printk(KERN_ERR "is1000: found and removing <%s>\n",
			       targetName);
#endif
			h->aTaskData[h->aTidData[i].index].pTidData = NULL;
			h->aTaskData[h->aTidData[i].index].inUse = 0;
			h->aTidData[i].tid = -1;
			h->aTidData[i].index = -1;
			h->aTidData[i].status = -1;
			memset(h->aTidData[i].name, 0, ISCSI_TARGET_NAME_LENGTH);
			return;
		}
	}
	printk(KERN_ERR "is1000: unable to remove target: %s\n",
	       h->aTidData[i].name);
}         


static PISCSI_TID_DATA is1000_get_tiddata(u32 unit, u32 targetId)
{
	struct i2o_scsi_host *h;
	PISCSI_TID_DATA pTidData;

	h = is1000_get_hostdata(unit);
		
	if (h == NULL)
	{
		printk(KERN_ERR "is1000: SCSI host bus %d may not be registered\n",
		       unit);
		return NULL;
	}

	pTidData = h->aTaskData[targetId].pTidData;

	if (pTidData == NULL)
	{
		printk(KERN_ERR "is1000: Target with id %d not fully connected\n",
		       targetId);
		return NULL;
	}

#ifdef IOCTLDEBUG
	printk(KERN_ERR "is1000: returning pTidData 0x%x\n", (int)pTidData);
#endif
	return pTidData;
}


/* ControlIoctl - */
static int ControlIoctl( struct inode *inode, struct file *file,
			 unsigned int cmd, unsigned long arg)
{
        char *pArgData, *pBuffer = 0;
	int i, adapterNum = -1, res;
	struct i2o_controller *c = NULL;

#ifdef IOCTLDEBUG
	printk(KERN_ERR 
	       "\n******is1000: ioctl() entering *****  cmd = 0x%x\n",
	       cmd);
#if 0
	printk(KERN_ERR "ISCSI_IOCTL_ENUMERATE_ADAPTERS 0x%x\n", ISCSI_IOCTL_ENUMERATE_ADAPTERS);
	printk(KERN_ERR "ISCSI_IOCTL_GET_ADAPTER_PROPERTIES 0x%x\n", ISCSI_IOCTL_GET_ADAPTER_PROPERTIES);
	printk(KERN_ERR "ISCSI_IOCTL_SET_ADAPTER_PROPERTIES 0x%x\n", ISCSI_IOCTL_SET_ADAPTER_PROPERTIES);
	printk(KERN_ERR "ISCSI_IOCTL_GET_DIAGNOSTICS 0x%x\n", ISCSI_IOCTL_GET_DIAGNOSTICS);
	printk(KERN_ERR "ISCSI_IOCTL_GET_STATISTICS 0x%x\n", ISCSI_IOCTL_GET_STATISTICS);
	printk(KERN_ERR "ISCSI_IOCTL_GET_STORAGE_TARGET_COUNT 0x%x\n", ISCSI_IOCTL_GET_STORAGE_TARGET_COUNT);
	printk(KERN_ERR "ISCSI_IOCTL_GET_STORAGE_TARGET 0x%x\n", ISCSI_IOCTL_GET_STORAGE_TARGET);
	printk(KERN_ERR "ISCSI_IOCTL_BLINK_LED 0x%x\n", ISCSI_IOCTL_BLINK_LED);
	printk(KERN_ERR "ISCSI_IOCTL_ADD_TARGET 0x%x\n", ISCSI_IOCTL_ADD_TARGET);
	printk(KERN_ERR "ISCSI_IOCTL_REMOVE_TARGET 0x%x\n", ISCSI_IOCTL_REMOVE_TARGET);
	printk(KERN_ERR "ISCSI_IOCTL_GET_TARGET_PROPERTIES 0x%x\n", ISCSI_IOCTL_GET_TARGET_PROPERTIES);
	printk(KERN_ERR "ISCSI_IOCTL_STORAGE_TARGET_STATUS 0x%x\n", ISCSI_IOCTL_STORAGE_TARGET_STATUS);
	printk(KERN_ERR "ISCSI_IOCTL_TARGET_INITIATE_LOGIN 0x%x\n", ISCSI_IOCTL_TARGET_INITIATE_LOGIN);
	printk(KERN_ERR "ISCSI_IOCTL_TARGET_DISABLE 0x%x\n", ISCSI_IOCTL_TARGET_DISABLE);
	printk(KERN_ERR "ISCSI_IOCTL_TARGET_RECOVERY_DATA 0x%x\n", ISCSI_IOCTL_TARGET_RECOVERY_DATA);
	printk(KERN_ERR "ISCSI_IOCTL_SET_TARGET_CHAP 0x%x\n", ISCSI_IOCTL_SET_TARGET_CHAP);
	printk(KERN_ERR "ISCSI_IOCTL_RESET_TARGET 0x%x\n", ISCSI_IOCTL_RESET_TARGET);
	printk(KERN_ERR "ISCSI_IOCTL_GET_TARGET_STATISTICS 0x%x\n", ISCSI_IOCTL_GET_TARGET_STATISTICS);
	printk(KERN_ERR "ISCSI_IOCTL_TARGET_ERROR_STATUS 0x%x\n", ISCSI_IOCTL_TARGET_ERROR_STATUS);
#endif

#endif
	pArgData = (char *)kmalloc(MAX_IOCTL_ARG_DATA, GFP_KERNEL);
	if (pArgData == NULL)
	{
		printk(KERN_ERR "is1000: no memory for ioctl buffer.\n");
		goto outFail;
	}
	memset (pArgData, 0, sizeof(*pArgData));

	pBuffer = (char *)kmalloc(MAX_BUF_LIST_SIZE, GFP_KERNEL);
	if (pBuffer == NULL)
	{
		printk(KERN_ERR "is1000: no memory for ioctl buffer.\n");
		goto outFail;
	}
	memset (pBuffer, 0, sizeof(*pBuffer));

	switch (cmd) {

	case ISCSI_IOCTL_GET_DRIVER_VERSION:
	{
		ISCSI_GET_DRIVER_VERSION driverVersion;
	    
		driverVersion.version = DRIVER_VERSION;

		if (copy_to_user((void *)arg, &driverVersion,
				 sizeof(ISCSI_GET_DRIVER_VERSION)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_IOCTL_GET_DRIVER_VERSION\n");
			goto outFail;
		}
	}	    
	break;

	case ISCSI_IOCTL_ENUMERATE_ADAPTERS:
	{
		static int initScsiCalled = 0;
		u16 *pBufList;
		char macAddr[24];
		ISCSI_ENUMERATE_BOARDS *pEnumAdapters = (ISCSI_ENUMERATE_BOARDS *)pArgData;
		int pKeys[20];  /* MAX_BOARDS is 16, plus room for status */
		int fieldIdx[1];

		if ((res = copy_from_user(pEnumAdapters,(void *)arg,
					  sizeof(ISCSI_ENUMERATE_BOARDS)))) 
		{
		        printk(KERN_ERR "is1000: Cannot copy user Ioctl args   error %d\n", res);
			goto outFail;
		}

		memset (pKeys, 0, sizeof(pKeys));

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _ENUMERATE_ADAPTERS, i2o_num_controllers %d\n",
		       i2o_num_controllers);
#endif

		if (!initScsiCalled) 
		{
		        initScsiCalled = 1;

#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: before init_this_scsi_driver()\n");
#endif
			res = init_this_scsi_driver();
		     
			if (res < 0)
			{
			        if (res == -ENODEV)
				{
				        printk(KERN_INFO "is1000: No devices present currently\n");
				}
				else
				{
				        printk(KERN_ERR "is1000: Error in scsi initialization: %d\n",
					       res);
				}	      
				goto outFail;
			}
#ifdef IOCTLDEBUG
			else
			{
			        printk(KERN_INFO "is1000: (ioctl): %d new scsi devices\n", res);
			}
#endif

			/* Get table from each controller c   */
			for (i = 0 ; i < i2o_num_controllers ; i++)
			{
				pBufList = (u16 *)pBuffer;

				if ((c = i2o_find_controller(i)) == NULL)
					continue;

				fieldIdx[0] = ISCSI_ISM_NETIF_TABLE_KEY_IDX;

				res = i2o_query_table (I2O_PARAMS_TABLE_GET,
						       c, 
						       8 /* need to get legitimately */,
						       ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO, 1, 
						       fieldIdx, sizeof(fieldIdx),
						       pKeys, sizeof(pKeys));
				if (res < 0)
				{
					printk(KERN_ERR 
					       "is1000: query failed to enumerate adapters (error %d)\n", res);
					goto outFail;
				}

				*pBufList++ = ISCSI_ISM_NETIF_TABLE_MACADDR_IDX;
				*pBufList++ = 1; /* key count */
				memcpy(pBufList, &pKeys[3], sizeof(int));

				res = i2o_query_table (I2O_PARAMS_LIST_GET,
						       c, 
						       8 /* need to get legitimately */,
						       ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO, 1, 
						       pBuffer, sizeof(u32) + sizeof(u32),
						       macAddr, sizeof(macAddr));
				if (res < 0)
				{
					printk(KERN_ERR "is1000: query adapter failed %d\n", res);
					pEnumAdapters->dwStatus = ENUMERATION_STATUS_FAILED;
					goto outFail;
				}

				/* Both queries succeeded. Store results and private data for
				   all adapters. */

				if (pKeys[2] == 1)
				{
					pEnumAdapters->iSCSIAdapters[i].adapterNumber = 
						iscsi_adapter_priv[i].hostAdapterNum;

#ifdef IOCTLDEBUG
					printk(KERN_INFO 
					       "is1000: pEnumAdapters->iSCSIAdapters[%d].adapterNumber %d\n",
					       i, (int)pEnumAdapters->iSCSIAdapters[i].adapterNumber);
#endif

					memcpy(&pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress,
					       &macAddr[8], MAC_ADDR_LENGTH);
					pEnumAdapters->nNumberAdapters++;

					iscsi_adapter_priv[i].ifTableKey = pKeys[3];
					memcpy(&iscsi_adapter_priv[i].macAddr, 
					       &pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress, MAC_ADDR_LENGTH);

#ifdef IOCTLDEBUG
					printk(KERN_INFO 
					       "is1000: pEnumAdapters->iSCSIAdapters[%d].AdapterMACAddress 0x%08x 0x%08x\n",
					       i, (int)pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress.first,
					       (int)pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress.second);
					printk(KERN_INFO "is1000: pEnumAdapters->nNumberAdapters %d\n",
					       pEnumAdapters->nNumberAdapters);
#endif
				}
				else
				{
					printk(KERN_ERR "is1000: Ioctl query of adapter failed, num interfaces: %d\n", pKeys[2]);
					pEnumAdapters->dwStatus = ENUMERATION_STATUS_FAILED;
					goto outFail;
				}

				i2o_unlock_controller(c);
				c = 0;

			} /* end for */

			/* This buffer is used for each target query. After a successful
			 * get storage target count, all the data for that query is here.
			 * The caller must call get storage targets next (before the
			 * next query) and provide an index into this table, to get
			 * the target info. Room for the target's ip and port is added
			 * at the end to allow us to check that the subsequent get
			 * storage targets call is indeed for the same target.
			 */
			pEnumTargets = (ISCSI_TOTAL_ENUM_TARGETS *) 
				kmalloc(MAX_TARGETS_PER_ENTITY * sizeof(ISCSI_TOTAL_ENUM_TARGETS) +
					sizeof(u32) + sizeof(u16), GFP_KERNEL);
			if (pEnumTargets == NULL)
			{
				printk(KERN_ERR "is1000: no memory for ioctl buffer.\n");
				goto outFail;
			}
			/* This is not needed. We zero the buffer before each query 
			   memset (pEnumTargets, 0,
			   MAX_TARGETS_PER_ENTITY * sizeof(ISCSI_TOTAL_ENUM_TARGETS));
			*/
		} /* end if scsi init not called yet */

		else { /* else we have all the enum data in the adapter_priv struct already */

#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: incoming pEnumAdapters->nNumberAdapters %d\n",
			       pEnumAdapters->nNumberAdapters);
#endif

			for (i = 0; i < i2o_num_controllers ; i++)
			{
				pEnumAdapters->iSCSIAdapters[i].adapterNumber = 
					iscsi_adapter_priv[i].hostAdapterNum;

				memcpy(&pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress,
				       &iscsi_adapter_priv[i].macAddr, MAC_ADDR_LENGTH);
				pEnumAdapters->nNumberAdapters++;

#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: _ENUMERATE_ADAPTER called previously, returning cached data:\n");
				printk(KERN_INFO 
				       "is1000: pEnumAdapters->iSCSIAdapters[%d].AdapterMACAddress 0x%08x 0x%08x\n",
				       i, (int)pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress.first,
				       (int)pEnumAdapters->iSCSIAdapters[i].AdapterMACAddress.second);
#endif
			}

		}

		pEnumAdapters->dwStatus = ENUMERATION_STATUS_SUCCESS;

		if (copy_to_user((void *)arg, pEnumAdapters,
				 sizeof(ISCSI_ENUMERATE_BOARDS)))
		{
		        printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_IOCTL_ENUMERATE_ADAPTERS\n");
			goto outFail;
		}
	}
	break;


	case ISCSI_IOCTL_SET_TARGET_CHAP:
	{
		PISCSI_STORAGE_TARGET_INFO pSetChap = (PISCSI_STORAGE_TARGET_INFO)pArgData;
		int tid;
		// struct i2o_scsi_host *h;
		PISCSI_TID_DATA pTidData;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _SET_TARGET_CHAP:\n");
#endif	

		if (copy_from_user(pSetChap,(char *)arg, 
				   sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;	   
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pSetChap->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: chap.useCHAP %d chap.CHAPusername: %s chap.CHAPpassword: %s\n",
		       (int)pSetChap->sTarget.chap.useCHAP, pSetChap->sTarget.chap.CHAPusername,
		       pSetChap->sTarget.chap.CHAPpassword);

		printk(KERN_INFO "is1000: chap.useInboundCHAP %d inboundCHAPusername: %s inboundCHAPpassword: %s\n",
		       pSetChap->sTarget.chap.useInboundCHAP, 
		       pSetChap->sTarget.chap.inboundCHAPusername,
		       pSetChap->sTarget.chap.inboundCHAPpassword);

		printk(KERN_INFO "is1000:     ->sTarget.iscsi_nodename: %s\n",
		       pSetChap->sTarget.iscsi_nodename);
		printk(KERN_INFO "is1000: **UNUSED**:    timeout %d retries %d\n",
		       (int)pSetChap->sTarget.timeout,
		       (int)pSetChap->sTarget.retries);
#endif

		if (pSetChap->sTarget.targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR "is1000: Set chap data on unenabled target not supported\n");
			goto outFail;
		}

		/* start with adapterNum and targetId, get back pTidData -
		 * need to check for NULL from there.
		 */
		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pSetChap->sTarget.targetId)))
		{
			goto outFail;
		}

		tid = pTidData->tid;

		if (tid == -1)
		{
			printk (KERN_ERR "is1000: No targetId %d found on adapter %d\n",
				pSetChap->sTarget.targetId, adapterNum);
			goto outFail;
		}

		if (pSetChap->sTarget.chap.useCHAP)
		{
			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_CHAP_USERNAME,
					      pSetChap->sTarget.chap.CHAPusername,
					      ISCSI_CHAP_FIELD_LENGTH);
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}

			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_CHAP_PASSWORD,
					      pSetChap->sTarget.chap.CHAPpassword,
					      ISCSI_CHAP_FIELD_LENGTH);
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}
		}

		res =  i2o_set_scalar(c, tid,
				      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
				      ISCSI_ISM_PER_HBA_USE_CHAP,
				      &pSetChap->sTarget.chap.useCHAP,
				      sizeof(u32));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;	    
		}


		if (pSetChap->sTarget.chap.useInboundCHAP)
		{
			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_CHAP_TGT_USERNAME,
					      pSetChap->sTarget.chap.inboundCHAPusername,
					      ISCSI_CHAP_FIELD_LENGTH);
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}

			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_CHAP_TGT_PASSWORD,
					      pSetChap->sTarget.chap.inboundCHAPpassword,
					      ISCSI_CHAP_FIELD_LENGTH);
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}
		}
		res =  i2o_set_scalar(c, tid,
				      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
				      ISCSI_ISM_PER_HBA_USE_INBOUND_CHAP,
				      &pSetChap->sTarget.chap.useInboundCHAP,
				      sizeof(u32));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;	    
		}

		pSetChap->dwStatus = ENUMERATION_STATUS_SUCCESS;
			
		if (copy_to_user((void *)arg, pSetChap,
				 sizeof(ISCSI_STORAGE_TARGET_INFO)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for _SET_TARGET_CHAP\n");
			goto outFail;
		}
	}
	break;

	case ISCSI_IOCTL_TARGET_INITIATE_LOGIN:
	{
		PISCSI_STORAGE_TARGET_INFO pTgtStatus = (PISCSI_STORAGE_TARGET_INFO)pArgData;
		PI2O_TARGET_INIT_LOGIN pInitLogin;
		u16 *pBufList = (u16 *)pBuffer;
		int resBlk[10], len;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _TARGET_INITIATE_LOGIN:\n");
#endif	

		if (copy_from_user(pTgtStatus,(char *)arg, 
				   sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;	   
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTgtStatus->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _TARGET_INITIATE_LOGIN adapter %d \n", adapterNum);

		printk(KERN_INFO "is1000:  dwStatus %d  ->sTarget.address.portalgroup %d\n",
		       (int)pTgtStatus->dwStatus, pTgtStatus->sTarget.address.portalgroup);
		printk(KERN_INFO "is1000:     ->sTarget.address.ip 0x%x addressport %d\n",
		       (int)pTgtStatus->sTarget.address.ip, (int)pTgtStatus->sTarget.address.port);
		printk(KERN_INFO "is1000:     ->sTarget.MACAddr 0x%08x%08x\n",
		       (int)pTgtStatus->MACAddr.first, (int)pTgtStatus->MACAddr.second);
		if (pTgtStatus->sTarget.chap.useCHAP != 0)
		{
			printk(KERN_INFO 
			       "is1000: chap.useCHAP %d chap.CHAPusername: %s chap.CHAPpassword: %s\n",
			       (int)pTgtStatus->sTarget.chap.useCHAP, pTgtStatus->sTarget.chap.CHAPusername,
			       pTgtStatus->sTarget.chap.CHAPpassword);
		}
		if (pTgtStatus->sTarget.chap.useInboundCHAP != 0)
		{
			printk(KERN_INFO 
			       "is1000: chap.useInboundCHAP %d inboundCHAPusername %s inboundCHAPpassword %s\n",
			       pTgtStatus->sTarget.chap.useInboundCHAP, 
			       pTgtStatus->sTarget.chap.inboundCHAPusername,
			       pTgtStatus->sTarget.chap.inboundCHAPpassword);
		}
		printk(KERN_INFO "is1000:     ->sTarget.iscsi_nodename: %s\n",
		       pTgtStatus->sTarget.iscsi_nodename);
		printk(KERN_INFO "is1000: **UNUSED**:   timeout %d retries %d\n",
		       (int)pTgtStatus->sTarget.timeout, (int)pTgtStatus->sTarget.retries);
#endif
	
		*pBufList++ = ISCSI_ISM_TARGET_TABLE_KEY_IDX;
		*pBufList++ = 10;
		*pBufList++ = 11;
		*pBufList++ = 12;
		*pBufList++ = 13;
		*pBufList++ = 14;
		*pBufList++ = 15;
		*pBufList++ = ISCSI_ISM_TARGET_TABLE_ENABLE_IDX;
		*pBufList++ = 1;

		pInitLogin = (PI2O_TARGET_INIT_LOGIN) pBufList;

		strncpy(pInitLogin->key, pTgtStatus->sTarget.iscsi_nodename, 
			ISCSI_TARGET_NAME_LENGTH - 1);

		len = strlen(pTgtStatus->sTarget.iscsi_nodename); 

		memset (pInitLogin->key, 0, sizeof(pInitLogin->key));
		strncpy(pInitLogin->key, ((u8 *)pTgtStatus->sTarget.iscsi_nodename), 
			IS_MAX_NODENAME_STRING - 1); /* could be len here */

		pInitLogin->key[len] = '~';
		
		sprintf (&pInitLogin->key[len+1], "%08x%04x", pTgtStatus->sTarget.address.ip, 
			 pTgtStatus->sTarget.address.port);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pInitLogin->key: %s\n", pInitLogin->key);
#endif

		pInitLogin->enable = ISCSI_PARAM_TARGET_ENABLE;

		/* Fill in chap data, if needed */
		if ((pInitLogin->chapEnable = pTgtStatus->sTarget.chap.useCHAP))
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: pInitLogin->chapEnable %d\n",
			       pInitLogin->chapEnable);
#endif
			memcpy (pInitLogin->chapUsername, 
				pTgtStatus->sTarget.chap.CHAPusername, ISCSI_CHAP_FIELD_LENGTH);
			memcpy (pInitLogin->chapPassword, 
				pTgtStatus->sTarget.chap.CHAPpassword, ISCSI_CHAP_FIELD_LENGTH);
		}

		if ((pInitLogin->chapAuthTarget = pTgtStatus->sTarget.chap.useInboundCHAP))
		{
			memcpy (pInitLogin->chapTargetUsername, 
				pTgtStatus->sTarget.chap.inboundCHAPusername, ISCSI_CHAP_FIELD_LENGTH);
			memcpy (pInitLogin->chapTargetPassword, 
				pTgtStatus->sTarget.chap.inboundCHAPpassword, ISCSI_CHAP_FIELD_LENGTH);
		}

		res = i2o_set_list_table(I2O_PARAMS_LIST_SET, 
					 c,
					 8, ISCSI_ISM_TARGET_TABLE_GROUP_NO,
					 8,
					 pBuffer, sizeof(I2O_TARGET_INIT_LOGIN) + 16 + 32,
					 resBlk, sizeof(resBlk));
		if (res < 0) {
			printk(KERN_ERR 
			       "is1000: Target login failed to contact adapter %d\n", 
			       adapterNum);
			goto outFail;	 
		}
		else if (resBlk[1] & 0x00ff0000)
		{
#ifdef IOCTLDEBUG
			printk(KERN_ERR 
			       "is1000: Target login failed on adapter %d with 0x%08x\n", 
			       adapterNum, resBlk[1]);
#endif
		}
		pTgtStatus->dwStatus = ENUMERATION_STATUS_PENDING;                          
			
#ifdef IOCTLDEBUG
		printk(KERN_ERR "is1000: resBlk[0] 0x%08x  0x%08x  0x%08x  0x%08x\n", 
		       resBlk[0], resBlk[1], resBlk[2], resBlk[3]);
		printk(KERN_INFO "is1000: i2o_set_list_table success %d, returning ENUMERATION_STATUS_PENDING\n", res);
#endif
		if (copy_to_user((void *)arg, pTgtStatus,
				 sizeof(ISCSI_STORAGE_TARGET_INFO)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_INITIATE_LOGIN\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_STORAGE_TARGET_STATUS :
	{
		PISCSI_STORAGE_TARGET_INFO pTgtStatus = (PISCSI_STORAGE_TARGET_INFO)pArgData;
		u16 *pBufList = (u16 *)pBuffer;
		int resBlk[64], len;
		int tid, connStatus;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _STORAGE_TARGET_STATUS:\n");
#endif	

		if (copy_from_user(pTgtStatus,(char *)arg, sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTgtStatus->MACAddr)))
		{
			goto outFail;
		}

		/* Check if targetId is valid. Maybe target is logged in etc and
		 * the caller is just checking the target status.
		 */

		/* Note: This is on hold. Currently a 3 second wait before
		 *       inquiring is provided as a work-around.
		 */
#if 0
		if (pTgtStatus->sTarget.targetId != IS_TARGETID_INVALID)
		{
			h = is1000_get_hostdata(adapterNum);
		    
			if (h == NULL)
			{
				printk(KERN_ERR "is1000: _GET_TARGET_STATUS found NULL i2o_scsi_host\n");
				goto outFail;
			}

			pTidData = h->aTaskData[pTgtStatus->sTarget.targetId].pTidData;

			if (pTidData == NULL)
			{
#ifdef IOCTLDEBUG			
				printk(KERN_ERR "is1000: _GET_TARGET_STATUS pTidData == NULL\n");
				/* ?? */
#endif
			}
			else
			{
				if ((pTidData->tid != -1) && (pTidData->status != -1))
				{
					if (pTidData->status == ISCSI_TARGET_STATUS_CONNECTED)
					{
#ifdef IOCTLDEBUG			
						printk(KERN_ERR 
						       "is1000: _GET_TARGET_STATUS: CONNECTION_OK\n");
#endif
						pTgtStatus->dwStatus = CONNECTION_OK;
					}
					else
					{
#ifdef IOCTLDEBUG			
						printk(KERN_ERR 
						       "is1000: _GET_TARGET_STATUS: CONNECTION_DISCONNECTED\n");
#endif
						pTgtStatus->dwStatus = CONNECTION_DISCONNECTED;
					}
					if (copy_to_user((void *)arg, pTgtStatus,
							 sizeof(ISCSI_STORAGE_TARGET_INFO)))
					{
						printk(KERN_ERR 
						       "is1000: copy_to_user failed: ISCSI_IOCTL_STORAGE_TARGET_STATUS\n");
						goto outFail;
					}
				}
			}
		}
#endif

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _TARGET_STATUS adapter %d \n", adapterNum);

		printk(KERN_INFO "is1000: pTgtStatus->dwStatus %d      ->targetId %d\n",
		       (int)pTgtStatus->dwStatus, (int)pTgtStatus->sTarget.targetId);
		printk(KERN_INFO "is1000:     ->sTarget.address.ip 0x%x port %d  portalgroup %d\n",
		       (int)pTgtStatus->sTarget.address.ip, 
		       (int)pTgtStatus->sTarget.address.port,
		       (int)pTgtStatus->sTarget.address.portalgroup);
		printk(KERN_INFO "is1000:     ->sTarget.iscsi_nodename: %s\n",
		       pTgtStatus->sTarget.iscsi_nodename);
#endif
	
		memset (resBlk, 0, sizeof (resBlk));

		*pBufList++ = ISCSI_ISM_TARGET_TABLE_STATUS_IDX;
		*pBufList++ = 1; /* key count */

		/* append portal group tag to target name */
		len = strlen(pTgtStatus->sTarget.iscsi_nodename); 
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: strlen iscsi_nodename %d\n",
		       len);
#endif
		memset ((u8 *)pBufList, 0, sizeof(pTgtStatus->sTarget.iscsi_nodename));
		memcpy((u8 *)pBufList, pTgtStatus->sTarget.iscsi_nodename, 
		       len);

		*(((char *)pBufList) + len) = '~';
				
		sprintf (&((u8 *)pBufList)[len+1], "%08x%04x", pTgtStatus->sTarget.address.ip,
			 pTgtStatus->sTarget.address.port);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pBuflist <%s>  strlen %d\n", (char *)pBufList,
		       strlen((char *)pBufList));
#endif
		res = i2o_query_table(I2O_PARAMS_LIST_GET, 
				      c, 8, 
				      ISCSI_ISM_TARGET_TABLE_GROUP_NO,
				      1, pBuffer, (ISCSI_TARGET_NAME_LENGTH - 1) + 8,
				      resBlk, sizeof(resBlk));

		/* 
		 * This is potentially a problem, although the timeout time is very
		 * long now. If the call timed out, and is finishing later, we're unlocking
		 * access to controller now. Most likely the scsi->detect still has a
		 * use count anyway, so quite unlikely to be a problem.
		 */
		//		i2o_unlock_controller(c);
		//		c = 0;

		if (res == -ETIMEDOUT) {
#ifdef IOCTLDEBUG
		        printk(KERN_INFO "is1000: NEED TO KNOW IF THIS HAPPENS:\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
			printk(KERN_INFO "is1000: adapter access timed out, returning _PENDING\n");
#endif
			pTgtStatus->dwStatus = CONNECTION_PENDING;
		}
		else if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d\n", res);
			goto outFail;
		}
		
		connStatus = resBlk[2];

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: resBlk[0] 0x%x resBlk[1] 0x%x resBlk[2] 0x%x (status) resBlk[3] 0x%x\n",
		       resBlk[0], resBlk[1], resBlk[2], resBlk[3]);
#endif
		if ((resBlk[0] == 0) && (connStatus == 0))
		{  /* TEST HACK TEST HACK  - how to test for status when target not added yet */
			printk(KERN_ERR "is1000: Asking for status on non-existent target %d\n",
			       pTgtStatus->sTarget.targetId);
			pTgtStatus->dwStatus = CONNECTION_NO_MATCH;
		}
		else if (connStatus == CONNECTION_NOT_ENABLED)
		{
			printk(KERN_ERR "is1000: Asking for status on un-enabled target %d\n",
			       pTgtStatus->sTarget.targetId);
			pTgtStatus->dwStatus = CONNECTION_NOT_ENABLED;
		}
		else
		{
			/* if the status is success or failure, for whatever reason, I'm still
			   expecting an lct update, so wait for a tid match and then return
			   whatever the status is.
			*/
			//			pTgtStatus->dwStatus = resBlk[3];

			/* race condition with firmware here - until failed or succeeded,
			 * will it work to return pending?
			 */
			tid = is1000_tid_match(&pTgtStatus->sTarget.targetId,
					       (char *)pBufList,
					       c->unit);
#ifdef IOCTLDEBUG
			printk(KERN_ERR "is1000: pTgtStatus->sTarget.targetId %d    tid %d\n",
			       pTgtStatus->sTarget.targetId, tid);
#endif
			if (tid > 0)
			{
#ifdef IOCTLDEBUG
				printk (KERN_INFO "is1000: found tid %d, with target status %d\n",
					tid, connStatus);
#endif
				pTgtStatus->dwStatus = connStatus;
			}
			else if (tid == 0)
			{
				printk (KERN_INFO "is1000: found tid %d, with target status %d\n",
					tid, connStatus);
				pTgtStatus->dwStatus = CONNECTION_FAILED_LOGIN;
			}
			else
			{
#ifdef IOCTLDEBUG
				printk (KERN_INFO 
					"is1000: not tid yet, target status %d, --> returning _PENDING\n",
					connStatus);
				printk (KERN_INFO 
					"        but got targetId %d\n", pTgtStatus->sTarget.targetId);
#endif

				pTgtStatus->dwStatus = CONNECTION_PENDING;
			}
		}
		if (copy_to_user((void *)arg, pTgtStatus,
				 sizeof(ISCSI_STORAGE_TARGET_INFO)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_IOCTL_STORAGE_TARGET_STATUS\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_GET_DIAGNOSTICS : 
	{
	        PISCSI_DIAGNOSTICS pDiag = (PISCSI_DIAGNOSTICS) pArgData;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "\nis1000: ISCSI_IOCTL_GET_DIAGNOSTICS:\n");
#endif
		if (copy_from_user(pDiag,(char *)arg, sizeof(ISCSI_DIAGNOSTICS))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pDiag->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _DIAGNOSTICS adapter %d \n", adapterNum);
#endif

		res =  i2o_query_scalar(c, 8,
					ISCSI_ISM_DIAGNOSTIC_GROUP_NO,
					0,
					&pDiag->diagValue,
					sizeof(u32));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}

		/* This is the value to use if possible to record a reset
		 * here in the driver: ISCSI_DIAG_RESET_IN_PROGRESS
		 */
		pDiag->diagValue |= iscsiDiagnostic;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pDiag->diaValue 0x%08x \n", 
		       pDiag->diagValue);
#endif

		if (copy_to_user((void *)&((PISCSI_DIAGNOSTICS)arg)->diagValue, 
				 &pDiag->diagValue, sizeof(u32)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_DIAGNOSTICS\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_GET_STATISTICS: 
	{
		PISCSI_ADAPTER_STATISTICS pStats = (PISCSI_ADAPTER_STATISTICS) pArgData;
		int opBlock;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "\nis1000: ISCSI_IOCTL_GET_STATISTICS:\n");
#endif
		if (copy_from_user(pStats,(char *)arg, sizeof(ISCSI_ADAPTER_STATISTICS))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pStats->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _STATISTICS adapter %d \n", adapterNum);
#endif

		if (pStats->dwStatus == 4)
		{
			/* reset statistics */
			opBlock = ISCSI_PARAM_RESET_STATS;
			res = i2o_set_scalar(c, 8,
					     ISCSI_ISM_HOST_SPECIFIED_OPERATION_GROUP_NO,
					     ISCSI_ISM_HOST_SPECIFIC_OP_OPERATION_IDX,
					     &opBlock, sizeof(opBlock));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif
			pStats->dwStatus = IOCTL_STATUS_SUCCESS;
		}
		else if (pStats->dwStatus == 0)
		{
#ifdef IOCTLDEBUG
			memset(&pStats->stats, 'e', sizeof(IS_SESSION_HBA_STATS));
#endif	    
			/* get statistics */
			res =  i2o_query_scalar(c, 8,
						ISCSI_ISM_EXTENDED_HBA_STATS_GROUP_NO,
						-1,
						&pStats->stats,
						sizeof(IS_SESSION_HBA_STATS));
			if (res < 0) {
				printk(KERN_ERR "is1000: adapter access failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: adapter access success %d\n", res);
#endif
		}
		else {
			printk(KERN_ERR "is1000: Ioctl - Unknown statistics request %d\n",
			       (int)pStats->dwStatus);
			goto outFail;	 
		}

		if (copy_to_user((PISCSI_ADAPTER_STATISTICS)arg, pStats,
				 sizeof(ISCSI_ADAPTER_STATISTICS)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_STATISTICS\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_GET_TARGET_STATISTICS :
	{
		PISCSI_GET_TARGET_STATISTICS pStats = (PISCSI_GET_TARGET_STATISTICS) pArgData;
		int tid;
		//	    struct i2o_scsi_host *h;
		PISCSI_TID_DATA pTidData;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "\nis1000: ISCSI_IOCTL_GET_TARGET_STATISTICS:\n");
#endif
		if (copy_from_user(pStats,(char *)arg, sizeof(ISCSI_GET_TARGET_STATISTICS))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pStats->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: GET_TARGET_STATS adapter %d \n", adapterNum);
#endif


		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pStats->targetId)))
		{
			goto outFail;
		}

		tid = pTidData->tid;

		if (tid == -1)
		{
			printk (KERN_ERR "is1000: No targetId %d found on adapter %d\n",
				pStats->targetId, adapterNum);
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk (KERN_ERR "is1000: Getting target stats for targetId %d and tid %d\n",
			pStats->targetId, tid);
		memset(&pStats->stats, 'e', sizeof(IS_SESSION_STATS));
#endif	    

		res =  i2o_query_scalar(c, tid,
					ISCSI_ISM_EXTENDED_STATS_GROUP_NO,
					-1,
					&pStats->stats,
					sizeof(IS_SESSION_STATS));
		if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d\n", res);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: adapter access success %d\n", res);
#endif

		if (copy_to_user((PISCSI_GET_TARGET_STATISTICS)arg, pStats,
				 sizeof(ISCSI_GET_TARGET_STATISTICS)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for _GET_TARGET_STATISTICS\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_BLINK_LED : 
	{  
		ISCSI_BLINK_LED *pBlink = (PISCSI_BLINK_LED) pArgData;
		int opVal;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "\nis1000: ISCSI_IOCTL_BLINK_LED:\n");
#endif
		if (copy_from_user(pBlink,(char *)arg, sizeof(ISCSI_BLINK_LED))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pBlink->iSeconds %d\n",
		       pBlink->iSeconds);
#endif

		if (!(c = iscsi_find_adapter(&adapterNum, &pBlink->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _DIAGNOSTICS adapter %d \n", adapterNum);
#endif

		opVal = pBlink->iSeconds;
		res =  i2o_set_scalar(c, 8,
				      ISCSI_ISM_HOST_SPECIFIED_OPERATION_GROUP_NO, 
				      ISCSI_ISM_HOST_SPECIFIC_OP_PARAM_IDX, 
				      &opVal, sizeof(opVal));

		if (res < 0) {
			printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		else {
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
		}
#endif

		/* write blink led command to operation field */

		opVal = ISCSI_PARAM_BLINK_LEDS;
		res =  i2o_set_scalar(c, 8,
				      ISCSI_ISM_HOST_SPECIFIED_OPERATION_GROUP_NO, 
				      ISCSI_ISM_HOST_SPECIFIC_OP_OPERATION_IDX,
				      &opVal, sizeof(opVal));

		if (res < 0) {
			printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		else {
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
		}
#endif

	}
	break;
	case ISCSI_IOCTL_GET_ADAPTER_PROPERTIES : 
	{  
		I2O_ISCSI_BOARD_PROPERTIES *pProp = (PI2O_ISCSI_BOARD_PROPERTIES) pBuffer;
		ISCSI_GET_ADAPTER_PROPERTIES *pAdapter = (PISCSI_GET_ADAPTER_PROPERTIES) pArgData;
		u16 *pOpBlock;
		char opBlock[38];
		int *pInt;
		PI2O_ISCSI_NET_IF_FIELDS2 pNetIf;
		PI2O_ISCSI_NET_IF_FIELDS pNetIfIp;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "\nis1000: ISCSI_IOCTL_GET_ADAPTER_PROPERTIES:\n");
#endif
		if (copy_from_user(pAdapter,(char *)arg, sizeof(ISCSI_GET_ADAPTER_PROPERTIES))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pAdapter->MACAddr)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: incoming pAdapter->props.Status %d  adapternum %d\n", 
		       pAdapter->props.Status, adapterNum);
#endif

		res =  i2o_query_scalar(c, 8, 
					ISCSI_ISM_BOARD_PROPERTIES_GROUP_NO,
					-1,
					pProp, sizeof(I2O_ISCSI_BOARD_PROPERTIES));
		if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d\n", res);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: adapter access success %d\n", res);
#endif
		pAdapter->props.ProcessorID = pProp->chipNum;

		pAdapter->props.BoardRevisionNumber = pProp->boardRev;
		pAdapter->props.CPLDRevisionNumber = pProp->cpldRev;
		pAdapter->props.ProcessorRevision = pProp->processorRev;
		pAdapter->props.AdapterFirmwareVersion = pProp->adapFirmRev;
		pAdapter->props.AdapterRTOSVersion = pProp->adapRtosRev;

		/* retrieve data from private adapter struct */
		memcpy(pAdapter->props.s.iscsi_nodename, iscsi_adapter_priv[adapterNum].init_name,
		       IS_MAX_NODENAME_STRING - 1);
		pAdapter->props.s.UseDhcp = iscsi_adapter_priv[adapterNum].useDhcp;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: getting ip addr, netmask, gateway from adapter\n"); 
#endif
		/* Now get interface and link related adapter properties */

		pOpBlock = (u16 *)&opBlock;

		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_KEY_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_IPADDR_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_SUBNET_MASK_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_GATEWAY_IDX;

		*pOpBlock++ = 1; /* key count */
		memcpy(pOpBlock, &iscsi_adapter_priv[adapterNum].ifTableKey, sizeof(int));

		res = i2o_query_table (I2O_PARAMS_LIST_GET,
				       c,
				       8 /* need to get legitimately */,
				       ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO, 
				       4, 
				       &opBlock, 22,
				       pBuffer, sizeof(I2O_ISCSI_NET_IF_FIELDS) + 16);
	   
		if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d, adapter number %d\n", 
			       res, adapterNum);
			goto outFail;	 
		}

		pInt = (int *)pBuffer;
		if (pInt[1] & 0x00ff0000)
		{
#ifdef IOCTLDEBUG
			printk(KERN_ERR "is1000: Error getting DHCP info from  adapter %d  0x%08x\n", 
			       adapterNum, pInt[1]);
#endif
			pAdapter->props.Status = ISCSI_ADAPTER_CONFIG_FAILED;

			if (copy_to_user((void *)&(((PISCSI_GET_ADAPTER_PROPERTIES)arg)->props.Status), 
					 &pAdapter->props.Status, sizeof(pAdapter->props.Status)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _GET_ADAPTER_PROPERTIES\n");
				goto outFail;
			}
			break;
		}

		pNetIfIp = (PI2O_ISCSI_NET_IF_FIELDS) (pBuffer + 12);

		pAdapter->props.s.AdapterIPAddress = inet_addr(pNetIfIp->ipAddress);
		pAdapter->props.s.AdapterIPMask = pNetIfIp->subnetMask;
		pAdapter->props.s.AdapterGateway = inet_addr(pNetIfIp->gateway);

		pOpBlock = (u16 *)&opBlock;

		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_KEY_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_MTU_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_ETHER_ID_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_ETHER_REV_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_SPEED_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_DUPLEX_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_LINK_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_TXPAUSED_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_CHIPINIT_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_AUTONEG_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_NEGMASK_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_FLOWCTRL_IDX;

		*pOpBlock++ = 1; /* key count */
		memcpy(pOpBlock, &iscsi_adapter_priv[adapterNum].ifTableKey, sizeof(int));

		res = i2o_query_table (I2O_PARAMS_LIST_GET,
				       c,
				       8 /* need to get legitimately */,
				       ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO, 
				       12,
				       &opBlock, 38,
				       pBuffer, sizeof(I2O_ISCSI_NET_IF_FIELDS2) + 12);

		if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d\n", res);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: adapter access success %d\n", res);
#endif

		pNetIf = (PI2O_ISCSI_NET_IF_FIELDS2) ((char *)pBuffer + 12);

		pAdapter->props.EthernetDeviceID = pNetIf->etherDevId & 0x0000ffff;

		/* We don't know how to get for real. eab
		   pAdapter->EthernetRevision = pNetIf->etherRev; */
		pAdapter->props.EthernetRevision = 0;
		pAdapter->props.LinkSpeed = pNetIf->linkSpeed / 1000000;
		pAdapter->props.IsFullDuplex = pNetIf->duplex;

		pAdapter->props.s.FrameSize = pNetIf->mtu;
		pAdapter->props.s.ForcedLinkSpeed = pNetIf->linkSpeed / 1000000;
		pAdapter->props.s.ForcedFullDuplex = pNetIf->duplex;

		pAdapter->props.HasLink = pNetIf->linkConn;
		pAdapter->props.TransmissionPaused = pNetIf->txPaused;
		pAdapter->props.EthernetInitialized = pNetIf->chipInit;
		pAdapter->props.s.AutoNegEnabled = pNetIf->autoNeg;
		if (!(pNetIf->autoNegMask & MII_TECH_10BASE_T))
		{
			pAdapter->props.s.Disallow10Half = 1;
		}
		if (!(pNetIf->autoNegMask & MII_TECH_10BASE_FD))
		{
			pAdapter->props.s.Disallow10Full = 1;
		}
		if (!(pNetIf->autoNegMask & MII_TECH_100BASE_TX))
		{
			pAdapter->props.s.Disallow100Half = 1;
		}
		if (!(pNetIf->autoNegMask & MII_TECH_100BASE_TX_FD))
		{
			pAdapter->props.s.Disallow100Full = 1;
		}
		if (!(pNetIf->autoNegMask & MII_PHY_1000T_FD))
		{
			pAdapter->props.s.Disallow1000Full = 1;
		}

		if (pNetIf->flowControl == 1)
		{
			pAdapter->props.s.FlowControlEnabled = 1;
		}

		pAdapter->props.MaxTargets = IS_MAX_CONNECTIONS;

		/* Get the locally administred mac from our cache */
		memcpy (&pAdapter->props.s.LAA_Address,
			&iscsi_adapter_priv[adapterNum].currentMacAddr, 
			MAC_ADDR_LENGTH);
#ifdef IOCTLDEBUG
		printk(KERN_INFO 
		       "is1000: returning LAA_Address 0x%08x 0x%08x\n",
      		       pAdapter->props.s.LAA_Address.first, pAdapter->props.s.LAA_Address.second);
		printk(KERN_INFO "is1000: pNetIf->autoNeg 0x%08x\n", pNetIf->autoNeg);
		printk(KERN_INFO "is1000: pNetIf->autoNegMask 0x%08x\n", pNetIf->autoNegMask);
		printk(KERN_INFO "is1000: pNetIf->flowControl 0x%08x\n", pNetIf->flowControl);
#endif

		if (copy_to_user((void *)arg, pAdapter,
				 sizeof(ISCSI_GET_ADAPTER_PROPERTIES)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for ISCSI_IOCTL_ADAPTER_PROPERTIES\n");
			goto outFail;
		}
	}
	break;

	case ISCSI_IOCTL_UPDATE_FLASH : 
		if (!ioctl_swdl(arg))
			goto outFail;

		break;

	case ISCSI_IOCTL_ADD_TARGET :
	{
		PISCSI_STORAGE_TARGET_INFO pTarget = 
			(PISCSI_STORAGE_TARGET_INFO) pArgData;
		PI2O_ISCSI_TARGET_TABLE pTargetTable;
		u16 *pBufList = (u16 *)pBuffer;
		u32 nIPAddress;
		int len;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_ADD_TARGET\n");
#endif	
		if (copy_from_user(pTarget,(char *)arg, sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTarget->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "* * * * pTarget->dwStatus %d\n\n", pTarget->dwStatus);
		printk(KERN_INFO "   --> targetId %d\n", pTarget->sTarget.targetId);
		printk(KERN_INFO "IPAddress %x  port %d\n",
		       pTarget->sTarget.address.ip, pTarget->sTarget.address.port);
		printk(KERN_INFO "target name: %s\n",
		       pTarget->sTarget.iscsi_nodename);
		printk(KERN_INFO "useChap %d  CHAPusename %s   CHAPpassword %s\n",
		       pTarget->sTarget.chap.useCHAP,
		       pTarget->sTarget.chap.CHAPusername,
		       pTarget->sTarget.chap.CHAPpassword);
		printk(KERN_INFO "useChap %d  CHAPusename %s   CHAPpassword %s\n",
		       pTarget->sTarget.chap.useInboundCHAP,
		       pTarget->sTarget.chap.inboundCHAPusername,
		       pTarget->sTarget.chap.inboundCHAPpassword);
		printk(KERN_INFO "     pTarget->sTarget.timeout %d  .retries %d\n",
		       pTarget->sTarget.timeout, pTarget->sTarget.retries);
		printk(KERN_INFO "     pTarget->sTarget.crcMode: header? %d  data? %d  (%x)\n",
		       (pTarget->sTarget.crcMode & 0x1) ? 1 : 0,
		       (pTarget->sTarget.crcMode & 0x2) ? 1 : 0,
		       pTarget->sTarget.crcMode);
#endif

		pTargetTable = (PI2O_ISCSI_TARGET_TABLE)
			kmalloc(sizeof(I2O_ISCSI_TARGET_TABLE), GFP_KERNEL);
		if (pTargetTable == NULL)
		{
			printk(KERN_ERR "is1000: no memory for ioctl buffer.\n");
			goto outFail;
		}
		memset (pTargetTable, 0, sizeof(I2O_ISCSI_TARGET_TABLE));

		pTargetTable->port = pTarget->sTarget.address.port;

		nIPAddress = ntohl(pTarget->sTarget.address.ip);
		inet_ntoa(nIPAddress,pTargetTable->ipAddress);

		len = strlen(pTarget->sTarget.iscsi_nodename); 
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: strlen iscsi_nodename %d\n",
		       len);
#endif
		memset(pTargetTable->key, 0, sizeof(pTargetTable->key));
		strncpy(pTargetTable->key, ((u8 *)pTarget->sTarget.iscsi_nodename), 
			IS_MAX_NODENAME_STRING - 1); /* could be len here */

		pTargetTable->key[len] = '~';
		
		sprintf (&pTargetTable->key[len+1], "%08x%04x", pTarget->sTarget.address.ip,
			 pTarget->sTarget.address.port);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pTargetTable->key <%s>\n", pTargetTable->key);
#endif
		strncpy(pTargetTable->initName, iscsi_adapter_priv[adapterNum].init_name,
			sizeof(iscsi_adapter_priv[adapterNum].init_name));

		pTargetTable->timeout = pTarget->sTarget.timeout * 1000 * 1000;

		pTargetTable->retries = pTarget->sTarget.retries;

		pTargetTable->headerCrc = (pTarget->sTarget.crcMode & 0x1) ? 1 : 0;
		pTargetTable->dataCrc = (pTarget->sTarget.crcMode & 0x2) ? 1 : 0;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "initName from priv cache: %s\n", pTargetTable->initName);
#endif
		*pBufList++ = ISCSI_ISM_TARGET_TABLE_KEY_IDX;
		*pBufList++ = 2; /* IP address */
		*pBufList++ = 3; /* port */
		*pBufList++ = 8; /* connect on boot */
		*pBufList++ = 9; /* init name */
		*pBufList++ = 17; /* reconnect time */
		*pBufList++ = 18; /* reconnect threshold count */
		*pBufList++ = 20; /* header crc */
		*pBufList++ = 21; /* data crc */
		*pBufList++ = 1; /* row count */
		memcpy(pBufList, pTargetTable, sizeof(*pTargetTable));

		/* send the add row request to the adapter */
		res = i2o_row_add_table(c, 8,
					ISCSI_ISM_TARGET_TABLE_GROUP_NO, 9, 
					pBuffer, sizeof(*pTargetTable) + 12 + 12);

		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to add target: %s\n", pTargetTable->key);
			kfree(pTargetTable);
			goto outFail;	    
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Successfully added target: %s   buflen %d  res %d\n", 
		       pTargetTable->key, sizeof(*pTargetTable) + 24, res);
#endif

		if (pTarget->sTarget.targetId != IS_TARGETID_INVALID)
		{
			struct i2o_scsi_host *h = is1000_get_hostdata(adapterNum);
		      
			if (h == NULL)
			{
				printk(KERN_ERR "is1000: _ADD_TARGET found NULL i2o_scsi_host *\n");
				kfree(pTargetTable);
				goto outFail;	    
			}

			if (h->aTaskData[pTarget->sTarget.targetId].inUse == 1)
			{
				printk(KERN_ERR"is1000: Reusing target id %d?\n", 
				       pTarget->sTarget.targetId);
			}
			/* Reserved this target id for when the tid etc happen */
			h->aTaskData[pTarget->sTarget.targetId].inUse = 1;
		}
		kfree(pTargetTable);

	}
	break;

	case ISCSI_IOCTL_SET_ADAPTER_PROPERTIES :
	{
	        I2O_ISCSI_NET_IF_ADD_FIELDS ifTable, *pIfTable;
		PISCSI_SET_ADAPTER_PROPERTIES pAdapter = 
			(PISCSI_SET_ADAPTER_PROPERTIES) pArgData;
		int resBlk[10];
		u16 *pBufList = (u16 *)pBuffer;
		char initNameData[ISCSI_INIT_NAME_LENGTH - 1];

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_SET_ADAPTER_PROPERTIES\n");
#endif	

		pIfTable = &ifTable;

		if (copy_from_user(pAdapter,(char *)arg, sizeof(ISCSI_SET_ADAPTER_PROPERTIES))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pAdapter->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Setting props on adapter %d \n", adapterNum);

		printk(KERN_INFO "MAC address = %X%X    LAA_Address = %X%X\n", 
		       (int)pAdapter->MACAddr.first, 
		       (int)pAdapter->MACAddr.second,
		       (int)pAdapter->props.LAA_Address.first, 
		       (int)pAdapter->props.LAA_Address.second);
		printk(KERN_INFO "iscsi_nodename: <%s>   %X  %X\n",
		       pAdapter->props.iscsi_nodename, pAdapter->props.AdapterIPAddress,
		       pAdapter->props.AdapterIPMask);
		printk(KERN_INFO "UseDhcp %d  FrameSize %d \n",
		       pAdapter->props.UseDhcp, pAdapter->props.FrameSize);
		printk(KERN_INFO "AutoNegEnabled %d  FlowControlEnabled %d \n",
		       pAdapter->props.AutoNegEnabled, pAdapter->props.FlowControlEnabled);
		printk(KERN_INFO "ForcedLinkSpeed %d  ForcedFullDuplex %d \n",
		       pAdapter->props.ForcedLinkSpeed, pAdapter->props.ForcedFullDuplex);
		printk(KERN_INFO "iscsi_adapter_priv[adapterNum].ifTableKey %X\n",
		       iscsi_adapter_priv[adapterNum].ifTableKey);

#endif
		pIfTable->key = iscsi_adapter_priv[adapterNum].ifTableKey;

		pAdapter->props.AdapterIPAddress = htonl(pAdapter->props.AdapterIPAddress);
		inet_ntoa(pAdapter->props.AdapterIPAddress, pIfTable->ipAddress);
		pIfTable->subnetMask = pAdapter->props.AdapterIPMask;

		pAdapter->props.AdapterGateway = htonl(pAdapter->props.AdapterGateway);
		inet_ntoa(pAdapter->props.AdapterGateway, pIfTable->gateway);
		pIfTable->mtu = pAdapter->props.FrameSize;
		pIfTable->index = 0; /* always zero on san aug cards */
		pIfTable->dhcp = pAdapter->props.UseDhcp;
		
		/* Not needed, but harmless. Always 0 for Intel PRO 1000 T IP */
		pIfTable->configureOnBoot = 0;

		if ((pAdapter->props.LAA_Address.first != 0) &&
		    (pAdapter->props.LAA_Address.second != 0))
		{	
#ifdef IOCTLDEBUG
		        printk(KERN_INFO "Using LAA_Address\n");
#endif
			memcpy(&pIfTable->macAddr, &pAdapter->props.LAA_Address,
			       MAC_ADDR_LENGTH);
			memcpy(&iscsi_adapter_priv[adapterNum].currentMacAddr, 
			       &pAdapter->props.LAA_Address, MAC_ADDR_LENGTH);
		}
		else
		{
#ifdef IOCTLDEBUG
		        printk(KERN_INFO "Not using LAA_Address\n");
#endif
			memcpy(&pIfTable->macAddr, &pAdapter->MACAddr,
			       MAC_ADDR_LENGTH);
			memset(&iscsi_adapter_priv[adapterNum].currentMacAddr, 
			       0, MAC_ADDR_LENGTH);
		}

		if (pAdapter->props.FlowControlEnabled)
		{
			pIfTable->flowControl = 1;
		}
		else
		{
			pIfTable->flowControl = 0;
		}
    
		/* assignment and test going on here */
		if ((pIfTable->autoNeg = pAdapter->props.AutoNegEnabled))
		{
			u32 mask = ALL_MII_BITS;
			pIfTable->duplex = -1; // don't write to field 
			pIfTable->linkSpeed = -1; // don't write to field
			pIfTable->autoNeg = 1; // in case it's other than 1

			if (pAdapter->props.Disallow10Half)
			{
				mask &= ~MII_TECH_10BASE_T;
			}
			if (pAdapter->props.Disallow10Full)
			{
				mask &= ~MII_TECH_10BASE_FD;
			}
			if (pAdapter->props.Disallow100Half)
			{
				mask &= ~MII_TECH_100BASE_TX;
			}
			if (pAdapter->props.Disallow100Full)
			{
				mask &= ~MII_TECH_100BASE_TX_FD;
			}
			if (pAdapter->props.Disallow1000Full)
			{
				mask &= ~MII_PHY_1000T_FD;
			}

			pIfTable->autoNegMask = mask;                   

#ifdef IOCTLDEBUG
			printk(KERN_INFO "autoneg mask = %X\n", mask); 
#endif

		}
		else /* implies pIfTable->autoNeg == 0 meaning no auto neg */
		{
			pIfTable->autoNegMask = -1; // don't write to field
        
			/* Leaving the intelligence to the user or perhaps user space? */
			pIfTable->linkSpeed = pAdapter->props.ForcedLinkSpeed * 1000000;
			pIfTable->duplex = pAdapter->props.ForcedFullDuplex;
		}

		pIfTable->commit = 1;

		*pBufList++ = ISCSI_ISM_NETIF_TABLE_KEY_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_IPADDR_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_SUBNET_MASK_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_GATEWAY_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_MTU_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_INDEX_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_DHCP_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_CONFIG_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_MACADDR_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_SPEED_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_DUPLEX_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_AUTONEG_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_NEGMASK_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_FLOWCTRL_IDX;
		*pBufList++ = ISCSI_ISM_NETIF_TABLE_COMMIT_IDX;
		*pBufList++ = 1; /* row count */
		memcpy(pBufList, pIfTable, sizeof(ifTable));

		res = i2o_set_list_table(I2O_PARAMS_LIST_SET, 
					 c,
					 8, 
					 ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO,
					 15,
					 pBuffer, sizeof(ifTable)+ 16 + 32,
					 resBlk, sizeof(resBlk));
		if (res < 0) {
			printk(KERN_ERR "is1000: Error initializing adapter %d\n", adapterNum);
			goto outFail;
		}
		else if (resBlk[1] & 0x00ff0000)
		{
			printk(KERN_ERR "is1000: Error initializing adapter %d  resBlk 0x%08x  res %d\n", 
			       adapterNum, resBlk[1], res);
			printk(KERN_ERR "is1000:  0x%08x  0x%08x  0x%08x  0x%08x\n", 
			       resBlk[2], resBlk[3], resBlk[4], resBlk[5]);
			if (pAdapter->props.UseDhcp != 0)
				pAdapter->dwStatus = ISCSI_ADAPTER_CONFIG_DHCP_FAILED;
			else
				pAdapter->dwStatus = ISCSI_ADAPTER_CONFIG_FAILED;

			if (copy_to_user((void *)&((PISCSI_SET_ADAPTER_PROPERTIES)arg)->dwStatus, 
					 &pAdapter->dwStatus,sizeof(pAdapter->dwStatus)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _SET_ADAPTER_PROPERTIES\n");
				goto outFail;
			}
			break;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: i2o_set_list_table success %d\n", res); 
#endif
		/* Save whether we use DHCP or not. We'll retrieve the IP, mask
		 * and gateway fresh everytime so don't cache that anymore.
		 */
		iscsi_adapter_priv[adapterNum].useDhcp = pAdapter->props.UseDhcp;

		memset(initNameData, 0, sizeof(initNameData));

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: sizeof iscsi_nodename %d\n",
		       sizeof(pAdapter->props.iscsi_nodename) - 1);
#endif
		memcpy(initNameData, pAdapter->props.iscsi_nodename,
		       ISCSI_INIT_NAME_LENGTH - 1);
		res =  i2o_set_scalar(c, 8,
				      ISCSI_ISM_HBA_SETTINGS_GROUP_NO, 
				      ISCSI_ISM_HBA_SEL_NAME_IDX, 
				      initNameData, sizeof(initNameData));

		if (res < 0) {
			printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
			goto outFail;	 
		}
		/* save adapter's initiator name for use in target login */
		memcpy(iscsi_adapter_priv[adapterNum].init_name,
		       initNameData, ISCSI_INIT_NAME_LENGTH - 1);

		pAdapter->dwStatus = ISCSI_ADAPTER_CONFIG_SUCCESS;

		if (copy_to_user((void *)&((PISCSI_SET_ADAPTER_PROPERTIES)arg)->dwStatus, 
				 &pAdapter->dwStatus,sizeof(pAdapter->dwStatus)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for _SET_ADAPTER_PROPERTIES\n");
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: storing init name: <%s>\n",
		       iscsi_adapter_priv[adapterNum].init_name);
		
		printk(KERN_INFO 
		       "is1000: is1000_set_scalar success, exiting _SET_ADAPTER_PROPERTIES %d\n\n", 
		       res);
#endif
	}
	break;

	case ISCSI_IOCTL_GET_STORAGE_TARGET_COUNT :
	{
		PISCSI_GET_STORAGE_TARGET_COUNT pTargetCount = 
			(PISCSI_GET_STORAGE_TARGET_COUNT) pArgData;
		u32 intField;
		u16 shortField;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_GET_STORAGE_TARGET_COUNT\n");
#endif	
		if (copy_from_user(pTargetCount,(char *)arg, sizeof(ISCSI_GET_STORAGE_TARGET_COUNT))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTargetCount->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pTargetCount->IPAddress %X %d  status %d\n", pTargetCount->IPAddress,
		       pTargetCount->port, pTargetCount->status);
		if (pTargetCount->chap.useCHAP)
			printk(KERN_INFO "useChap %d  CHAPusename %s   CHAPpassword %s\n",
			       pTargetCount->chap.useCHAP,
			       pTargetCount->chap.CHAPusername,
			       pTargetCount->chap.CHAPpassword);
		if (pTargetCount->chap.useInboundCHAP)		
			printk(KERN_INFO "useChap %d  CHAPusename %s   CHAPpassword %s\n",
			       pTargetCount->chap.useInboundCHAP,
			       pTargetCount->chap.inboundCHAPusername,
			       pTargetCount->chap.inboundCHAPpassword);
#endif

		if (pTargetCount->status == ENUMERATION_STATUS_SUCCESS)
		{
			u8 * pIpAddr;
			u16 *opBlock = (u16 *)pBuffer;
	    
			/* clear old targets from previous enum */
			memset (pEnumTargets, 0,
				MAX_TARGETS_PER_ENTITY * sizeof(ISCSI_TOTAL_ENUM_TARGETS));

			memcpy(&((char *)pEnumTargets)[MAX_TARGETS_PER_ENTITY * 
						       sizeof(ISCSI_TOTAL_ENUM_TARGETS)],
			       &pTargetCount->IPAddress, sizeof(u32));
			memcpy(&((char *)pEnumTargets)[MAX_TARGETS_PER_ENTITY * 
						       sizeof(ISCSI_TOTAL_ENUM_TARGETS) + sizeof(u32)],
			       &pTargetCount->port, sizeof(u16));


			intField = pTargetCount->port;
			res =  i2o_set_scalar(c, 8,
					      ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					      ISCSI_ISM_TARGET_SCAN_SEL_PORT_IDX, 
					      &intField, sizeof(intField));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			pIpAddr = (u8 *) &pTargetCount->IPAddress;
        
			sprintf((char *) opBlock, "%d.%d.%d.%d", pIpAddr[3] & 0xff, 
				pIpAddr[2] & 0xff, pIpAddr[1] & 0xff, pIpAddr[0] & 0xff);  

			res =  i2o_set_scalar(c, 8,
					      ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					      ISCSI_ISM_TARGET_SCAN_SEL_IPADDR_IDX,
					      opBlock, ISCSI_TARGET_ADDRESS_LENGTH - 1);

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			if (pTargetCount->chap.useCHAP)
			{
				/* If we are to use chap, copy the name and password */
				res = i2o_set_scalar(c, 8,
						     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
						     ISCSI_ISM_TARGET_SCAN_SEL_CHAP_NAME_IDX,
						     pTargetCount->chap.CHAPusername, 
						     ISCSI_CHAP_FIELD_LENGTH);

				if (res < 0) {
					printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
					goto outFail;	 
				}
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

				res = i2o_set_scalar(c, 8,
						     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
						     ISCSI_ISM_TARGET_SCAN_SEL_CHAP_PW_IDX,
						     pTargetCount->chap.CHAPpassword, 
						     ISCSI_CHAP_FIELD_LENGTH);

				if (res < 0) {
					printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
					goto outFail;	 
				}
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			}

			shortField = pTargetCount->chap.useCHAP;
			res = i2o_set_scalar(c, 8,
					     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					     ISCSI_ISM_TARGET_SCAN_SEL_CHAP_ENABLE_IDX,
					     &shortField, sizeof(shortField));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			if (pTargetCount->chap.useInboundCHAP)
			{
				/* If we are to use inbound chap, copy the name and password */
				/* If we are to use chap, copy the name and password */
				res = i2o_set_scalar(c, 8,
						     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
						     ISCSI_ISM_TARGET_SCAN_SEL_TGT_CHAP_NAME_IDX,
						     pTargetCount->chap.inboundCHAPusername, 
						     ISCSI_CHAP_FIELD_LENGTH);

				if (res < 0) {
					printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
					goto outFail;	 
				}
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

				res = i2o_set_scalar(c, 8,
						     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
						     ISCSI_ISM_TARGET_SCAN_SEL_TGT_CHAP_PW_IDX,
						     pTargetCount->chap.inboundCHAPpassword, 
						     ISCSI_CHAP_FIELD_LENGTH);

				if (res < 0) {
					printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
					goto outFail;	 
				}
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif
			}

			shortField = pTargetCount->chap.useInboundCHAP;
			res = i2o_set_scalar(c, 8,
					     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					     ISCSI_ISM_TARGET_SCAN_SEL_AUTH_TARGET_IDX,
					     &shortField, sizeof(shortField));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			intField = ISCSI_DISC_INITIATE_DISC;
			res = i2o_set_scalar(c, 8,
					     ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					     ISCSI_ISM_TARGET_SCAN_SEL_POLL_SCAN_IDX,
					     &intField, sizeof(intField));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif

			/* Set all fields, now return pending status */
			pTargetCount->status = ENUMERATION_STATUS_PENDING;                          

			if (copy_to_user((void *)&((PISCSI_GET_STORAGE_TARGET_COUNT)arg)->status, 
					 &pTargetCount->status,
					 sizeof(pTargetCount->status)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _GET_STORAGE_TARGET_COUNT\n");
				goto outFail;
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: returning ENUMERATION_STATUS_PENDING\n");
#endif
		}
		
		else if (pTargetCount->status == ENUMERATION_STATUS_POLL)
		{

			res =  i2o_query_scalar(c, 8,
						ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
						ISCSI_ISM_TARGET_SCAN_SEL_POLL_SCAN_IDX, 
						&intField, sizeof(intField));

			if (res < 0) {
				printk(KERN_ERR "is1000: adapter access failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: Target scan poll success %d\n", res);
#endif

			if (intField == ISCSI_DISC_STATUS_SUCCESS)
			{
				int moreFlag = 0, currentAddress = 0, targetKey = 0;
				int *pInt;
				PISCSI_ENUM_TARGETS pTarget;
				char opBlock[14];
				u16 *pOpBlock;
				ISCSI_TOTAL_ENUM_TARGETS *pCurrentTarget = pEnumTargets;

				struct
				{
					u16 result_count;
					u16 pad;
					u16 block_size;
					u8  block_status;
					u8  error_info_size;
					u16 row_count;
					u16 more_flag;
					u8 target[300];
				} result;

				/* We're starting a new query, clear old count */
				numEnumTargets = 0;

				pOpBlock = (u16 *)&opBlock;
				*pOpBlock++ = ISCSI_ISM_TARGET_SCAN_TABLE_ISCSI_NAME;
				*pOpBlock++ = ISCSI_ISM_TARGET_SCAN_TABLE_IP_ADDR;
				*pOpBlock++ = ISCSI_ISM_TARGET_SCAN_TABLE_PORT;
				*pOpBlock++ = ISCSI_ISM_TARGET_SCAN_TABLE_TAG;
				
				do {
					if (moreFlag == 0)
					{
						res = i2o_query_table (I2O_PARAMS_TABLE_GET,
								       c,
								       8,
								       ISCSI_ISM_TARGET_SCAN_TABLE_GROUP_NO,
								       4, 
								       &opBlock, 8,
								       &result, //pBuffer, 
								       sizeof(result));

						if (res < 0)
						{
							printk(KERN_ERR "is1000: query failed %d to enumerate adapters\n", res);
							goto outFail;
						}
					}
					else /* do a MORE_GET */
					{
						*(u32 *)pOpBlock = targetKey - 1;

#ifdef IOCTLDEBUG
						printk(KERN_ERR "\nis1000: doing a _MORE_GET, previous key %d\n",
						       targetKey  - 1);
#endif
						res = i2o_query_table (I2O_PARAMS_MORE_GET,
								       c,
								       8,
								       ISCSI_ISM_TARGET_SCAN_TABLE_GROUP_NO,
								       4, 
								       &opBlock, 12,
								       &result, //pBuffer, 
								       sizeof(result));

						if (res < 0)
						{
							printk(KERN_ERR "is1000: query failed %d to enumerate adapters\n", res);
							goto outFail;
						}
					}

					/* targetKey is the previous key */
					targetKey++;
					moreFlag = result.more_flag;
					pInt = (int *)&result;
					pTarget = (ISCSI_ENUM_TARGETS *)result.target;

#ifdef IOCTLDEBUG
					printk(KERN_ERR "\nis1000: result.row_count %d  result.more_flag %04x\n", 
					       result.row_count, result.more_flag);
					printk(KERN_ERR "is1000: block_size %d block_status %d \n", 
					       result.block_size, result.block_status);
					printk(KERN_ERR "is1000: error_info_size %d\n",
					       result.error_info_size);
					printk(KERN_ERR "is1000: pInt  %08x   %08x   %08x   %08x\n",
					       *pInt, *(pInt +1), *(pInt + 2), *(pInt +3));
					printk(KERN_ERR "is1000: pInt  %08x   %08x   %08x   %08x\n",
					       *(pInt +4), *(pInt +5), *(pInt + 6), *(pInt +7));
					printk(KERN_ERR "is1000: pTarget->name <%s> \n", pTarget->name);

					printk(KERN_ERR "is1000: pTarget->ipAddress <%s> \n", pTarget->ipAddress);
					printk(KERN_ERR "is1000: pTarget->port %d  ->tag %d \n", pTarget->port,
					       pTarget->tag);
#endif
					/* before incrementing numEnumTargets, see if this latest
					 * returned target is the same as the last one
					 */
					if (!memcmp(pTarget->name, pCurrentTarget->iscsi_nodename,
						    ISCSI_TARGET_NAME_LENGTH - 1))
					{
						/* copy only the address stuff, into current
						 * pCurrentTarget slot, and increment the numAddresses
						 */
#ifdef IOCTLDEBUG
						printk(KERN_INFO "\nJust adding addresses, number %d\n",
						       currentAddress);
						printk(KERN_INFO "numEnumTargets %d  currentAddress %d\n\n", 
						       numEnumTargets, currentAddress);
#endif
						pCurrentTarget->addresses[currentAddress].ip = inet_addr(pTarget->ipAddress);
						pCurrentTarget->addresses[currentAddress].port = (u16) pTarget->port;
						pCurrentTarget->addresses[currentAddress].portalgroup = (u16) pTarget->tag;
						pCurrentTarget->numAddresses = ++currentAddress;
					}
					else  
					{
						/* it's the first time, or a new target, get name and all 
						   in new slot, and start or reset counters.. ??  */
						pCurrentTarget = pEnumTargets + numEnumTargets;
						currentAddress = 0;

						memcpy(pCurrentTarget->iscsi_nodename, pTarget->name,
						       ISCSI_TARGET_NAME_LENGTH - 1);
						pCurrentTarget->addresses[currentAddress].ip = inet_addr(pTarget->ipAddress);
						pCurrentTarget->addresses[currentAddress].port = (u16) pTarget->port;
						pCurrentTarget->addresses[currentAddress].portalgroup = (u16) pTarget->tag;
						pCurrentTarget->numAddresses = ++currentAddress;
						numEnumTargets++;

#ifdef IOCTLDEBUG
						printk(KERN_INFO "**new target: <%s>\n", pTarget->name);
						printk(KERN_INFO "numEnumTargets %d  currentAddress %d\n\n", 
						       numEnumTargets, currentAddress);
#endif
					}

				} while (moreFlag != 0);

#ifdef IOCTLDEBUG
				printk(KERN_INFO "total targets: numEnumTargets %d\n", 
				       numEnumTargets);
#endif
				pTargetCount->numTargets = numEnumTargets;
				pTargetCount->status = ENUMERATION_STATUS_SUCCESS;  
			}
			else if (intField == ISCSI_DISC_STATUS_PENDING)
			{
				pTargetCount->status = ENUMERATION_STATUS_PENDING;                          
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: still polling...\n");
#endif
			} 
			else if (intField == ISCSI_DISC_STATUS_FAILED)
			{
				pTargetCount->status = ENUMERATION_STATUS_FAILED;
#ifdef IOCTLDEBUG
				printk(KERN_INFO "is1000: target scan failed\n");
#endif
			}
			else /* a more detailed error that the upper layer will parse */
			{
				pTargetCount->status = intField;
#ifdef IOCTLDEBUG
				printk(KERN_INFO 
				       "is1000: Detailed error status from target enumeration: 0x%08x\n", 
				       intField);
#endif
			}
		
			if (copy_to_user((void *)arg, pTargetCount,
					 sizeof(ISCSI_GET_STORAGE_TARGET_COUNT)))
			{
				printk(KERN_ERR 
				       "is1000: copy_to_user failed: IOCLT_GET_STORAGE_TARGET_COUNT\n");
				goto outFail;
			}
		}
		else if (pTargetCount->status == ENUMERATION_STATUS_ABORT)
		{

#ifdef IOCTLDEBUG
		        printk(KERN_INFO "is1000: ENUMERATION_STATUS_ABORT\n");
#endif
			intField = ISCSI_DISC_ABORT_DISC;
			res =  i2o_set_scalar(c, 8,
					      ISCSI_ISM_TARGET_SCAN_SELECTOR_GROUP_NO, 
					      ISCSI_ISM_TARGET_SCAN_SEL_POLL_SCAN_IDX,
					      &intField, sizeof(intField));

			if (res < 0) {
				printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
				goto outFail;	 
			}
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: is1000_set_scalar success %d\n", res);
#endif
			/* dwStatus field already _STATUS_ABORT - no need to copy to user */

		}

	}
	
	break;

	case ISCSI_IOCTL_GET_STORAGE_TARGET :
	{
		PISCSI_GET_STORAGE_TARGET pTarget = 
			(PISCSI_GET_STORAGE_TARGET) pArgData;
		ISCSI_TOTAL_ENUM_TARGETS *pCurrentTarget;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_GET_STORAGE_TARGET - testing\n");
#endif	
		if (copy_from_user(pTarget,(char *)arg, sizeof(ISCSI_GET_STORAGE_TARGET))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTarget->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: pTarget->IPAddress %X %d\n", pTarget->IPAddress,
		       pTarget->port);
		printk(KERN_INFO "is1000: pTarget->targetIndex %d\n", pTarget->targetIndex);
#endif

		if (memcmp(&((char *)pEnumTargets)[MAX_TARGETS_PER_ENTITY * 
						   sizeof(ISCSI_TOTAL_ENUM_TARGETS)],
			   &pTarget->IPAddress, sizeof(u32)))
		{
			printk(KERN_ERR 
			       "is1000: Using different IP address than used for target query\n");
			pTarget->status = ENUMERATION_STATUS_FAILED;
		}
		else if (memcmp(&((char *)pEnumTargets)[MAX_TARGETS_PER_ENTITY * 
							sizeof(ISCSI_TOTAL_ENUM_TARGETS) + sizeof(u32)],
				&pTarget->port, sizeof(u16)))
		{
			printk(KERN_ERR 
			       "is1000: Using different port than used for target query\n");
			pTarget->status = ENUMERATION_STATUS_FAILED;
		}

		else if ((pTarget->targetIndex < 0) || (pTarget->targetIndex >= numEnumTargets))
		{
			printk(KERN_ERR "is1000: Target index %d out of bounds\n",
			       pTarget->targetIndex);
			pTarget->status = ENUMERATION_STATUS_FAILED;
		}
		else
		{
			pCurrentTarget = pEnumTargets + pTarget->targetIndex;
			memcpy(pTarget->iscsi_nodename, pCurrentTarget->iscsi_nodename,
			       IS_MAX_NODENAME_STRING - 1);
			pTarget->iscsi_nodename[IS_MAX_NODENAME_STRING - 1] = '\0';

			pTarget->numAddresses = pCurrentTarget->numAddresses;
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: ->numAddresses %d\n",
			       pTarget->numAddresses);
#endif
			for (i = 0; i < pTarget->numAddresses ; i++)
			{
				pTarget->target_addresses[i].ip = pCurrentTarget->addresses[i].ip;
				pTarget->target_addresses[i].port = pCurrentTarget->addresses[i].port;
				pTarget->target_addresses[i].portalgroup = pCurrentTarget->addresses[i].portalgroup;
			}
			pTarget->status = ENUMERATION_STATUS_SUCCESS;
		}
                if (copy_to_user((void *)arg, pTarget,
				 sizeof(ISCSI_GET_STORAGE_TARGET)))
		{
		        printk(KERN_ERR "is1000: copy_to_user failed for _GET_STORAGE_TARGET\n");
			goto outFail;
		}
	}
	break;

	case ISCSI_IOCTL_TARGET_DISABLE:
	{
		PISCSI_STORAGE_TARGET_INFO pTarget = (PISCSI_STORAGE_TARGET_INFO) pArgData;
		u16 *pBufList = (u16 *)pBuffer;
		PISCSI_TID_DATA pTidData;
		PI20_TARGET_DISABLE pDisableData;
		int resBlk[10];

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _TARGET_DISABLE\n");
#endif	

		if (copy_from_user(pTarget,(char *)arg, sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTarget->MACAddr)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Disabling target %d adapter %d \n",
		       pTarget->sTarget.targetId, adapterNum);
#endif

		if (pTarget->sTarget.targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR 
			       "is1000: Invalid target Id %d, cannot disable target\n", 
			       pTarget->sTarget.targetId);
			goto outFail;
		}

		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pTarget->sTarget.targetId)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_ERR "DISABLING TARGET WITH NAME: <%s>\n",
		       pTidData->name);
#endif

		*pBufList++ = ISCSI_ISM_TARGET_TABLE_KEY_IDX;
		*pBufList++ = ISCSI_ISM_TARGET_TABLE_ENABLE_IDX;
		*pBufList++ = 1;

		pDisableData = (PI20_TARGET_DISABLE) pBufList;

		memset(pDisableData->targetname, 0, sizeof(pDisableData->targetname));
		strncpy(pDisableData->targetname, pTidData->name, ISCSI_TARGET_NAME_LENGTH - 1);

		pDisableData->disable = ISCSI_PARAM_TARGET_DISABLE;

		res = i2o_set_list_table(I2O_PARAMS_LIST_SET, 
					 c,
					 8, ISCSI_ISM_TARGET_TABLE_GROUP_NO,
					 2,
					 pBuffer, sizeof(I20_TARGET_DISABLE) + 16,
					 resBlk, sizeof(resBlk));
		if (res < 0) {
			printk(KERN_ERR "is1000: Target disable failed on adapter %d\n", adapterNum);
			goto outFail;	 
		}
	}
	break;
	case ISCSI_IOCTL_REMOVE_TARGET :
	{
		PISCSI_STORAGE_TARGET_INFO pTarget = (PISCSI_STORAGE_TARGET_INFO) pArgData;
		u16 *pBufList = (u16 *)pBuffer;
		PISCSI_TID_DATA pTidData;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _REMOVE_TARGET\n");
#endif	

		if (copy_from_user(pTarget,(char *)arg, sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTarget->MACAddr)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Removing targets on adapter %d \n", adapterNum);

		printk(KERN_INFO "nodename <%s> targetId %d\n", 
		       pTarget->sTarget.iscsi_nodename, pTarget->sTarget.targetId);
#endif

		if (pTarget->sTarget.targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR 
			       "is1000: Invalid target Id %d, cannot remove target\n", 
			       pTarget->sTarget.targetId);
			goto outFail;
		}

		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pTarget->sTarget.targetId)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_ERR "REMOVING TARGET WITH NAME: <%s>\n",
		       pTidData->name);
#endif
		memset((u8 *)pBufList, 0, sizeof(pTarget->sTarget.iscsi_nodename));
		memcpy(pBufList, pTidData->name, ISCSI_TARGET_NAME_LENGTH - 1);


#ifdef IOCTLDEBUG
		printk(KERN_INFO "name in pBufList: %s\n", (char *)pBufList);
#endif

		/* Now remove all entries in hostdata related to this target */
		is1000_remove_target ((char *)pBufList, adapterNum);
		  
		res = i2o_row_del_table(c, 8,
					ISCSI_ISM_TARGET_TABLE_GROUP_NO, 1,
					pBuffer, ISCSI_TARGET_NAME_LENGTH -1);

		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to remove target: %s\n", 
			       pTarget->sTarget.iscsi_nodename);
			goto outFail;	    
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Successfully removed target: %s\n", 
		       pBuffer);
#endif
	}
	break;
	case ISCSI_IOCTL_GET_TARGET_PROPERTIES :
	{
		PISCSI_STORAGE_TARGET_INFO pTarget = (PISCSI_STORAGE_TARGET_INFO)pArgData;
		PISCSI_TARGET_TABLE_FIELDS pTargetTable, pTableBuffer;
		u16 *pBufList = (u16 *)pBuffer;
		char ipAddress[ISCSI_TARGET_ADDRESS_LENGTH];
		PISCSI_TID_DATA pTidData;
		int tid;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: _GET_TARGET_PROPERTIES:\n");
#endif	

		if (copy_from_user(pTarget,(char *)arg, 
				   sizeof(ISCSI_STORAGE_TARGET_INFO))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;	   
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pTarget->MACAddr)))
		{
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Get storage target prop on adapter %d \n", adapterNum);

#endif

		/* Verify the target id is valid */
		if (pTarget->sTarget.targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR "is1000: Recovery data set on unenabled target not supported yet\n");
			goto outFail;
		}

		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pTarget->sTarget.targetId)))
		{
			goto outFail;
		}

		pTableBuffer = (PISCSI_TARGET_TABLE_FIELDS)
			kmalloc(sizeof(ISCSI_TARGET_TABLE_FIELDS) + 16, GFP_KERNEL);
		if (pTableBuffer == NULL)
		{
			printk(KERN_ERR "is1000: no memory!\n");
			goto outFail;
		}

		*pBufList++ = 2;  /* ipAddress */
		*pBufList++ = 3;  /* port */
		*pBufList++ = 4;  /* portalTag */
		*pBufList++ = 6;
		*pBufList++ = 7;
		*pBufList++ = 9;
		*pBufList++ = 10;
		*pBufList++ = 11;
		*pBufList++ = 12;
		*pBufList++ = 13;
		*pBufList++ = 14;
		*pBufList++ = 15;
		*pBufList++ = 1;

		memset((u8 *)pBufList, 0, sizeof(pTarget->sTarget.iscsi_nodename));
		memcpy (pBufList, pTidData->name, IS_MAX_NODENAME_STRING - 1);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: target name as key: <%s>\n", (char *)pBufList);
#endif
	       
		res = i2o_query_table (I2O_PARAMS_LIST_GET,
				       c, 
				       8 /* need to get legitimately */,
				       ISCSI_ISM_TARGET_TABLE_GROUP_NO, 12, 
				       pBuffer, (IS_MAX_NODENAME_STRING - 1) + 36,
				       pTableBuffer, sizeof(ISCSI_TARGET_TABLE_FIELDS) + 16);

		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to get target information: %s\n", 
			       pTarget->sTarget.iscsi_nodename);
			kfree(pTableBuffer);
			goto outFail;	    
		}

		else if (*((u32 *)pTableBuffer + 1) & 0x00ff0000)
		{
			printk(KERN_ERR "is1000: Error getting target info %s\n", 
			       pTarget->sTarget.iscsi_nodename);

			kfree(pTableBuffer);
			goto outFail;

		}

		pTargetTable = (PISCSI_TARGET_TABLE_FIELDS) (((u32 *)pTableBuffer) + 2);

#ifdef IOCTLDEBUG
		printk(KERN_ERR "is1000: Got target information. res %d\n", res);

		printk(KERN_ERR "is1000: pTargetTable->ipAddress %s   port %d\n",
		       pTargetTable->ipAddress, pTargetTable->port);
#endif

		pTarget->sTarget.address.ip = ntohl(inet_addr(pTargetTable->ipAddress));
		pTarget->sTarget.address.port = pTargetTable->port;
		if ((pTarget->sTarget.chap.useCHAP  = pTargetTable->chapEnabled))
		{
			memcpy (pTarget->sTarget.chap.CHAPusername, pTargetTable->chapUsername,
				IS_MAX_USERNAME_STRING);
			memcpy (pTarget->sTarget.chap.CHAPpassword, pTargetTable->chapPassword,
				IS_MAX_PASSWORD_STRING);
		}
		if ((pTarget->sTarget.chap.useInboundCHAP = pTargetTable->chapInboundEnabled))
		{
			memcpy (pTarget->sTarget.chap.inboundCHAPusername, 
				pTargetTable->chapInboundUsername, IS_MAX_USERNAME_STRING);
			memcpy (pTarget->sTarget.chap.inboundCHAPpassword, 
				pTargetTable->chapInboundPassword, IS_MAX_PASSWORD_STRING);
		}

		kfree(pTableBuffer);

		/* Now get the per hba session table stuff, we already have h and pTidData */

		tid = pTidData->tid;

		if (tid == -1)
		{
			printk (KERN_ERR "is1000: No targetId %d found on adapter %d\n",
				pTarget->sTarget.targetId, adapterNum);
			goto outFail;
		}

		res =  i2o_query_scalar(c, tid,
					ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					ISCSI_ISM_PER_HBA_IP_ADDRESS,
					ipAddress,
					ISCSI_TARGET_ADDRESS_LENGTH);
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}

		pTarget->sTarget.address.ip = inet_addr(ipAddress);

		res =  i2o_query_scalar(c, tid,
					ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					ISCSI_ISM_PER_HBA_PORT,
					&pTarget->sTarget.address.port, sizeof(u16));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}

		/* Convenience if ephemeral port is ever desired. Note: requires
		 * ephemeral port field is receiving struct. */
#if 0
		res =  i2o_query_scalar(c, tid,
					ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					ISCSI_ISM_PER_HBA_EPHEMERAL_PORT,
					&pTarget->sTarget.address.portalgroup, sizeof(u16));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}
#endif
		{
			PISCSI_ISM_SESSION_CONFIG sessConfig = (PISCSI_ISM_SESSION_CONFIG) pBuffer;

			res =  i2o_query_scalar(c, tid,
						ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
						ISCSI_ISM_PER_HBA_SESS_CONFIG,
						sessConfig,
						sizeof(ISCSI_ISM_SESSION_CONFIG));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;
			}
#ifdef IOCTLDEBUG		  
			printk(KERN_INFO "is1000: sessConfig->targetPortalGroup %d\n",
			       sessConfig->targetPortalGroup);
#endif
			pTarget->sTarget.address.portalgroup = sessConfig->targetPortalGroup;
		}
		  
		if (copy_to_user((void *)arg, pTarget,
				 sizeof(ISCSI_STORAGE_TARGET_INFO)))
		{
			printk(KERN_ERR "is1000: copy_to_user failed for _GET_TARGET_PROPERTIES\n");
			goto outFail;
		}

	}
	break;
	case ISCSI_IOCTL_PING :
	{
		PISCSI_PING pPing = (PISCSI_PING) pArgData;
		u32 ipAddress;
		char sIPAddress[ISCSI_TARGET_ADDRESS_LENGTH - 1];

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_PING\n");
#endif	
		if (copy_from_user(pPing,(char *)arg, sizeof(ISCSI_PING))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pPing->MACAddr)))
		{
			goto outFail;
		}

		ipAddress = ntohl(pPing->IPAddress);
		inet_ntoa(ipAddress, sIPAddress);

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Pinging router from %d \n", adapterNum);
		printk(KERN_INFO "IP Address %X  (string: <%s>)\n", 
		       (int)pPing->IPAddress, sIPAddress);
#endif

		res =  i2o_set_scalar(c, 8,
				      ISCSI_ISM_ICMP_GROUP_NO, 
				      0, 
				      &sIPAddress, sizeof(sIPAddress));

		if (res == 8) /* um.. why 8? */
		{
			printk(KERN_INFO "is1000: PING succeeded\n");
			pPing->dwStatus = IOCTL_STATUS_SUCCESS;
		}
		else if (res < 0)
		{
			printk(KERN_ERR "is1000: is1000_set_scalar failed %d\n", res);
			pPing->dwStatus = CONNECTION_NO_MATCH;
		}
		else
		{
			printk(KERN_INFO "is1000: PING failed, error result %d\n", res);
			pPing->dwStatus = IOCTL_STATUS_FAILED; 
		}
                if (copy_to_user((void *)&((PISCSI_PING)arg)->dwStatus, &pPing->dwStatus, 
				 sizeof(pPing->dwStatus)))
		{
		        printk(KERN_ERR "is1000: copy_to_user failed for _PING\n");
			goto outFail;
		}
		if (res < 0)
			goto outFail;

	}
	break;

	case ISCSI_IOCTL_GET_EEPROM_INFO :
	{
		PISCSI_GET_EEPROM_DATA pEepromData = (PISCSI_GET_EEPROM_DATA) pArgData;
		char opBlock[22];
		u16 *pOpBlock;
		u8* pData;

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_GET_EEPROM_INFO -\n");
#endif	

		if (copy_from_user(pEepromData, (char *)arg, sizeof(ISCSI_GET_EEPROM_DATA))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pEepromData->MACAddr)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ->dwStatus %d  ->timeout %d (unused)  ->bufferSize %d\n",
		       pEepromData->dwStatus, pEepromData->timeOut, pEepromData->bufferSize);
#endif		

		pOpBlock = (u16 *)&opBlock;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_KEY_IDX;
		*pOpBlock++ = ISCSI_ISM_NETIF_TABLE_EEPROM_IDX;

		*pOpBlock++ = 1; /* key count */
		memcpy(pOpBlock, &iscsi_adapter_priv[adapterNum].ifTableKey, sizeof(int));

		res = i2o_query_table (I2O_PARAMS_LIST_GET,
				       c,
				       8 /* need to get legitimately */,
				       ISCSI_ISM_NETWORK_INTERFACE_TABLE_GROUP_NO, 
				       2, 
				       &opBlock, 22,
				       pBuffer, IS_EEPROM_DATA_SIZE + 16);
	   
		if (res < 0) {
			printk(KERN_ERR "is1000: adapter access failed %d, adapter number %d\n", 
			       res, adapterNum);
			goto outFail;	 
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: EEPROM get success %d\n", res);
#endif
		pData = (u8 *) (pBuffer + 12);

		memcpy (pEepromData->dataBuffer, pData, IS_EEPROM_DATA_SIZE);
		pEepromData->dwStatus = EEPROM_STATUS_SUCCESS;

                if (copy_to_user((void *)arg, pEepromData, 
				 sizeof(*pEepromData)))
		{
		        printk(KERN_ERR "is1000: copy_to_user failed for GET_EEPROM_DATA\n");
			goto outFail;
		}
	}
        break;

	case ISCSI_IOCTL_TARGET_RECOVERY_DATA:
	{
		PISCSI_TARGET_RECOVERY_DATA pRecovData = (PISCSI_TARGET_RECOVERY_DATA) pArgData;
		PISCSI_TID_DATA pTidData;
		uint tid;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_TARGET_RECOVERY_DATA \n");
#endif	

		if (copy_from_user(pRecovData, (char *)arg, sizeof(ISCSI_TARGET_RECOVERY_DATA))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pRecovData->MACAddr)))
		{
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ->dwStatus %d (0 == set, 1 == get)\n",
		       pRecovData->dwStatus);
		printk(KERN_INFO "is1000:  ->targetId %d\n", pRecovData->targetId);
		printk(KERN_INFO "is1000:  ->timeout %d  ->timeout %d (after) ->maxRetries %d\n", 
		       pRecovData->timeOut,
		       pRecovData->timeOut * 1000 * 1000, pRecovData->maxRetries);
#endif		


		if (pRecovData->targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR "is1000: Recovery data set on unenabled target not supported yet\n");
			goto outFail;
		}

		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pRecovData->targetId)))
		{
			goto outFail;
		}

		tid = pTidData->tid;

		if (tid == -1)
		{
			printk (KERN_ERR "is1000: No targetId %d found on adapter %d\n",
				pRecovData->targetId, adapterNum);
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_ERR "is1000: accessing recovery data on tid %d\n", tid);
#endif
		if (pRecovData->dwStatus == ISCSI_RECOVERY_DATA_SET)
		{
			int micsTimeOut = pRecovData->timeOut * 1000 * 1000;
			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_RECONNECT_TIME,
					      &micsTimeOut,
					      sizeof(u32));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}

			res =  i2o_set_scalar(c, tid,
					      ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					      ISCSI_ISM_PER_HBA_MAX_RETRIES,
					      &pRecovData->maxRetries,
					      sizeof(u32));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;	    
			}
 
			pRecovData->dwStatus = IOCTL_STATUS_SUCCESS;
			if (copy_to_user((void *)&((PISCSI_TARGET_RECOVERY_DATA)arg)->dwStatus, 
					 &pRecovData->dwStatus, 
					 sizeof(pRecovData->dwStatus)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_RECOVERY_DATA\n");
				goto outFail;
			}

		}
		else if (pRecovData->dwStatus == ISCSI_RECOVERY_DATA_GET)
		{

			res =  i2o_query_scalar(c, tid,
						ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
						ISCSI_ISM_PER_HBA_RECONNECT_TIME,
						&pRecovData->timeOut, sizeof(u32));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;
			}

			pRecovData->timeOut /= (1000 * 1000);

			res =  i2o_query_scalar(c, tid,
						ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
						ISCSI_ISM_PER_HBA_MAX_RETRIES,
						&pRecovData->maxRetries, sizeof(u32));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;
			}

			res =  i2o_query_scalar(c, tid,
						ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
						ISCSI_ISM_PER_HBA_CURRENT_RETRIES,
						&pRecovData->currentRetries, sizeof(u32));
			if (res < 0) {
				printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
				goto outFail;
			}

			pRecovData->dwStatus = IOCTL_STATUS_SUCCESS;
			if (copy_to_user((void *)arg, pRecovData, 
					 sizeof(ISCSI_TARGET_RECOVERY_DATA)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_RECOVERY_DATA\n");
				goto outFail;
			}
		}
		else
		{
			printk(KERN_ERR "is1000: Unknown target recovery request.\n");
			goto outFail;
		}
	}
	break; 
	case  ISCSI_IOCTL_RESET_TARGET:
	{
		PISCSI_TARGET_RESET pReset = (PISCSI_TARGET_RESET) pArgData;
		PISCSI_TID_DATA pTidData;
		
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_TARGET_RESET \n");
#endif	

		if (copy_from_user(pReset, (char *)arg, sizeof(ISCSI_TARGET_RESET))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pReset->MACAddr)))
		{
			printk(KERN_ERR "is1000: _RESET_TARGET failed on adapter %d\n",
			       adapterNum);
			pReset->status = ISCSI_RESET_TARGET_FAILED;
			if (copy_to_user((void *)arg, pReset, 
					 sizeof(ISCSI_TARGET_RESET)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_RESET\n");
				goto outFail;
			}
			break;
		}
		
		res = is1000_bus_reset(adapterNum, pReset->targetId, 0);

		if (res < 0)
		{
			printk(KERN_ERR "is1000: _RESET_TARGET failed on adapter %d\n",
			       adapterNum);
			pReset->status = ISCSI_RESET_TARGET_FAILED;
			if (copy_to_user((void *)arg, pReset, 
					 sizeof(ISCSI_TARGET_RESET)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_RESET\n");
				goto outFail;
			}
			break;
		}
		
		/* Mark the target status as failed - we'll get an event for
		 * failed status later, and hopefully followed by a normal event.
		 */
		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pReset->targetId)))
		{
			printk(KERN_ERR "is1000: Target reset may have failed on adapter %d\n",
			       adapterNum);
			pReset->status = ISCSI_RESET_TARGET_FAILED;
			if (copy_to_user((void *)arg, pReset, 
					 sizeof(ISCSI_TARGET_RESET)))
			{
				printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_RESET\n");
			}
		}

		pTidData->status = ISCSI_TARGET_STATUS_FAILED;
	}
	break;

	case ISCSI_IOCTL_LOOPBACK :
	{
		PISCSI_LOOPBACK pLoopback = (PISCSI_LOOPBACK) pArgData;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_LOOPBACK\n");
#endif
		
		if (copy_from_user(pLoopback, (char *)arg, sizeof(ISCSI_LOOPBACK))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pLoopback->MACAddr)))
		{
			goto outFail;
		}

		res =  i2o_set_scalar(c, 8,
				      ISCSI_ISM_LOOPBACK_GROUP_NO,
				      0,
				      pLoopback->buffer,
				      IS_MAX_LOOPBACK_BUFFER_SIZE);
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}

#ifdef IOCTLDEBUG
		memset(pLoopback->buffer, 0, 255);
		printk(KERN_INFO "is1000: sent loopback data to adapter %d\n", adapterNum);
#endif
		res =  i2o_query_scalar(c, 8,
					ISCSI_ISM_LOOPBACK_GROUP_NO,
					0,
					pBuffer,
					IS_MAX_LOOPBACK_BUFFER_SIZE);
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}

#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: received loopback data to adapter %d\n", adapterNum);
#endif

		memcpy(pLoopback->buffer, pBuffer, IS_MAX_LOOPBACK_BUFFER_SIZE);

		if (copy_to_user((void *)arg, pLoopback, sizeof(ISCSI_LOOPBACK))) {
			printk(KERN_ERR "is1000: Cannot copy to user\n");
			goto outFail;
		}
	}
	break;
	case ISCSI_IOCTL_TARGET_ERROR_STATUS :
	{
		PISCSI_TID_DATA pTidData;
		u32 tid;

		PISCSI_TARGET_ERROR_STATUS pStatus = (PISCSI_TARGET_ERROR_STATUS) pArgData;
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: ISCSI_IOCTL_TARGET_ERROR_STATUS\n");
#endif
		
		if (copy_from_user(pStatus, (char *)arg, sizeof(ISCSI_TARGET_ERROR_STATUS))) {
			printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
			goto outFail;
		}

		if (!(c = iscsi_find_adapter(&adapterNum, &pStatus->MACAddr)))
		{
			goto outFail;
		}

		if (pStatus->targetId == IS_TARGETID_INVALID)
		{
			printk(KERN_ERR "is1000: Invalid target id: cannot get error status\n");
			goto outFail;
		}

		if (!(pTidData = 
		      is1000_get_tiddata(adapterNum, pStatus->targetId)))
		{
			goto outFail;
		}

		tid = pTidData->tid;

		if (tid == -1)
		{
			printk (KERN_ERR "is1000: No targetId %d found on adapter %d\n",
				pStatus->targetId, adapterNum);
			goto outFail;
		}

		res =  i2o_query_scalar(c, tid,
					ISCSI_ISM_PER_HBA_SESSION_GROUP_NO,
					ISCSI_ISM_PER_HBA_ERROR_STATE,
					&pStatus->sessionState,
					sizeof(u32));
		if (res < 0) {
			printk(KERN_ERR "is1000: Failed to access adapter: %d\n", adapterNum);
			goto outFail;
		}
		
		if (copy_to_user((void *)&((PISCSI_TARGET_ERROR_STATUS)arg)->sessionState, 
				 &pStatus->sessionState, 
				 sizeof(pStatus->sessionState)))
		{
		        printk(KERN_ERR "is1000: copy_to_user failed for _TARGET_ERROR_STATUS\n");
			goto outFail;
		}
#ifdef IOCTLDEBUG
		printk(KERN_INFO 
		       "is1000: sessionState 0x%08x\n", pStatus->sessionState);
#endif

	}
	break;
	default:
	{
#ifdef IOCTLDEBUG
		printk(KERN_INFO 
		       "is1000: default: ControlIoctl() cmd 0x%x\n", cmd);
		printk(KERN_INFO "is1000: _IOC_NR(cmd) = %d\n\n", _IOC_NR(cmd));
		
#endif
		goto outFail;
	}
	}

	kfree(pArgData);
	kfree(pBuffer);
	if (c)
	{
#ifdef IOCTLDEBUG
		if (atomic_read(&c->users) != 0)
			dprintk(KERN_ERR 
				"is1000: Exiting Ioctl with lock: c 0x%x users %d\n",
				(int)c, atomic_read(&c->users));
#endif		
		i2o_unlock_controller(c);
	}
	return 0;

 outFail:
	kfree(pArgData);
	kfree(pBuffer);
	if (c)
	{
#ifdef IOCTLDEBUG
		if (atomic_read(&c->users) != 0)
			printk(KERN_ERR 
			       "is1000: (Fail case) Exiting Ioctl with lock: c 0x%x users %d\n",
			       (int)c, atomic_read(&c->users));
#endif		
		i2o_unlock_controller(c);
	}
	return -EINVAL;
}


void is1000EventHandler(struct i2o_handler *pI2oHandler, 
			struct i2o_controller * pI2oController,
			struct i2o_message *pMsg)
{
	u32 *pMsgWord = (u32 *)pMsg;
	u8 status = pMsgWord[5] & 0xff;
	u8 detail = pMsgWord[6] & 0xff;
	u8 tid = pMsgWord[1] & 0xff;
	int i, targetId = -1;
	char *tildeLoc = 0;
	struct i2o_scsi_host *h;
	PISCSI_TID_DATA pTidData;
	IS1000_EVENT localEvent;

	if (iscsi_adapter_priv[pI2oController->unit].hostAdapterNum == -1)
	{
		printk(KERN_ERR  "is1000: Connection state change on unknown host adapter.\n");
		return;
	}

	h = is1000_get_hostdata(pI2oController->unit);

#ifdef IOCTLDEBUG
	printk(KERN_INFO "\nis1000: ENTERING EVENT HANDLER: ->unit %d  tid %d  status %d\n",
	       pI2oController->unit, tid, status);
#endif	

	if (h == NULL)
	{
		printk(KERN_ERR "is1000: is1000EventHandler found NULL iSCSI host\n");
		return;
	}

	memset(&localEvent, 0, sizeof(IS1000_EVENT));

	/* If it's a DDM related event... */
	if (tid == 8)
	{
		if (status == ISCSI_ISM_NET_LINK_DOWN)
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: Net link down on adapter %d\n",
			       pI2oController->unit);
#endif
			localEvent.event = ISEVENT_HOST_ADAPTER_LINK_DOWN;
		}
		else if (status == ISCSI_ISM_NET_LINK_UP)
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: Net link up on adapter %d\n",
			       pI2oController->unit);
#endif
			localEvent.event = ISEVENT_HOST_ADAPTER_LINK_UP;
		}
		else if (status == ISCSI_ISM_NET_DHCP_LEASE_NEW)
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: DHCP: new lease acquired on adapter %d\n",
			       pI2oController->unit);
#endif
			localEvent.event = ISEVENT_HOST_ADAPTER_DHCP_LEASE_NEW;
		}
		else if (status == ISCSI_ISM_NET_DHCP_LEASE_EXPIRE)
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: DHCP: lease expired on adapter %d\n",
			       pI2oController->unit);
#endif
			localEvent.event = ISEVENT_HOST_ADAPTER_DHCP_LEASE_EXPIRE;
		}

		else if (status == ISCSI_ISM_NET_DHCP_BIND_FAIL)
		{
#ifdef IOCTLDEBUG
			printk(KERN_INFO "is1000: DHCP: bind failed on adapter %d\n",
			       pI2oController->unit);
#endif
			localEvent.event = ISEVENT_HOST_ADAPTER_DHCP_BIND_FAIL;
		}

		localEvent.host = 
			iscsi_adapter_priv[pI2oController->unit].hostAdapterNum;
		memcpy(&localEvent.MACAddr, 
		       &iscsi_adapter_priv[pI2oController->unit].macAddr, 
		       MAC_ADDR_LENGTH);

#ifdef IOCTLDEBUG
		printk (KERN_INFO "detail in event reply: 0x%x\n", detail);
#endif
		localEvent.eventDetail = detail;

		memcpy(&connectionEvent[is1000_evt_in], &localEvent, sizeof(IS1000_EVENT));

		spin_lock(&is1000_evt_lock);
		MODINC(is1000_evt_in, IS1000_EVT_Q_LEN);
		if(is1000_evt_q_len == IS1000_EVT_Q_LEN)
			MODINC(is1000_evt_out, IS1000_EVT_Q_LEN);
		else
			is1000_evt_q_len++;
		spin_unlock(&is1000_evt_lock);

#ifdef IOCTLDEBUG
		printk (KERN_INFO "is1000: EventHandler waking any waiters\n");
#endif	    
		up(&is1000_evt_sem);
		
		//	    wake_up_interruptible(&is1000_evt_wait);

		return;
	}

	/* Otherwise it's a target-specific event. Must find
	 * target Id and make sure it's properly registered.
	 */

	for (i = 0 ; i < IOP_MAXIMUM_TARGETS ; i++)
	{
		PISCSI_TID_DATA pTidData = h->aTaskData[i].pTidData;

		if (pTidData == NULL)
		{
			continue;
		}

		if (pTidData->tid == tid)
		{
			targetId = i;
			break;
		}
	}

	if (targetId == -1)
	{
#ifdef IOCTLDEBUG
		printk (KERN_ERR 
			"is1000: is1000EventHandler for unknown target on adapter %d\n",
			pI2oController->unit);
#endif
		return;
	}

	pTidData = h->aTaskData[targetId].pTidData;

#ifdef IOCTLDEBUG
	printk("\nis1000: is1000EventHandler hostAdapterNum %d   targetId %d\n", 
	       iscsi_adapter_priv[pI2oController->unit].hostAdapterNum, targetId);
#endif

	localEvent.targetId = targetId;
	localEvent.host = iscsi_adapter_priv[pI2oController->unit].hostAdapterNum;
	
	memcpy(localEvent.targetname, pTidData->name, IS_MAX_NODENAME_STRING);
	tildeLoc = strchr(localEvent.targetname, '~');
	if (tildeLoc > 0)
	{
		memset(tildeLoc, 0, 13);
	}
	
	memcpy(&localEvent.MACAddr, &iscsi_adapter_priv[pI2oController->unit].macAddr, 
	       MAC_ADDR_LENGTH);

	if(status == ISCSI_ISM_LOGIN_REDIR_PERM)
	{
#ifdef IOCTLDEBUG	  
		printk(KERN_INFO "is1000: Target change on scsi host adapter %d:\n",
		       iscsi_adapter_priv[pI2oController->unit].hostAdapterNum);
		printk(KERN_INFO "is1000: target id %d: Permanent login redirection.\n", targetId);

		printk (KERN_INFO "_FAILED status; pI2oController->unit %d  tid %d\n", 
			pI2oController->unit, (int)tid);
#endif
		localEvent.event = ISEVENT_TARGET_PERMANENTLY_REDIRECTED;
	}
	else if(status == I2O_EVT_STATE_CHANGE_FAILED)
	{
#ifdef IOCTLDEBUG	  
		printk(KERN_INFO "is1000: Target Connection state change on scsi host adapter %d:\n",
		       iscsi_adapter_priv[pI2oController->unit].hostAdapterNum);
		printk(KERN_INFO "is1000: target id %d: Connection failed.\n",
		       targetId);
		printk (KERN_INFO "_FAILED status; pI2oController->unit %d  tid %d\n", 
			pI2oController->unit, (int)tid);
#endif
		localEvent.event = ISEVENT_TARGET_CONNECTION_FAILED;
		pTidData->status = ISCSI_TARGET_STATUS_FAILED;
	}
	else if (status == I2O_EVT_STATE_CHANGE_NORMAL)
	{
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Target connection state change on scsi host adapter %d:\n",
		       iscsi_adapter_priv[pI2oController->unit].hostAdapterNum);
		printk(KERN_INFO "is1000: target id %d: Connection normal.\n",
		       targetId);
		printk (KERN_INFO "pI2oController->unit %d, tid %d\n", pI2oController->unit, 
			(int)tid);
#endif
		localEvent.event = ISEVENT_TARGET_CONNECTION_NORMAL;
		pTidData->status = ISCSI_TARGET_STATUS_CONNECTED;
	}	
	else if(status == I2O_EVT_STATE_CHANGE_FAULTED)
	{
#ifdef IOCTLDEBUG
		printk(KERN_INFO "is1000: Connection state change on scsi host adapter %d:\n",
		       iscsi_adapter_priv[pI2oController->unit].hostAdapterNum);
		printk(KERN_INFO "is1000: target id %d: Connection faulted.\n",
		       targetId);
		printk (KERN_INFO "pI2oController->unit %d  tid %d\n", pI2oController->unit, (int)tid);
#endif
		localEvent.event = ISEVENT_TARGET_CONNECTION_FAULTED;
		pTidData->status = ISCSI_TARGET_STATUS_FAULTED;
	}
	else if (status == ISCSI_ISM_ERROR_EVENT)
	{
#ifdef IOCTLDEBUG
		printk (KERN_INFO "is1000: ISCSI_ISM_ERROR_EVENT\n");
#endif
		localEvent.event = ISEVENT_ERROR;
	}

#ifdef IOCTLDEBUG
	printk (KERN_INFO "detail in event reply: 0x%x\n", detail);
#endif
	localEvent.eventDetail = detail;

	memcpy(&connectionEvent[is1000_evt_in], &localEvent, sizeof(IS1000_EVENT));

	spin_lock(&is1000_evt_lock);
	MODINC(is1000_evt_in, IS1000_EVT_Q_LEN);
	if(is1000_evt_q_len == IS1000_EVT_Q_LEN)
		MODINC(is1000_evt_out, IS1000_EVT_Q_LEN);
	else
		is1000_evt_q_len++;
	spin_unlock(&is1000_evt_lock);
	
#ifdef IOCTLDEBUG
	printk (KERN_INFO "is1000: EventHandler waking any waiters\n");
#endif
	up(&is1000_evt_sem);

	//  wake_up_interruptible(&is1000_evt_wait);
}


int is1000_init_module(void)
{
	int i;
	printk(KERN_INFO "Loading is1000 - Intel PRO 1000 T IP Driver\n");

#if (defined(CONFIG_HIGHIO) || defined(AS)) && defined(IS1000_HIGHMEMIO)
	printk(KERN_INFO "is1000: Highmem I/O Enabled.\n");
#endif

	if (i2o_install_handler(&i2o_core_handler) < 0)
	{
		printk(KERN_ERR 
		       "is1000: Unable to install core handler.\n");
		return -ENODEV;
	}
	core_context = i2o_core_handler.context;

	/*  Event Handler */
	if(i2o_install_handler(&sIs1000EventHandler) < 0)
	{
		printk(KERN_ERR "is1000: Unable to register I2O Socket Event Handler\n");
		i2o_remove_handler(&i2o_core_handler);
		return -ENOSPC;
	}

	/*
	 * Attach core to I2O PCI transport (and others as they are developed)
	 */
#ifdef RH90
	if(i2o_pci_core_attach() < 0)
#else
		if(i2o_pci_core_attach(&i2o_core_functions) < 0)
#endif
		{
			printk(KERN_INFO "is1000: No Intel PRO 1000 T IP Adapters found.\n");
			i2o_remove_handler(&i2o_core_handler);
			return -ENODEV;
		}

	/*
	 * Initialize event handling thread
	 */	
	init_MUTEX_LOCKED(&evt_sem);
	evt_pid = kernel_thread(i2o_core_evt, &evt_reply, CLONE_SIGHAND);
	if(evt_pid < 0)
	{
		printk(KERN_ERR "is1000: Could not create event handler kernel thread\n");
		i2o_remove_handler(&i2o_core_handler);
		return evt_pid;
	}
	else
	{
		printk(KERN_INFO "is1000: Event thread created as pid %d\n", evt_pid);
	}

	init_MUTEX_LOCKED(&is1000_evt_sem);

	if(i2o_num_controllers)
		is1000_sys_init();

	for(i=0 ; i < i2o_num_controllers ; i++)
	{
		if(i2o_controllers[i] != NULL)
		{
		        dprintk(KERN_ERR "is1000: registering for event on adapter %d\n", i);
			i2o_event_register(i2o_controllers[i], 8, 
					   sIs1000EventHandler.context, 0,0x80000000);
		}
	}

	/* THIS IS A TEMPORARY SOLUTION! It doesn't handle
	 * the case of multiple adapters w/ one or more failing
	 * initialization!!
	 */
	if (!i2o_num_controllers)
	{
		printk(KERN_ERR "is1000: could not init adapter\n");
		return -ENODEV;
	}

	register_reboot_notifier(&i2o_reboot_notifier);


	/* create a char dev */
	ControlMajor = register_chrdev( 0, ControlName, &ControlFops );
	if ( ControlMajor < 0 ) {
		printk(KERN_ERR "is1000: failed to register the control device: %d\n", 
		       ControlMajor);
		return ControlMajor;
	}

	/* printk(KERN_INFO" iSCSI control device major number %d\n", ControlMajor); */

	memset (&iscsi_adapter_priv, 0, sizeof(iscsi_adapter_priv));

	return 0;
}

module_init(is1000_init_module );

void is1000_cleanup_module(void)
{
	int stat;

	exit_this_scsi_driver();

	if(i2o_num_controllers)
		i2o_sys_shutdown();

	/*
	 * If this is shutdown time, the thread has already been killed
	 */
	if(evt_running) {
		printk("Terminating is1000 threads...");
		stat = kill_proc(evt_pid, SIGTERM, 1);
		if(!stat) {
			printk("waiting...");
			wait_for_completion(&evt_dead);
		}
		printk("done.\n");
	}

	i2o_pci_core_detach();

	i2o_remove_handler(&i2o_core_handler);

	unregister_reboot_notifier(&i2o_reboot_notifier);
	
	if ( ControlMajor > 0 ) {
		stat = unregister_chrdev( ControlMajor, ControlName );
		if ( stat ) {
			printk(KERN_ERR 
			       "iSCSI Error trying to unregister control device\n");
		} else {
			ControlMajor = 0;
		}
	}
	if (pEnumTargets)
		kfree(pEnumTargets);
}

module_exit(is1000_cleanup_module);

//#else
/* special stuff for RH90 port */
#if 0
extern int i2o_block_init(void);
extern int i2o_config_init(void);
extern int i2o_pci_init(void);
extern int i2o_proc_init(void);
extern int i2o_scsi_init(void);
extern int i2o_sock_init(void);

int __init i2o_init(void)
{
	printk(KERN_INFO "Loading I2O Core - (c) Copyright 1999 Red Hat Software\n");
	
	if (i2o_install_handler(&i2o_core_handler) < 0)
	{
		printk(KERN_ERR 
		       "i2o_core: Unable to install core handler.\nI2O stack not loaded!");
		return 0;
	}

	core_context = i2o_core_handler.context;

	/*
	 * Initialize event handling thread
	 * We may not find any controllers, but still want this as 
	 * down the road we may have hot pluggable controllers that
	 * need to be dealt with.
	 */	
	init_MUTEX_LOCKED(&evt_sem);
	if((evt_pid = kernel_thread(i2o_core_evt, &evt_reply, CLONE_SIGHAND)) < 0)
	{
		printk(KERN_ERR "I2O: Could not create event handler kernel thread\n");
		i2o_remove_handler(&i2o_core_handler);
		return 0;
	}

#ifdef CONFIG_I2O_PCI
	i2o_pci_init();
#endif

	if(i2o_num_controllers)
		is1000_sys_init();

	register_reboot_notifier(&i2o_reboot_notifier);

	i2o_config_init();
#ifdef CONFIG_I2O_BLOCK
	i2o_block_init();
#endif

#ifdef CONFIG_I2O_PROC
	i2o_proc_init();
#endif
	return 0;
}

#endif  /* #if 0    special stuff for RH90 port */


const unsigned char *__ctype = &__ctype_tab[1];

#define wrs_toupper(c)    ((('a' <= (c))&&((c) <= 'z')) ? ((c) - 'a' + 'A') : (c))
#define wrs_isalpha(c)    (__ctype[c] & (_C_UPPER | _C_LOWER))
#define wrs_isupper(c)    (__ctype[c] & (_C_UPPER))
#define wrs_isdigit(c)    (__ctype[c] & (_C_NUMBER))
#define wrs_isxdigit(c)   (__ctype[c] & (_C_HEX_NUMBER))
#define wrs_islower(c)    (__ctype[c] & (_C_LOWER))
#define wrs_isspace(c)    (__ctype[c] & (_C_WHITE_SPACE | _C_CONTROL))

/*****************************************************************************
 *
 * wrs_strtoul - convert a string to an unsigned long integer (ANSI)
 *
 * This routine converts the initial portion of a string <nptr> to
 * `unsigned long int' representation.  First, it decomposes the input
 * string into three parts:  an initial, possibly empty, sequence of
 * white-space characters (as specified by isspace()); a subject sequence
 * resembling an unsigned integer represented in some radix determined by
 * the value <base>; and a final string of one or more unrecognized
 * characters, including the terminating null character of the input string.
 * Then, it attempts to convert the subject sequence to an unsigned integer,
 * and returns the result.
 *
 * If the value of <base> is zero, the expected form of the subject sequence
 * is that of an integer constant, optionally preceded by a plus or minus
 * sign, but not including an integer suffix.  If the value of <base> is
 * between 2 and 36, the expected form of the subject sequence is a sequence
 * of letters and digits representing an integer with the radix specified by
 * letters from a (or A) through z (or Z) which are ascribed the values 10 to
 * 35; only letters whose ascribed values are less than <base> are
 * premitted.  If the value of <base> is 16, the characters 0x or 0X may
 * optionally precede the sequence of letters and digits, following the sign
 * if present.
 *
 * The subject sequence is defined as the longest initial subsequence of the
 * input string, starting with the first non-white-space character, that is
 * of the expected form.  The subject sequence contains no characters if the
 * input string is empty or consists entirely of white space, or if the first
 * non-white-space character is other than a sign or a permissible letter or
 * digit.
 *
 * If the subject sequence has the expected form and the value of <base> is
 * zero, the sequence of characters starting with the first digit is
 * interpreted as an integer constant.  If the subject sequence has the
 * expected form and the value of <base> is between 2 and 36, it is used as
 * the <base> for conversion, ascribing to each letter its value as given
 * above.  If the subject sequence begins with a minus sign, the value
 * resulting from the conversion is negated.  A pointer to the final string
 * is stored in the object pointed to by <endptr>, provided that <endptr> is
 * not a null pointer.
 *
 * In other than the "C" locale, additional implementation-defined subject
 * sequence forms may be accepted.  VxWorks supports only the "C" locale;
 * it assumes that the upper- and lower-case alphabets and digits are
 * each contiguous.
 *
 * If the subject sequence is empty or does not have the expected form, no
 * conversion is performed; the value of <nptr> is stored in the object
 * pointed to by <endptr>, provided that <endptr> is not a null pointer.
 *
 * RETURNS:
 * The converted value, if any.  If no conversion could be performed it
 * returns zero.  If the correct value is outside the range of representable
 * values, it returns ULONG_MAX, and stores the value of the macro ERANGE in
 * <errno>.
 *
 * NOTE: This version doesn't set errno.
 */

uint wrs_strtoul( const char *     nptr,	/* string to convert */
		  char **          endptr,	/* ptr to final string */
		  int              base	/* radix */)
{
	const char *s = nptr;
	uint       acc;
	int         c;
	uint       cutoff;
	int         neg = 0;
	int         any;
	int         cutlim;

	/*
	 * See strtol for comments as to the logic used.
	 */
	do 
        {
		c = *s++;
        } while (wrs_isspace (c));

	if (c == '-') 
        {
		neg = 1;
		c = *s++;
        } 
	else if (c == '+')
		c = *s++;

	if (((base == 0) || (base == 16)) &&
	    (c == '0') && 
	    ((*s == 'x') || (*s == 'X'))) 
        {
		c = s[1];
		s += 2;
		base = 16;
        }

	if (base == 0)
		base = (c == '0' ? 8 : 10);

	cutoff = (uint) ULONG_MAX / (uint) base;
	cutlim = (uint) ULONG_MAX % (uint) base;

	for (acc = 0, any = 0;; c = *s++) 
        {
		if (wrs_isdigit (c))
			c -= '0';
		else if (wrs_isalpha (c))
			c -= (wrs_isupper (c) ? 'A' - 10 : 'a' - 10);
		else
			break;

		if (c >= base)
			break;

		if ((any < 0) || (acc > cutoff) || ((acc == cutoff) && (c > cutlim)))
			any = -1;
		else 
		{
			any = 1;
			acc *= base;
			acc += c;
		}
        }

	if (any < 0) 
        {
		acc = ULONG_MAX;
        } 
	else if (neg)
		acc = - (int)acc;

	if (endptr != 0)
		*endptr = (any ? CHAR_FROM_CONST (s - 1) : CHAR_FROM_CONST (nptr));

	return (acc);
}

/*******************************************************************************
 *
 * inet_addr - convert a dot notation Internet address to a long integer
 *
 * This routine interprets an Internet address.  All the network library
 * routines call this routine to interpret entries in the data bases
 * which are expected to be an address.  The value returned is in network order.
 *
 * EXAMPLE
 * The following example returns 0x5a000002:
 * .CS
 *     inet_addr ("90.0.0.2");
 * .CE
 *
 * RETURNS: The Internet address, or ERROR.
 */

uint inet_addr(register char *inetString    /* string inet address */)
{
#define MAX_PARTS 4     /* Maximum number of parts in an IP address. */

	register uint val, base, n;
	register char c;
	uint parts[MAX_PARTS], *pp = parts;

	/* check for NULL pointer */

	if (inetString == (char *) NULL)
	{
		return (ERROR);
	}

 again:

	/* Collect number up to ``.''.  Values are specified as for C:
	 * 0x=hex, 0=octal, other=decimal. */

	val = 0; base = 10;
	if (*inetString == '0')
		base = 8, inetString++;
	if (*inetString == 'x' || *inetString == 'X')
		base = 16, inetString++;
	while ((c = *inetString))
	{
		if (wrs_isdigit ((int) c))
		{
			val = (val * base) + (c - '0');
			inetString++;
			continue;
		}
		if (base == 16 && wrs_isxdigit ((int) c))
		{
			val = (val << 4) + (c + 10 - (wrs_islower ((int) c) ? 'a' : 'A'));
			inetString++;
			continue;
		}
		break;
	} /* while */

	if (*inetString == '.')
	{
		/*
		 * Internet format:
		 *  a.b.c.d
		 *  a.b.c   (with c treated as 16-bits)
		 *  a.b (with b treated as 24 bits)
		 * Check each value for greater than 0xff for each part of the IP addr.
		 */

		if ((pp >= parts + (MAX_PARTS - 1)) || val > 0xff)
		{
			return (ERROR);
		}
		*pp++ = val, inetString++;
		goto again;
	}

	/* Check for trailing characters */

	if (*inetString && !wrs_isspace ((int) *inetString)) 
	{
		return (ERROR);
	}
	*pp++ = val;

	/* Concoct the address according to the number of parts specified. */

	n = pp - parts;
	switch ((int) n)
	{
	case 1:             /* a -- 32 bits */
		val = parts[0];
		break;

	case 2:             /* a.b -- 8.24 bits */
		if (val > 0xffffff)
		{
			return (ERROR);
		}

		val = (parts[0] << 24) | parts[1];
		break;

	case 3:             /* a.b.c -- 8.8.16 bits */
		if (val > 0xffff)
		{
			return (ERROR);
		}

		val = (parts[0] << 24) | (parts[1] << 16) | parts[2];
		break;

	case 4:             /* a.b.c.d -- 8.8.8.8 bits */
		if (val > 0xff)
		{
			return (ERROR);
		}

		val = (parts[0] << 24) | (parts[1] << 16) |
			(parts[2] << 8) | parts[3];
		break;

	default:
		return (ERROR);
	}

	return (wrs_htonl (val));
}

/*******************************************************************************
 *
 * wrs_strpbrk - find the first occurrence in a string of a character from a 
 *               given set (ANSI)
 *
 * This routine locates the first occurrence in string <s1> of any character
 * from string <s2>.
 *
 * INCLUDE FILES: string.h
 *
 * RETURNS:
 * A pointer to the character found in <s1>, or
 * NULL if no character from <s2> occurs in <s1>.
 *
 * SEE ALSO: strcspn()
 */

char * wrs_strpbrk(const char * s1,       /* string to search */
		   const char * s2        /* set of characters to look for in <s1> */)
{
	char *scanp;
	int   c;
	int   sc;

	while ((c = *s1++) != 0)            /* wait until end of string */
	{
		/* loop, searching for character */

		for (scanp = CHAR_FROM_CONST(s2); (sc = *scanp++) != 0;)
		{
			if (sc == c)
				return (CHAR_FROM_CONST(s1 - 1));
		}
	}

	return (NULL);
}
    
/******************************************************************************
 *
 * wrs_strspn - return the string length up to the first character not in a 
 *              given set (ANSI)
 *
 * This routine computes the length of the maximum initial segment of
 * string <s> that consists entirely of characters from the string <sep>.
 *
 * INCLUDE FILES: string.h
 *
 * RETURNS:
 * The length of the string segment.
 *
 * SEE ALSO: strcspn()
 */
 
size_t wrs_strspn(const char * s,             /* string to search */
		  const char * sep            /* set of characters to look for in <s> */)
{
	const char *save;
	const char *p;
	char c1;
	char c2;

	for (save = s + 1; (c1 = *s++) != EOS; )
		for (p = sep; (c2 = *p++) != c1; )
		{
			if (c2 == EOS)
				return (s - save);
		}

	return (s - save);
}

/*****************************************************************************
 *
 * wrs_strtok_r - break down a string into tokens (reentrant) (POSIX)
 *
 * This routine considers the null-terminated string <string> as a sequence
 * of zero or more text tokens separated by spans of one or more characters
 * from the separator string <separators>.  The argument <ppLast> points to a
 * user-provided pointer which in turn points to the position within <string>
 * at which scanning should begin.
 *
 * In the first call to this routine, <string> points to a null-terminated
 * string; <separators> points to a null-terminated string of separator
 * characters; and <ppLast> points to a NULL pointer.  The function returns a
 * pointer to the first character of the first token, writes a null character
 * into <string> immediately following the returned token, and updates the
 * pointer to which <ppLast> points so that it points to the first character
 * following the null written into <string>.  (Note that because the
 * separator character is overwritten by a null character, the input string
 * is modified as a result of this call.)
 *
 * In subsequent calls <string> must be a NULL pointer and <ppLast> must
 * be unchanged so that subsequent calls will move through the string <string>,
 * returning successive tokens until no tokens remain.  The separator
 * string <separators> may be different from call to call.  When no token
 * remains in <string>, a NULL pointer is returned.
 *
 * INCLUDE FILES: string.h
 * 
 * RETURNS
 * A pointer to the first character of a token,
 * or a NULL pointer if there is no token.
 *
 * SEE ALSO: strtok()
 */

char * wrs_strtok_r(char *       string,    /* string to break into tokens */
		    const char * separators,    /* the separators */
		    char **      ppLast     /* pointer to serve as string index */)
{
	if ((string == NULL) && ((string = *ppLast) == NULL))
		return (NULL);

	if (*(string += wrs_strspn (string, separators)) == EOS)
		return (*ppLast = NULL);

	if ((*ppLast = wrs_strpbrk (string, separators)) != NULL)
		*(*ppLast)++ = EOS;

	return (string);
}

#define SW_TYPE_IRTOS 0x11
#define SW_DOWNLOAD_FLAGS_VERIFIED 0x02
#define SW_DOWNLOAD_FLAGS_PERMANENT 0x01

static int ioctl_swdl(unsigned long arg)
{  
	PISCSI_FLASH_UPDATE_PACKET pFUP = NULL;
	struct i2o_controller *c = NULL;
	int  adapterNum = -1;
	u32  msg[9];
	u32  status;
	const u32 flags = SW_DOWNLOAD_FLAGS_PERMANENT | SW_DOWNLOAD_FLAGS_VERIFIED;
	const u32 type = SW_TYPE_IRTOS;

	// Allocate a flash update structure
	pFUP = (PISCSI_FLASH_UPDATE_PACKET)
		kmalloc(sizeof(ISCSI_FLASH_UPDATE_PACKET), GFP_KERNEL);

	if (pFUP == NULL)
	{
		printk(KERN_ERR "is1000: kmalloc failed (1)\n");
		return FALSE;
	}

#ifdef IOCTLDEBUG
	printk(KERN_INFO "\nis1000: ISCSI_IOCTL_UPDATE_FLASH:\n");
#endif

	// Copy the structure into kernel space
	if (copy_from_user(pFUP, (char *)arg, sizeof(ISCSI_FLASH_UPDATE_PACKET))) 
	{
		printk(KERN_ERR "is1000: Cannot copy user Ioctl args\n");
		goto outFailSW;
	}

#ifdef IOCTLDEBUG
	printk(KERN_INFO "is1000: Fragment %d of %d (len = %d, total = %d)\n",
	       pFUP->packet.frag_number, pFUP->packet.max_frag, 
	       pFUP->packet.frag_size, pFUP->packet.total_size);

	// Quick sanity check
	if (pFUP->packet.max_frag > 255)
	{
		printk(KERN_ERR "is1000: invalid max_frag argument\n");
		goto outFailSW;
	}
#endif

	// Find the adapter referenced by pFUP->MACAddr
	if (!(c = iscsi_find_adapter(&adapterNum, &pFUP->MACAddr)))
	{
		goto outFailSW;
	}

#ifdef IOCTLDEBUG
	printk(KERN_INFO "is1000: _DIAGNOSTICS adapter %d \n", adapterNum);
#endif

	// Set up the I2O message
	msg[0] = NINE_WORD_MSG_SIZE | SGL_OFFSET_7;
	msg[1] = I2O_CMD_SW_DOWNLOAD<<24 | HOST_TID<<12 | ADAPTER_TID;
	msg[2] = 0; // MNH (u32)cfg_handler.context;
	msg[3] = 0;
	msg[4] = (flags << 24) |
		(type << 16) | 
		(pFUP->packet.max_frag << 8) |
		pFUP->packet.frag_number;
	msg[5] = pFUP->packet.total_size;
	msg[6] = pFUP->packet.software_id;
	msg[7] = (0xD0000000 | pFUP->packet.frag_size);
	msg[8] = virt_to_bus(pFUP->packet.buffer);

#ifdef IOCTLDEBUG
	printk(KERN_INFO "is1000: msg[0] = %x\n", msg[0]);
	printk(KERN_INFO "is1000: msg[1] = %x\n", msg[1]);
	printk(KERN_INFO "is1000: msg[2] = %x\n", msg[2]);
	printk(KERN_INFO "is1000: msg[3] = %x\n", msg[3]);
	printk(KERN_INFO "is1000: msg[4] = %x\n", msg[4]);
	printk(KERN_INFO "is1000: msg[5] = %x\n", msg[5]);
	printk(KERN_INFO "is1000: msg[6] = %x\n", msg[6]);
	printk(KERN_INFO "is1000: msg[7] = %x\n", msg[7]);
	printk(KERN_INFO "is1000: msg[8] = %x\n", msg[8]);
#endif

	status = i2o_post_wait_mem(c, msg, sizeof(msg), 120, pFUP, NULL);
	if (status == -ETIMEDOUT)
	{
		// it fails if you try and send frags out of order
		// and for some yet unknown reasons too
		printk(KERN_INFO "is1000: flash update timed out\n");
		return FALSE;
	}
	else if (status != I2O_POST_WAIT_OK)
	{
		printk(KERN_INFO "is1000: flash update failed, DetailedStatus = %d\n", status);
		return FALSE;
	}

	kfree(pFUP);
	i2o_unlock_controller(c);
	return TRUE;

 outFailSW:
	kfree(pFUP);
	if (c)
		i2o_unlock_controller(c);
	return FALSE;
}

